/***************************************************************************
                          Link.h  -  description
                             -------------------
    begin                : ven jun 27 2003
    copyright            : (C) 2003 by Michel Dubois, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LINK_H
#define LINK_H
#include "Leg.h"
#include "SimLink.h"
#include "SimLinkedLeg.h"
#include "Molecule.h"

//#include "Edge.h"
#ifndef LEGLINK
class SimLink;
#endif

namespace API {
  class Edge;
  class Leg;
  class Molecule;
/**
* Classe definissant une connexion
*
* Une connexion est une paire de pattes connectees, chaque patte appartenant a deux atomes d'une meme molecule.
*
*@author Michel Dubois, Yann Le Guyadec
*@version du 27/05/03 (27 juin 2003)
*@since 0.1
*/

  class Link 
  {
    class Util;
    friend class Util;

    public:
/**
* Classe utilitaire completant la definition d'une connexion dans le simulateur
*
*@author Michel Dubois, Yann Le Guyadec
*@version du 27/05/03 (27 juin 2003)
*@since 0.2
*/
      class Util {
        friend class Link;
        private:
          #ifndef LEGLINK
            SimLink * slink;
          #endif
          #ifdef LEGLINK
            SimLinkedLeg * slink;
          #endif

          Link * link;
          Util(Link * link);
          ~Util();
        public:

/**
	* affiche l'etat de la connexion courante
	*/
          void print() const;
          #ifndef LEGLINK
/**
	*
	* retourne l'instance de la representation interne a la simulation de la connexion
	* @return la reference de la connexion simulee
	*/

            SimLink * getSimulatedObject() const;
          #endif
          #ifdef LEGLINK
            SimLinkedLeg * getSimulatedObject() const;
          #endif
       };

/**
	* Reference de l'instance de la classe specifique a la simulation d'une connexion
	*
	*/
      Util * util;

    private:
      Leg * leg1;
      Leg * leg2;
      Molecule * molecule;

      public:
/**
	* Retourne la reference de la molecule de la connexion courante
	*
	* @return La reference de l'instance de la molecule qui possede la connexion courante
  	*/
	    Molecule * getMolecule() const;


/**
	* retourne l'instance de la patte numero 1  de la connexion
	* @return la reference de la patte numero 1 de la connexion courante
	*/      
      Leg * getLeg1() const;

/**
	* retourne l'instance de la patte numero 2  de la connexion
	* @return la reference de la patte numero 2 de la connexion courante
	*/
      Leg * getLeg2() const;

/**
	* retourne si une patte fait partie de la connexion
	* @param aLeg reference de la patte
  *
  * @return l'appartenance d'une patte a la connexion
	*/
      bool isUsing(Leg * aLeg) const;

/**
	* retourne l'instance de l'autre patte de la connexion
	* @param aLeg reference de la patte
  *
  * @return la reference de l'autre patte de la connexion courante
	*/
      Leg * getOtherLeg(Leg * aLeg) const;

/**
	* intervertit les deux pattes de la connexion
	*/
      void swap();
  
    protected:

/**
	* constructeur d'une connexion
	*
	* @param aLeg reference de la premiere patte
	* @param anotherLeg reference de la deuxieme patte
  */
      Link(Leg * aLeg, Leg * anotherLeg);

/**
	* destructeur d'une connexion
	*
	*/
      ~Link();

/**
	* Affecte la connexion courante a une molecule
	* @param aMolecule Reference de la molecule
	*/
      void setMolecule(Molecule * aMolecule);
      
  friend class Leg;  
//  friend class Edge;
  };
}
#endif
