/***************************************************************************
                          SimLeg.h  -  description
                             -------------------
    begin                : mer jun 25 2003
    copyright            : (C) 2003 by Yann Le Guyadec, Michel DUBOIS
    email                : Yann.Le-Guyadec@univ-ubs.fr, Michel.Dubois@univ-ubs.fr 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SIMLEG_H
#define SIMLEG_H

#include "RigidBody.h"
#include "RigidBodyEntity.h"
#include "SimServoMotor.h"
#include "Matrix22.h"
#include "Vector2.h"
#include "Vector3.h"
#include "SphereGeometry.h"
#include "RigidBodyCollider.h"
#include "CylinderGeometry.h"
#include "LegGeometry.h"
#include "SimAtom.h"
//#include "RayGeometry.h"
#include "SimLinkedLeg.h"

/**
  *@author Yann Le Guyadec, Michel DUBOIS
  */

#define CAPPEDCYLINDER 0
#define BOX            1
#define FLATCYLINDER   2
#define COMPLEX        3

class SimAtom;
class SimLinkedLeg;

class SimLeg : public RigidBodyEntity  {
  friend class SimLink;
#ifdef LEGLINK
  friend class SimLinkedLeg;
#endif
  friend class SimServoMotor;
  
  protected:
    int ID;
    SimAtom * simatom;
    SimRigidBody * sphere;
    SphereGeometry * sphereGeometry;
    RigidBodyCollider * sphereCollider;
    SimRigidBody * body;
    Geometry * bodyGeometry;    
    RigidBodyCollider * bodyCollider;
    SimServoMotor * servo[2];
#ifdef LEGLINK
    SimLinkedLeg * linkedLeg;
#endif
#ifndef LEGLINK
    SimLink * slink;
#endif

  public:
    SimLeg(SimAtom * atom, int aLegID);
    ~SimLeg();
    void handleEvent (Event * event);
    unsigned int getSimServoMotorCount() const;
    SimServoMotor * getSimServoMotor(unsigned int aSimServoMotorID) const;
    Vector3 getVector(int Geometry=0) const;
    Vector3 getAbsPosSphere() const;
    Vector3 getAbsPosBody() const;
    Vector3 getAbsPosEnd() const;
    Vector3 getAbsPosConnexionDevice(unsigned int device) const;
    bool isConnectable(SimLeg * aSimLeg) const;
    Vector3 getAxis(unsigned int index,int Geometry=0) const;
    Real getRotationAngle(int Geometry=0) const;
    Vector3 getColor() const;
    void link(SimLeg * aSimLeg);
    void unlink();
    bool isLinked();
    virtual void print();
    static std::string legCollision;    
};

#endif
