/***************************************************************************
                          SimServoMotor.h  -  description
                             -------------------
    begin                : mar jui 15 2003
    copyright            : (C) 2003 by Yann Le Guyadec, Michel DUBOIS
    email                : Yann.Le-Guyadec@univ-ubs.fr, Michel.Dubois@univ-ubs.fr 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SIMSERVOMOTOR_H
#define SIMSERVOMOTOR_H
#include "nommgr.h"
#include <ode/ode.h>
#include "mmgr.h"
#include "HingeJoint.h"
#include "AMotorJoint.h"
#include "Entity.h"
#include "Event.h"
#include "SimLeg.h"

/**
  *@author Yann Le Guyadec, Michel DUBOIS
  */

class SimServoMotor : public Entity
{
  friend class SimLeg;
  friend class SimLinkedLeg;
  
  public: 
    SimServoMotor();
    SimServoMotor(HingeJoint * aHingeJoint, AMotorJoint * anAMotor);
//    SimServoMotor(HingeJoint * aHingeJoint, AMotor * anAMotor);
//    SimServoMotor(OldHingeJoint * aHingeJoint, AMotor * anAMotor);

    SimServoMotor(SimRigidBody * body1, SimRigidBody * body2);
    ~SimServoMotor();
//    OldHingeJoint * getHingeJoint() const;
    HingeJoint * getHingeJoint() const;

    AMotorJoint  * getAMotorJoint() const;
//    AMotor  * getAMotor() const;

    double getParam(int odeparameter) const;
    void setParam(int odeparameter, double value);
    /* commenter le codage */
    void setAxis(Vector3 anAxis);
    Real getAngle();
    void handleEvent (Event* event);
    int getMinPosition() const;
    int getMaxPosition() const;
    double getLowStopAngle() const;
    double getHighStopAngle() const;
    int Angle2Position(double anAngle) const;
    double Position2Angle(int aPosition) const;
    int getHalfPosition() const;
    SimServoMotor * getCoupledSimServoMotor() const;
    double getTargetVelocity() const;
    double getCurrentVelocity() const;
    virtual void print();
        
  protected:
    SimLeg *      sleg;
    unsigned int  ID;
    HingeJoint *  hinge;

//    OldHingeJoint *  hinge;
    AMotorJoint * amotor;
//    AMotor * amotor;    
    int           direction; // 0 or 1
    double        targetSpeed;
    SimServoMotor * coupledSSMotor;
    void setCoupledServoMotor(SimServoMotor * aCoupledSimServoMotor);
    void setHingeJoint(HingeJoint * aHingeJoint);
//    void setHingeJoint(OldHingeJoint * aHingeJoint);

    void setAMotorJoint(AMotorJoint * anAMotor);
//    void setAMotorJoint(AMotor * anAMotor);
    void checkConfig();
    static double servoMotorLowStopAngle;
    static double servoMotorHighStopAngle;
    static int servoMotorMinPos;
    static int servoMotorMaxPos;        
    static double servoMotorRatioPos2Angle;
    static double servoMotorRatioAngle2Pos;
    static double servoMotorMaxAmplitudePos;
    static double servoMotorMaxAmplitudeAngle;


};

#endif
