/***************************************************************************
                          SphericPosition.h  -  description
                             -------------------
    begin                : dim jui 8 2003
    copyright            : (C) 2003 by Michel Dubois, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


// Modle statique
/*
  * changes :
  *  - 08/06/03 MD Creation !
  */


#ifndef __SPHERICPOSITION__
#define __SPHERICPOSITION__

namespace API {
/**
* Classe definissant une position spherique
*
*@author Michel Dubois, Yann Le Guyadec
*@version du 08/06/03 (08 juin 2003)
*@since 0.1
*/

class SphericPosition
{
    class Util;
    friend class Util;

    public:
/**
* Classe utilitaire completant la definition d'une position spherique dans le simulateur
*
*@author Michel Dubois, Yann Le Guyadec
*@version du 08/06/03 (08 juin 2003)
*@since 0.2
*/
    
      class Util {
        friend class SphericPosition;
        private:
          SphericPosition * sp;
          Util(SphericPosition * sp);
          ~Util();
        public:
/**
	* Affiche l'etat de la position spherique courante
	*/
          void          print() const;
          
       };

/**
	*
	* Retourne l'instance de la representation interne a la simulation de la position spherique
	* @return la reference de la position spherique simulee
	*/
      Util * util;
 
/**
	* Constructeur de la classe SphericPosition
  *
	* Initialise tous les attributs de la classe SphericPosition  a 0
	*/
  SphericPosition();

/**
	* Constructeur de la classe SphericPosition
  *
	* Initialise tous les attributs de la classe SphericPosition avec les parametres
	* @param alpha valeur alpha de la position spherique
	* @param beta valeur beta de la position spherique
	*
	*/
  SphericPosition(double alpha, double beta);

/**
	* Renvoie l'angle alpha
	* @return angle alpha
	*
	*/
  double getAlpha();

/**
	*
	* Renvoie l'angle beta
	* @return angle beta
	*
	*/
  double getBeta();

/**
	* Modifie l'angle de alpha
	* @param aRadius valeur alpha de la position spherique
	*
	*/
	void setAlpha(double aRadius);

/**
	*
	* Modifie l'angle de beta
	* @param aRadius valeur beta de la position spherique
	*
	*/
  void setBeta(double aRadius);
 // void setPosition(double alpha, double beta);

 private:
  double alpha;
  double beta;
  
};// END CLASS DEFINITION SphericPosition
}
#endif // __SPHERICPOSITION__
