/***************************************************************************
                          Atom.cpp  -  description
                             -------------------
    begin                : mer jun 25 2003
    copyright            : (C) 2003 by Michel DUBOIS, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mmgr.h"
 
#include "Atom.h"
#include "Event.h"
#include "Vector3.h"


using namespace API;


API::Atom::Atom(){

  static int maxID=-1;
  this->setID(++maxID);
  this->util=new Util(this,new SimAtom());
  this->util->simatom->setID(this->ID);
  this->molecule=new Molecule();
  for (int i=0;i<getLegCount();i++) {
 
    leg[i]=new Leg(this, i);
    leg[i]->util->simleg=util->simatom->getSimLeg(i);
//    leg[i]->simleg=util->simatom->getSimLeg(i);
    leg[i]->lservo[0]->util->simservo=leg[i]->util->simleg->getSimServoMotor(0);
    leg[i]->lservo[1]->util->simservo=leg[i]->util->simleg->getSimServoMotor(1);    
 
  }

  this->molecule->addAtom(this);

}

API::Atom::~Atom(){

  for (int i=0;i<getLegCount();i++) {
    delete leg[i];
  }
  delete this->util->simatom;
  delete this->util;
  
  if (molecule)
	{
		if (molecule->getAtomCount () <= 1)
		{
			delete molecule;
		}
		else
		{
			molecule->removeAtom (this);
		}
	}
}

unsigned int API::Atom::getID() const
{
	return this->ID;
}

//unsigned int API::Atom::getDegree() const
unsigned int API::Atom::getConnectedLegCount() const
 
{
  int i;
  int degree=0;
  for (i=0;i<6;i++)
    if (this->leg[i]->isLinked())
      degree++;
  return degree;
}

Leg * API::Atom::getLeg(unsigned int aLegID) const
{
  if (aLegID<getLegCount())
    return leg[aLegID];
  else
    return 0;
}


Leg * API::Atom::getMappedLeg(unsigned int aLegID) const
{
  //return leg[aLegID-1];

}


Molecule * API::Atom::getMolecule()  const
{
return this->molecule;
}

/*
API::Atom *  API::Atom::getNeighbor(unsigned int neighbourIndex) const
{
  std::vector<Atom*> ret;
  //= new std::vector<Atom*>;
  for(unsigned int i=0; i<6; i++)
    if (leg[i]->isLinked())
    {
      bool found=false;
      for (unsigned int j=0;j<ret.size() && found==false;j++)
        if (leg[i]->getLinkedLeg()->getAtom()==ret[j])
          found=true;
      if (found==false)
        ret.push_back (leg[i]->getLinkedLeg()->getAtom());
    }
  return ret;
  
}
*/
unsigned int API::Atom::getConnectedAtomCount() const // 0..6
{
/*
  unsigned int ret=0;
  for(unsigned int i=0; i<6; i++)
    if (leg[i]->isLinked())
    {
      bool found=false;
      for (unsigned int j=0;j<ret.size() && found==false;j++)
        if (leg[i]->getLinkedLeg()->getAtom()==ret[j])
          found=true;
      if (found==false)
        ret.push_back (leg[i]->getLinkedLeg()->getAtom());
    }
  return ret;
*/
}

/*Leg * API::Atom::getLinkedLeg(unsigned int neighborIndex, unsigned int legIndex) const
{

}
*/
/*
unsigned int API::Atom::getLinkedLegCount(unsigned int neighborIndex) const
{

}
*/
std::vector<unsigned int> API::Atom::getMap() const
{
	return std::vector<unsigned int>();
}


Accelerometer * API::Atom::getAccelerometer(unsigned int anAccelerometerID) const
{
  if (anAccelerometerID<getAccelerometerCount())
    return accel[anAccelerometerID];
  else
    return 0;
}

unsigned int API::Atom::getAccelerometerCount() const
{
  return 3;
}

Clock * API::Atom::getClock()  const
{

}

Girometer * API::Atom::getGirometer(unsigned int aGirometerID) const
{
  if (aGirometerID<getGirometerCount())
    return giro[aGirometerID];
  else
    return 0;
}

unsigned int API::Atom::getGirometerCount() const
{
  return 3;
}

void API::Atom::setMap(unsigned int face1, unsigned int face2)
{

}

void API::Atom::stop()
{

}

void API::Atom::setID(unsigned int anAtomID)
{
  this->ID=anAtomID;
}

void API::Atom::setMolecule(Molecule * aMolecule)
{
  this->molecule=aMolecule;
}

API::Atom::Util::Util(Atom * anAtom, SimAtom * aSimAtom)
{
  this->atom=anAtom;
  this->simatom=aSimAtom;
}

API::Atom::Util::~Util()
{
}    
SimAtom * API::Atom::Util::getSimulatedObject () const
{
  return this->simatom;
}
/*
void Atom::setSimulatedObject( SimAtom * aSimulatedAtom)
{
  this->simatom=aSimulatedAtom;
}
*/

void API::Atom::Util::setCartesianPosition(CartesianPosition * aCartesianPosition)
{
  this->simatom->handleEvent(
    new PositionEvent ( Vector3( aCartesianPosition->getX(),
                                aCartesianPosition->getY(),
                                aCartesianPosition->getZ()
                                )
                      )
  );
  delete aCartesianPosition;
}

CartesianPosition API::Atom::Util::getCartesianPosition() const
{
    return CartesianPosition(this->simatom->getPosition());  
}

void API::Atom::Util::print() const
{
  printf ("Atom: ID=%i\n",this->atom->ID);
}

unsigned int API::Atom::getLegCount() const
{
  return this->util->simatom->getSimLegCount();
}

bool API::Atom::Util::getAttachToEnvMode() const
{
  return bool();
}

void API::Atom::Util::setAttachToEnvMode(bool onoff)
{

}

