/***************************************************************************
                          CartesianPosition.h  -  description
                             -------------------
    begin                : mer jun 08 2003
    copyright            : (C) 2003 by Michel Dubois, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
  * changes :
  *  - 08/06/03 MD Creation !
  */


#ifndef __CARTESIANPOSITION__
#define __CARTESIANPOSITION__
#include <Vector3.h>

namespace API {
/**
* Classe definissant une position dans le repere cartesien
*
*@author Michel Dubois, Yann Le Guyadec
*@version du 08/06/03 (08 juin 2003)
*@since 0.1
*/

  class CartesianPosition
  {
    class Util;
    friend class Util;

    public:
/**
* Classe utilitaire completant la definition d'une position dans le repere cartesien dans le simulateur
*
*@author Michel Dubois, Yann Le Guyadec
*@version du 25/09/03 (25 septembre 2003)
*@since 0.2
*/
    
      class Util {
        friend class CartesianPosition;
        private:
          CartesianPosition * cp;
          Util(CartesianPosition * cp);
          ~Util();
        public:
/**
	* affiche l'etat de la position courante dans le repere cartesien
	*/
        
          void print() const;
      };

/**
	* Reference de l'instance de la classe specifique a la simulation d'une position cartesienne
	*
	*/
      Util * util;

/**
	* constructeur d'une position cartesienne
	*
	*/            
      CartesianPosition();

/**
	* destructeur d'une position cartesienne
	*
	*/            
      ~CartesianPosition();

/**
	* constructeur d'une position cartesienne
	*
  *@param xCoordonate Coordonnee par rapport au premier axe
  *@param yCoordonate Coordonnee par rapport au deuxieme axe
  *@param zCoordonate Coordonnee par rapport au troisieme axe  
	*/            
      CartesianPosition(const double xCoordonate, const double yCoordonate, const double zCoordonate);

/**
	* constructeur d'une position cartesienne
	*
  *@param aVector Vecteur de coordonnees
	*/
      CartesianPosition(const Vector3& aVector);

/**
	*
	* Renvoie la position sur l'axe X
	* @return position sur l'axe X
	*/
      double getX() const;
/**
	*
	* Renvoie la position sur l'axe Y
	* @return position sur l'axe Y
	*/      
      double getY() const;
      
/**
	*
	* Renvoie la position sur l'axe Z
	* @return position sur l'axe Z
	*/
    	double getZ() const;

/**
	*
	* Modifie la position sur l'axe X
	* @param aCoordonate valeur x de la position cartesienne
	*/     
    	void setX(double aCoordonate);
/**
	*
	* Modifie la position sur l'axe Y
	* @param aCoordonate valeur y de la position cartesienne
	*/
     void setY(double aCoordonate);
/**
	*
	* Modifie la position sur l'axe Z
	* @param aCoordonate valeur z de la position cartesienne
	*/
     void setZ(double aCoordonate);

    private :
      double x,y,z;

    
  };// END CLASS DEFINITION CartesianPosition
}
#endif // __CARTESIANPOSITION__
