/***************************************************************************
                          Graph.cpp  -  description
                             -------------------
    begin                : sam jui 26 2003
    copyright            : (C) 2003 by Michel DUBOIS, Yann Le Guyadec 
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "Graph.h"
#include "mmgr.h" 

Connection::Connection(Vertex * aVertex, Vertex * anotherVertex)
{
  this->vertex1=aVertex;
  this->vertex2=anotherVertex;
  this->graph=0;
}

Vertex * Connection::getVertex1() const
{
  return this->vertex1;
}

Vertex * Connection::getVertex2() const
{
  return this->vertex2;
}

Connection::~Connection(){
}

void Connection::print() const
{
  if (getGraph()) {
    for (int i=0;i<this->getGraph()->getConnectionCount();i++)
      if (this->getGraph()->getConnection(i)==this)
      {
        printf ("Connection[%i]\n",i);
        this->vertex1->print();
        this->vertex2->print(); 
        return;
      }
  }
  printf ("Connection[?]\n");
  this->vertex1->print();
  this->vertex2->print();
}

void Connection::setGraph(Graph * aGraph){
  this->graph=aGraph;
}

Graph * Connection::getGraph() const{
  return this->graph;
}

unsigned int Connection::getConnectionCount() const
{
  return 1;
}

Connection* Connection::getConnection (unsigned int index)
{
  Connection * ret=0;
  if (index==0)
    ret=this;
	return ret;
}

int Connection::getConnectionIndex (Connection * aConnection) const
{
  if (aConnection==this)
    return 0;
  return -1;
}


Edge::Edge(Vertex * aVertex, Vertex * anotherVertex) : Connection(aVertex,anotherVertex)
{
  connections.clear();
}

Edge::~Edge()
{

}

void Edge::swap()
{
  Vertex * tmp=this->vertex1;
  this->vertex1=vertex2;
  this->vertex2=tmp;
  for (unsigned int i=0; i < connections.size(); i++)
    connections[i]->swap();  
}

unsigned int Edge::getConnectionCount() const
{
  return this->connections.size();
}

Connection* Edge::getConnection (unsigned int index)
{
	Connection* connection = 0;

	if (0 <= index && index < connections.size ())
	{
		connection = connections[index];
	}

	return connection;
}

void Edge::addConnection(Connection * aConnection)
{
// pas de test de redondance !
//  if (getConnectionIndex(aConnection)==-1)
    connections.push_back(aConnection);
}

int Edge::getConnectionIndex (Connection * aConnection) const
{
  for (unsigned int i = 0; i < connections.size (); i++)
	{
		if (connections[i] == aConnection)
		{
			return i;
		}
	}
	return -1;
}

void Edge::removeConnection(Connection * aConnection)
{
  int index=this->getConnectionIndex (aConnection);
  if (index!=-1) {
    for ( int i= index; i + 1 < connections.size(); ++i)
      connections[i] = connections[i + 1];
    connections.pop_back();
  }
}

void Edge::print() const
{
  if (getGraph()) {
    for (int i=0;i<this->getGraph()->getEdgeCount();i++)
      if (this->getGraph()->getEdge(i)==this)
      {
        printf ("Edge[%i] Connections: size=%i\n",i,connections.size());
        for (unsigned int i=0; i < connections.size(); i++)
          connections[i]->print();
        return;
      }
  }
  printf ("Edge[?] Connections: size=%i\n",connections.size());
  for (unsigned int i=0; i < connections.size(); i++)
    connections[i]->print();
}


Graph * Vertex::getGraph() const
{
  return this->graph;
}

void Vertex::setGraph(Graph * aGraph){
  this->graph=aGraph;
}

Vertex::Vertex(){
  graph=0;
}

Vertex::~Vertex(){
}

Graph::Graph(){
  vertexCount=0;
  connectionCount=0;
  edgeCount=0;
  connectedComponentCount=0;
}

Graph::~Graph()
{
  for (int i=0; i<getEdgeCount();i++)
    delete getEdge(i);
}

Vertex* Graph::getVertex (unsigned int vertexIndex) const
{
	Vertex * vertex = 0;
	if (0 <= vertexIndex && vertexIndex < vertices.size ())
	{
		vertex = vertices[vertexIndex];
	}
	return vertex;
}



unsigned int Graph::getVertexCount() const
{
	return this->vertexCount;

}

int Graph::getVertexIndex (Vertex * aVertex) const
{
  for (unsigned int i = 0; i < vertices.size (); i++)
	{
		if (vertices[i] == aVertex)
		{
			return i;
		}
	}
	return -1;
}

unsigned int Graph::getConnectionCount() const
{
  return connectionCount;
}

unsigned int Graph::getConnectionCount(unsigned int index1,unsigned int index2) const
{
  if (!(adj[index1][index2]))
    return 0;
  return adj[index1][index2]->getConnectionCount();
}

Connection * Graph::getConnection(unsigned int index1,unsigned int index2) const
{
  if (!(adj[index1][index2]))
    return 0;
  return adj[index1][index2]->getConnection(0);
}

Connection * Graph::getConnection(unsigned int index1,unsigned int index2,unsigned int index3) const
{
  if (!(adj[index1][index2]))
    return 0;
  return adj[index1][index2]->getConnection(index3);
}

Connection * Graph::getConnection(unsigned int index) const
{
  unsigned int k=0;
  if (!(edgeCount))
  {
    for ( unsigned int i=1; i < vertices.size(); i++)
      for ( unsigned int j=0; j < i; j++)
        if (adj[i][j])
          if (k==index)
            return adj[i][j];
              else
            k++;
    return 0;
  }
  for ( unsigned int i=1; i < vertices.size(); i++)
    for ( unsigned int j=0; j < i; j++)
      if (adj[i][j])
        for ( unsigned int l=0; l < adj[i][j]->getConnectionCount(); l++)
          if (k==index)
            return adj[i][j]->getConnection(l);
          else
            k++;
  return 0;  
}

int Graph::getConnectionIndex (Connection * aConnection) const
{
  if (!(aConnection))
    return -1;    
  unsigned int k=0;
  if (!(edgeCount))
  {
    for ( unsigned int i=1; i < vertices.size(); i++)
      for ( unsigned int j=0; j < i; j++)
        if (adj[i][j]==aConnection)
          return k;
        else
          k++;
	  return -1;
  }
  for ( unsigned int i=1; i < vertices.size(); i++)
    for ( unsigned int j=0; j < i; j++)
      if (adj[i][j])
        for ( unsigned int l=0; l < adj[i][j]->getConnectionCount(); l++)
          if (adj[i][j]->getConnection(l)==aConnection)
            return k;
          else
            k++;
  return -1;  
}

void Graph::printVertices() const
{
  printf ("Vertices: size=%i \n",vertices.size());
  for (unsigned int i=0; i < vertices.size(); i++)
    vertices[i]->print();
}

void Graph::printConnections() const
{
  printf ("Connections: size=%i \n",connectionCount);
  for (unsigned int i=0; i < connectionCount; i++)
    this->getConnection(i)->print();
}

void Graph::printEdges() const
{
  printf ("Edges: size=%i \n",edgeCount);
  for (unsigned int i=0; i < edgeCount; i++)
    this->getEdge(i)->print();
}


void Graph::printAdjacencyMatrix() const
{
  printf("Adjacency Matrix\n");
  for (unsigned int i=0; i<vertices.size();i++)
    printf("-");
  printf("\n");
  for (unsigned int i=0; i<vertices.size();i++)
  {
    for (unsigned int j=0; j<vertices.size();j++)
      if (adj[i][j])
      {
        Edge * edge=dynamic_cast<Edge *> (adj[i][j]);
        if (edge)
          printf("E");
        else
          printf("C");
      }
      else
        printf(" ");
    printf("\n");
  }
}

void Graph::printConnectedComponentID() const
{
  printf ("Vertices: size=%i \n",vertices.size());
  for (unsigned int i=0; i < vertices.size(); i++)
    printf ("[%i] CCID=%i\n",i,this->connectedComponentID[i]);
  for ( unsigned int i=1; i < vertices.size(); i++)
    for ( unsigned int j=0; j < i; j++)
      if (!(isConnected(i,j)))
        printf("%i et %i non connectes !\n",i,j);  
}

void Graph::print() 
{
  printf ("Graph:\n");
  this->printVertices();
  this->printEdges();
  this->printConnections();
  this->printAdjacencyMatrix();
  this->computeConnectedComponent();
  this->printConnectedComponentID();
}

void Graph::addVertex (Vertex * aVertex)
{
  if (aVertex)
	{
    if (this->getVertexIndex(aVertex)==-1) {
      vertexCount++;
		  aVertex->setGraph (this);
		  vertices.push_back (aVertex);
      //agrandissement de la matrice d'adjascence
      for (unsigned int i=0; i<vertexCount-1; i++)
        adj[i].push_back(0);
      std::vector<Connection *> tmp(vertexCount,0);
      adj.push_back(tmp);
    }
	}
}

void Graph::removeVertex (Vertex * aVertex)
{
	if (aVertex)
	{
    if (this->getVertexIndex(aVertex)!=-1) {

      int index=(unsigned int) this->getVertexIndex (aVertex);
      // Move elements after the index up one position
      // And remove spare element at the end.
      for (unsigned int j=0;j<adj.size();j++)
      {
        for (unsigned int i=index; i + 1 < adj[j].size(); ++i)
          adj[j][i] = adj[j][i + 1];
        adj[j].pop_back();
      }
      for ( unsigned int i=index; i + 1 < adj.size(); ++i)
        adj[i] = adj[i + 1];
      adj.pop_back();
      for ( unsigned int i=index; i + 1 < vertices.size(); ++i)
        vertices[i] = vertices[i + 1];
      vertices.pop_back();
      unsigned int oldEdgeCount=edgeCount;
      edgeCount=0;
      connectionCount=0;
      if (oldEdgeCount)
      {
        for ( unsigned int i=1; i < vertices.size(); i++)
          for ( unsigned int j=0; j < i; j++)
            if (adj[i][j])
            {
              //adj[i][j]->print();
              Edge * edge = dynamic_cast<Edge *> (adj[i][j]);
	            if (edge)
	            {
                edgeCount++;
                connectionCount+=edge->getConnectionCount();
              }
              else
                connectionCount++;
            }
      }
      else
        for ( unsigned int i=1; i < vertices.size(); i++)
          for ( unsigned int j=0; j < i; j++)
            if (adj[i][j])
              connectionCount++;
      vertexCount=vertices.size();
      //aVertex->setGraph (0);
    }
	}

}

void Graph::insertConnection (Connection* connection)
{
  Graph * otherGraph=connection->getVertex1()->getGraph();
  if (this==otherGraph)
    otherGraph=connection->getVertex2()->getGraph();
  if (this!=otherGraph)
  {
    /*
    printf("**Fusion**\n");
    printf("Graph predateur\n");
    this->print();
    printf("Graph victime\n");
    otherGraph->print();
    */    
    /************************
     * Fusion de deux graph
     ***********************/
    std::vector<Vertex *> atraiter;
    // Ajout des vertices et agrandissement de la matrice d'adjascence
    for (unsigned int i=0; i<otherGraph->vertexCount;i++)
    {
      this->addVertex(otherGraph->vertices[i]);
      atraiter.push_back(otherGraph->vertices[i]);
    }
    // Mise a jour de la matrice d'adjascence
    for (unsigned int i=0; i<otherGraph->connectionCount;i++)
    {
      this->insertConnection(otherGraph->getConnection(i));
    }
    // Destruction de la molecule absorbee
    /*for (unsigned int i=0; i<atraiter.size();i++)
    {
      otherGraph->removeVertex(atraiter[i]);
    }*/

    delete otherGraph;
  }

  int v1=(unsigned int) this->getVertexIndex(connection->getVertex1());
  int v2=(unsigned int) this->getVertexIndex(connection->getVertex2());

  if (adj[v1][v2]==0)
  {
    adj[v1][v2]=connection;
    adj[v2][v1]=connection;
  }
  else
  {
    Edge * edge = dynamic_cast<Edge *> (adj[v1][v2]);
	  if (edge)
	  {
      if ((edge->getConnectionIndex(connection))!=-1) return;
      edge->addConnection(connection);
    }
    else
    {
      if (adj[v1][v2]==connection) return;
      edgeCount++;
      edge=new Edge(connection->getVertex1(),connection->getVertex2());
      edge->setGraph(this); //suspect
      edge->addConnection(adj[v1][v2]);
      edge->addConnection(connection);
      adj[v1][v2]=edge;
      adj[v2][v1]=edge;
     // edge->print();
    }
  }
  connectionCount++;    
  connection->setGraph(this); //suspect
  //this->print();
}
  
bool Graph::hasMultipleConnections() const
{  return (edgeCount!=0);
}

void Graph::removeConnection (Connection*   connection)
{
  int v1=(unsigned int) this->getVertexIndex(connection->getVertex1());
  int v2=(unsigned int) this->getVertexIndex(connection->getVertex2());
  /*
  printf("***************************************** \n");
  Connection* tmp=connection;
  printf("Connection a supprimer :\n");
  tmp->print();
  printf("du Graph :\n");
  tmp->getGraph()->print();
  */
  if (v1==-1 || v2==-1)
  {
    printf("Attention: \n");
    Connection* tmp=connection;
    printf("Connection a supprimer :\n");
    tmp->print();
    return;
  }
  //on efface le lien de la matrice d'adjacence

  if (adj[v1][v2])
  {
    Edge * edge = dynamic_cast<Edge *> (adj[v1][v2]);
	  if (edge)
	  {
      edge->removeConnection(connection);
      connectionCount--;      
      if (edge->getConnectionCount()==1)
      {
        //edge->getConnection(0)->print();
        Connection * asauver=edge->getConnection(0);
        delete edge;
        adj[v1][v2]=asauver;
        adj[v2][v1]=asauver;
        //adj[v1][v2]->print();
        edgeCount--;
      }  
    }
    else
    {
      connectionCount--;
      adj[v1][v2]=0;
      adj[v2][v1]=0;
//    connection->setGraph(0);

    }
  }
  //on calcule le nombre de composants connectes
  this->computeConnectedComponent();
  //on veut garder dans le graphe le vertex de base (qui a fait la deconnection)
  this->setBaseConnectedComponent(this->getVertexIndex(connection->getVertex1()));
  std::vector<Vertex *> vertexAtraiter;
  std::vector<Connection *> connectionAtraiter;
  // Si des vertexes plus connectes avec celui ci
  while (connectedComponentCount>1)
  {

    vertexAtraiter.clear();
    connectionAtraiter.clear();

    // Sauvegarde des vertices et des liens
    for (unsigned int i=0; i<vertexCount;i++)
    {
      if (this->connectedComponentID[i]==connectedComponentCount-1)
      {
        vertexAtraiter.push_back(vertices[i]);
        Graph::adjIterator it=Graph::adjIterator(this,i);        
        for (int v=it.beg();!it.end();v=it.nxt())
        {
          Edge * edge = dynamic_cast<Edge *> (adj[i][v]);
          if (edge)
          {            
            for (int j=0; j<edge->getConnectionCount();j++)
              connectionAtraiter.push_back(edge->getConnection(j));
            delete edge;
          }
          else
            connectionAtraiter.push_back(adj[i][v]);
          adj[i][v]=0;
          adj[v][i]=0;          
        }
      }
    }

    Graph * newGraph=this->createGraph();
    for (unsigned int i=0; i<vertexAtraiter.size();i++)
    {
        this->removeVertex(vertexAtraiter[i]);
        newGraph->addVertex(vertexAtraiter[i]);
    }

    for (unsigned int i=0; i<connectionAtraiter.size();i++)
    {
        newGraph->insertConnection(connectionAtraiter[i]);
    }

    this->computeConnectedComponent();
    this->setBaseConnectedComponent(this->getVertexIndex(connection->getVertex1()));
  }
}

Graph::adjIterator::adjIterator(const Graph * G, int v) : G(G), i(-1), v(v)
{
  
}

int Graph::adjIterator::beg ()
{
  i=-1;
  return nxt();
}

int Graph::adjIterator::nxt ()
{
  for (i++; i<G->vertices.size(); i++)
  if ((G->adj[v][i]))
  {
    return i;
  }
  return -1;
}

bool Graph::adjIterator::end ()
{
  return i>=G->vertices.size();
}


void Graph::computeConnectedComponent()
{
  connectedComponentID.clear();
  connectedComponentID.assign(vertices.size(),-1);
  connectedComponentCount=0;
  for (unsigned int v=0; v<vertices.size();v++)
    if (connectedComponentID[v]==-1)
    {
      computeConnectedComponent(v);
      connectedComponentCount++;
    }  
}

void Graph::computeConnectedComponent(unsigned int vertexIndex)
{
  connectedComponentID[vertexIndex]=connectedComponentCount;
//  printf("CCC(%i): cCCount=%i\n",vertexIndex,connectedComponentCount);
  
  Graph::adjIterator it=Graph::adjIterator(this,vertexIndex);

  for (int v=it.beg();!it.end();v=it.nxt())
  {
//   printf("CCC(%i): v=%i CCID[v]=%i\n",vertexIndex,v,connectedComponentID[v]);
   
    if (connectedComponentID[v]==-1)
      computeConnectedComponent(v);
  }  
}

void Graph::setBaseConnectedComponent(unsigned int vertexIndex)
{
  int CCID=connectedComponentID[vertexIndex];
  if (CCID!=0)
  for (unsigned int v=0; v<vertices.size();v++)
  {
    if (connectedComponentID[v]==0)
      connectedComponentID[v]=CCID;
    else if (connectedComponentID[v]==CCID)
      connectedComponentID[v]=CCID;
  }  
}

unsigned int Graph::getConnectedComponentCount() const
{
  return connectedComponentCount;  
}

bool Graph::isConnected(unsigned int vertexIndex, unsigned int anotherVertexIndex) const
{
  return connectedComponentID[vertexIndex]==connectedComponentID[anotherVertexIndex];  
}

unsigned int Graph::getConnectedComponentCard(unsigned int index) const
{
  unsigned int k=0;
  for ( unsigned int i=0; i < vertices.size(); i++)
    if (connectedComponentID[i]==index)
      k++;
  return k;  
}

unsigned int Graph::getEdgeCount() const
{
  return edgeCount;  
}

Edge * Graph::getEdge(unsigned int index1,unsigned int index2) const
{
  return dynamic_cast<Edge *> (adj[index1][index2]);
}

Edge * Graph::getEdge(unsigned int index) const
{
  if (!(edgeCount))
    return 0;
  unsigned int k=0;
  for ( unsigned int i=1; i < vertices.size(); i++)
    for ( unsigned int j=0; j < i; j++)
    {
      Edge * edge=dynamic_cast<Edge *> (adj[i][j]);
      if (edge)
        if (k==index)
          return edge;
        else
          k++;
    }
  return 0;    
}

int Graph::getEdgeIndex (Edge * anEdge) const
{
  if (!(anEdge))
    return -1;
  if (!(edgeCount))
    return -1;
  unsigned int k=0;
  for ( unsigned int i=1; i < vertices.size(); i++)
    for ( unsigned int j=0; j < i; j++)
    {
      Edge * edge=dynamic_cast<Edge *> (adj[i][j]);
      if (edge)
          if (edge==anEdge)
            return k;
          else
            k++;
    }
  return -1;   
}

TestV::TestV()
{
  static int maxID=-1;
  this->ID=++maxID;
  testCs.clear();
  this->graph=new TestG();
  this->graph->addVertex(this);
}

TestV::~TestV()
{
  this->disconnect();
  this->graph->removeVertex(this);
  if (this->graph->getVertexCount()==0)
    delete this->graph;
}

void TestV::print()
{
  printf ("TestV ID=%i\n",this->ID);
}

void TestV::connect(TestV * aTestV)
{
  TestC * c=new TestC(this,aTestV);
  testCs.push_back(c);
  this->getGraph()->insertConnection(c);
}

void TestV::disconnect()
{
  for (int i=0;i<testCs.size();i++)
  {
    TestC * c=testCs[i];    
    if (testCs[i]->getVertex1()!=this)
      testCs[i]->swap();
    this->getGraph()->removeConnection(testCs[i]);
    delete testCs[i];
  }
  testCs.clear();
}

TestC::TestC(TestV * t1, TestV * t2) : Connection(t1,t2)
{
  static int maxID=-1;
  this->ID=++maxID;
}

void TestC::print() const
{
  printf ("TestC ID=%i\n",this->ID);
  Connection::print ();
}

void TestC::swap()
{
  Vertex * tmp=this->vertex1;
  this->vertex1=vertex2;
  this->vertex2=tmp;  
}

TestG::TestG()
{
  static int maxID=-1;
  this->ID=++maxID;
}

TestG::~TestG()
{  
}

void TestG::destroyGraph(Graph * aGraph)
{
  delete aGraph;
}

Graph * TestG::createGraph ()
{
  return new TestG();
}

void TestG::print() 
{
  printf ("TestG ID=%i\n",this->ID);
  Graph::print ();
}

