/***************************************************************************
                          HingeJoint.cc  -  description
                             -------------------
    begin                : Mon Mar 17 2003
    copyright            : (C) 2003 by Michel DUBOIS, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/**
  * changes :
  *  - 15/6/03 Creation !
  */

#include <ode/ode.h>

#include "mmgr.h"

#include "Simulator.h"
#include "HingeJoint.h"
#include "RigidBody.h"

HingeJoint::HingeJoint (SimRigidBody* rigidBody1, SimRigidBody* rigidBody2) : SimJoint (rigidBody1, rigidBody2)
{
	joint = dJointCreateHinge (Simulator::getInstance ()->getWorld (), 0);
  
	dJointAttach (joint, rigidBody1->getBody (), rigidBody2->getBody ());

	dJointSetHingeParam (joint, dParamLoStop,  0.0);
	dJointSetHingeParam (joint, dParamHiStop,  0.0);
	dJointSetHingeParam (joint, dParamVel,     0.0);
	//dJointSetHingeParam (joint, dParamStopCFM, 0.5);
	//dJointSetHingeParam (joint, dParamStopERP, 0.9);
	dJointSetHingeParam (joint, dParamFMax,    dInfinity);
 // printf("HingeJoint::HingeJoint\n");
}

HingeJoint::~HingeJoint ()
{

}

void HingeJoint::print () const
{
  printf ("HingeJoint:\n");
  SimJoint::print ();
}

void HingeJoint::set (char* name, char* value)
{

}

void HingeJoint::setAnchor (const Vector3& anchor)
{
	dJointSetHingeAnchor (joint, anchor.x, anchor.y, anchor.z);
}

void HingeJoint::setAxis (const Vector3& axis1)
{
	dJointSetHingeAxis (joint, axis1.x, axis1.y, axis1.z);
}

/*
void HingeJoint::setMaxForce (const Real maxForce1)
{	
	dJointSetHingeParam (joint, dParamFMax, maxForce1);
}
*/
void HingeJoint::setTargetVelocity (const Real velocity)
{
	dJointSetHingeParam (joint, dParamVel, velocity);
}

Real HingeJoint::getAngle()
{
 return dJointGetHingeAngle(joint);  
}

Real HingeJoint::getAngleRate()
{
  return dJointGetHingeAngleRate(joint);
}

void HingeJoint::setParam (int parameter, Real value)
{
  dJointSetHingeParam (joint, parameter, value);
  
}

Real HingeJoint::getParam (int parameter) const
{
 return dJointGetHingeParam(joint, parameter);
}

//OldHingeJoint::OldHingeJoint (SimRigidBody* rigidBody1, SimRigidBody* rigidBody2) : Joint (rigidBody1, rigidBody2), Connection(rigidBody1, rigidBody2), Controller()
OldHingeJoint::OldHingeJoint (SimRigidBody* rigidBody1, SimRigidBody* rigidBody2) : Joint (), Connection(rigidBody1, rigidBody2), Controller()
{
	joint = dJointCreateHinge (Simulator::getInstance ()->getWorld (), 0);

	dJointAttach (joint, rigidBody1->getBody (), rigidBody2->getBody ());
  this->rigidBody1 = rigidBody1;
	this->rigidBody2 = rigidBody2;
  this->getVertex1()->getGraph()->insertConnection(this);

	dJointSetHingeParam (joint, dParamLoStop,  0.0);
	dJointSetHingeParam (joint, dParamHiStop,  0.0);
	dJointSetHingeParam (joint, dParamVel,     0.0);
	//dJointSetHingeParam (joint, dParamStopCFM, 0.5);
	//dJointSetHingeParam (joint, dParamStopERP, 0.9);
	dJointSetHingeParam (joint, dParamFMax,    dInfinity);
}

OldHingeJoint::~OldHingeJoint ()
{
  this->print();
  this->getGraph()->removeConnection(this);

}

void OldHingeJoint::set (char* name, char* value)
{

}

void OldHingeJoint::setAnchor (const Vector3& anchor)
{
	dJointSetHingeAnchor (joint, anchor.x, anchor.y, anchor.z);
}

void OldHingeJoint::setAxis (const Vector3& axis1)
{
	dJointSetHingeAxis (joint, axis1.x, axis1.y, axis1.z);
}

/*
void HingeJoint::setMaxForce (const Real maxForce1)
{
	dJointSetHingeParam (joint, dParamFMax, maxForce1);
}
*/
void OldHingeJoint::setTargetVelocity (const Real velocity)
{
	dJointSetHingeParam (joint, dParamVel, velocity);
}

Real OldHingeJoint::getAngle()
{
 return dJointGetHingeAngle(joint);
}

Real OldHingeJoint::getAngleRate()
{
  return dJointGetHingeAngleRate(joint);
}

void OldHingeJoint::setParam (int parameter, Real value)
{
  dJointSetHingeParam (joint, parameter, value);

}

Real OldHingeJoint::getParam (int parameter) const
{
 return dJointGetHingeParam(joint, parameter);
}

void OldHingeJoint::swap()
{
}

void OldHingeJoint::setIsEnabled (const bool isEnabled)
{
  Controller::setIsEnabled (isEnabled);
}


void OldHingeJoint::print() const
{
  printf ("SimJoint (OldHingeJoint): Enable=%i\n",this->isEnabled);
  Connection::print ();
}
 
 