/***************************************************************************
                          Leg.cpp  -  description
                             -------------------
    begin                : mar juin 8 2003
    copyright            : (C) 2003 by Michel DUBOIS, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Modle statique

#include "mmgr.h"
#include "Leg.h"

using namespace API;

/**
  * changes :
  *  - 08/06/03 MD Creation !
  */


Leg::Leg(Atom* anAtom, int aLegID)
{
  this->setAtom(anAtom);
  this->setID(aLegID);
  this->setMappedID(aLegID);
  this->linkOfLeg=0;
  for (int i=0;i<getServoMotorCount();i++) {
    lservo[i]=new ServoMotor();
    lservo[i]->setID(i);
    lservo[i]->setLeg(this);        
  }
  this->util=new Util(this);
  this->lirsend=0;
  this->lirreceiver=0;
  for (int i=0;i<getForceSensorCount();i++) {
    lforce[i]=0;
  }
  this->ltouch=0;
  this->linkOfLeg=0;
}

Leg::~Leg()
{
  delete this->util;
  if (linkOfLeg) {
    delete linkOfLeg;
    linkOfLeg=0;
  }
  for (int i=0;i<2;i++) {
    if (lservo[i])
      delete lservo[i];
    lservo[i]=0;
  }
}

int Leg::getID() const
{
	return ID;
}

int Leg::getMappedID() const
{
	return MappedID;
}


Atom* Leg::getAtom() const
{
 return atom;
}


SphericPosition& Leg::getSphericPosition() const
{                                      
//  return SphericPosition(lservo[0]->getCurrentAngle(),lservo[1]->getCurrentAngle());
}

CartesianPosition& Leg::getRelCartesianPosition() const
{
//  return CartesianPosition(0,0,0);
}

Leg * Leg::getLinkedLeg() 
{
  if (linkOfLeg)
    return this->linkOfLeg->getOtherLeg(this);
  else
    return 0;
}

std::string Leg::getMessage()
{
	return " ";
}

unsigned int Leg::getServoMotorCount() const
{
  return 2;
}

ServoMotor* Leg::getServoMotor(unsigned int aServoMotorID) const
{
  if (aServoMotorID<getServoMotorCount())
    return lservo[aServoMotorID];
  else
    return 0;
}

unsigned int Leg::getForceSensorCount() const
{
  return 3;
}

ForceSensor* Leg::getForceSensor(unsigned int aForceSensorID) const
{
  if (aForceSensorID<getForceSensorCount())
    return lforce[aForceSensorID];
  else
    return 0;
}

TouchSensor* Leg::getTouchSensor() const
{
  return this->ltouch;
}

IRReceiver* Leg::getIRReceiver() const
{
  return this->lirreceiver;
}

IRSender* Leg::getIRSender() const
{
  return this->lirsend;
}

bool Leg::isLinked() const
{
  if (linkOfLeg)
    return true;
  else
    return false;
}

void Leg::setLink(Leg * aLeg)
{
 if ((!isLinked()) && (!aLeg->isLinked()))
  {
    Link * link=new Link(this,aLeg);
//    simleg->getSimServoMotor(0)->print();
//    simleg->getSimServoMotor(1)->print();
    //lservo[0]->simservo=simleg->getSimServoMotor(0);
    lservo[1]->util->simservo=util->simleg->getSimServoMotor(1);
    //aLeg->lservo[0]->simservo=aLeg->simleg->getSimServoMotor(0);
    aLeg->lservo[1]->util->simservo=aLeg->util->simleg->getSimServoMotor(1);

    this->linkOfLeg=link;
    aLeg->linkOfLeg=link;
    atom->getMolecule()->insertLink(link);
  }
}

Link * Leg::getLink() const
{
  return this->linkOfLeg;
}

void Leg::sendMessage(std::string aMessage)
{

}

Leg::Util::Util(Leg * aLeg)
{
  this->leg=aLeg;
  this->simleg=0;
}

Leg::Util::~Util()
{
}


void Leg::unlink()
{
 if (isLinked())
  {
    if (linkOfLeg->getLeg1()!=this)
      linkOfLeg->swap();
    Link * tmp=linkOfLeg;
    this->atom->getMolecule()->removeLink(linkOfLeg);
    tmp->getOtherLeg(this)->linkOfLeg=0;
    this->linkOfLeg=0;
    Leg * aLeg=tmp->getOtherLeg(this);
    delete tmp;
    //lservo[0]->simservo=simleg->getSimServoMotor(0);
    lservo[1]->util->simservo=util->simleg->getSimServoMotor(1);
    //aLeg->lservo[0]->simservo=aLeg->simleg->getSimServoMotor(0);
    aLeg->lservo[1]->util->simservo=aLeg->util->simleg->getSimServoMotor(1);

    
  }
}

void Leg::stop()
{
  lservo[0]->stop();
  lservo[1]->stop();
}

void Leg::setID(int aLegID)
{
  this->ID=aLegID;
}

void Leg::setMappedID(int aMappedID)
{
  this->MappedID=aMappedID;
}

void Leg::Util::setSphericPosition(SphericPosition aSphericPosition)
{
//  this->sp=aSphericPosition;
}

void Leg::setAtom(Atom* anAtom)
{
  this->atom=anAtom;
}

SimLeg * Leg::Util::getSimulatedObject() const
{
  return this->simleg;  
}  

bool Leg::Util::isConnectable(Leg * aLeg) const
{
  return this->simleg->isConnectable(aLeg->util->simleg);
}

bool Leg::Util::isOnGround() const
{
  return (Math::isEqual(this->simleg->getAbsPosEnd().z,0));
}
    
void Leg::Util::print () const
{
  printf ("Leg: physicalID=%i logicalID=%i atomID=%i\n",this->leg->ID, this->leg->MappedID,this->leg->atom->getID());
}
