/***************************************************************************
                          LegGeometry.cpp  -  description
                             -------------------
    begin                : jeu mar 18 2004
    copyright            : (C) 2004 by Michel Dubois, Yann Le Guyadec 
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


/*=========================================================================
UberSim Source Code Release
-------------------------------------------------------------------------
Copyright (C) 2002 Manuela Veloso, Brett Browning, Mike Bowling,
                   James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
                   Erick Tryzelaar {erickt}@andrew.cmu.edu
School of Computer Science, Carnegie Mellon University
-------------------------------------------------------------------------
This software is distributed under the GNU General Public License,
version 2.  If you do not have a copy of this licence, visit
www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
in the hope that it will be useful, but WITHOUT ANY WARRANTY,
including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-------------------------------------------------------------------------*/

#include "nommgr.h"

#ifdef USE_OPENGL
#include <drawstuff/drawstuff.h>
#endif
#include <ode/ode.h>
#include <ode/dCylinder.h>

#include "mmgr.h"

#include "LegGeometry.h"
/**
  * changes :
  *  - 22/6/03 Ajout methode print() !
  *  - 12/12/03 correction gros bug !!!!
  */
LegGeometry::LegGeometry (Real length, Real radius, char* name, int geom, int num) : Geometry (name)
{
	this->length = length;
	this->radius = radius;
  this->numleg=num;
  Real width=radius*2;
  Real height=width;
  if (geom==1)
	  geometry  = dCreateBox (0, length, width, height);
  if (geom==0) 
    geometry = dCreateCCylinder (0, radius, length-2*0.025);
  if (geom==2)
    geometry = dCreateCylinder (0, radius, length);
  if (geom==3) {
    geometry  = dCreateGeomTransform (0);
    dGeomTransformSetCleanup (geometry,1);
    sphereGeometry = dCreateSphere (0, radius);
    boxGeometry=dCreateBox (0, length-width, width, height);
    dGeomTransformSetGeom (geometry,boxGeometry);
    dGeomTransformSetGeom (geometry,sphereGeometry);
    dReal dposBox[3]={0+(length-width)/2,0,0};
    //dGeomSetPosition (boxGeometry, dposBox[0], dposBox[1], dposBox[2]);
    dReal dposSphere[3]={0+(length-width)+radius,0,0};
    //dGeomSetPosition (sphereGeometry, dposSphere[0], dposSphere[1], dposSphere[2]);
    
    

  }
	modelBound = BoundingSphere (Vector3::ZERO, (Vector3 (length, width, height) / 2.0).length ());
  

//	geometry  = dCreateSphere (0, radius); //gros bug !!!!!!!!!!!!!!!!!!!
//  geometry = dCreateCCylinder (0, radius, length-2*0.025);
// 	modelBound = BoundingSphere (Vector3::ZERO, radius + length * 0.5);
}

LegGeometry::~LegGeometry ()
{
}

void LegGeometry::draw ()
{
#ifdef USE_OPENGL
	dsSetColor   (color.x, color.y, color.z);
	dsSetTexture (DS_WOOD);

	dMatrix3 r;
	r[0]  = worldRotation[0][0];
	r[1]  = worldRotation[0][1];
	r[2]  = worldRotation[0][2];
	r[3]  = 0.0;
	r[4]  = worldRotation[1][0];
	r[5]  = worldRotation[1][1];
	r[6]  = worldRotation[1][2];
	r[7]  = 0.0;
	r[8]  = worldRotation[2][0];
	r[9]  = worldRotation[2][1];
	r[10] = worldRotation[2][2];
	r[11]  = 0.0;

  int type = dGeomGetClass (geometry);
  if (type == dBoxClass) {
    Real width=radius*2;
    Real height=width;
	  Vector3 ss (length, width, height);
	  dsDrawBoxD (worldPosition, r, ss);
  }
  else if (type == dCCylinderClass) {
    dsDrawCappedCylinderD (worldPosition, r, length, radius);
  }
  else if (type == dCylinderClassUser) {
    //Real width=radius*2;
    //Real height=width;
	  //Vector3 ss (length, width, height);
	  //dsDrawBoxD (worldPosition, r, ss);

    if (numleg==0)
    //devant,rouge
      dRFromAxisAndAngle (r,0, 1, 0, M_PI_2);
    else if (numleg==1)
    //derriere, vert
      dRFromAxisAndAngle (r,0, 1, 0, -M_PI_2);
    else if (numleg==2)
    //bas, bleu fonce
      dRFromAxisAndAngle (r,0, 0, 1, M_PI_2);
    else if (numleg==3)
    //haut, jaune
      dRFromAxisAndAngle (r,0, 0, 1, -M_PI_2);
    else if (numleg==4)
    //droite, bleu clair
      dRFromAxisAndAngle (r,1, 0, 0, M_PI_2);
    else if (numleg==5)
    //gauche, violet
      dRFromAxisAndAngle (r,1, 0, 0, -M_PI_2);  
      
	  dsDrawCylinderD (worldPosition, r, length, radius);
  }
  else if (type == dGeomTransformClass) {

  }

	
#endif
  Geometry::draw ();
}

/*
void BoxGeometry::draw ()
{
#ifdef USE_OPENGL

	dsSetColor   (color.x, color.y, color.z);
	dsSetTexture (DS_WOOD);

	dMatrix3 r;
	r[0]  = worldRotation[0][0];
	r[1]  = worldRotation[0][1];
	r[2]  = worldRotation[0][2];
	r[3]  = 0.0;
	r[4]  = worldRotation[1][0];
	r[5]  = worldRotation[1][1];
	r[6]  = worldRotation[1][2];
	r[7]  = 0.0;
	r[8]  = worldRotation[2][0];
	r[9]  = worldRotation[2][1];
	r[10] = worldRotation[2][2];
	r[11]  = 0.0;

	Vector3 ss (length, width, height);
	dsDrawBoxD (worldPosition, r, ss);
#endif
  Geometry::draw ();
}
*/
void LegGeometry::print ()
{
  if (name)
  printf ("Leg Name: %s\n", name);
  else
  printf ("Leg anonyme\n");
   printf ("UBERSIM LP: %f %f %f WP: %f %f %f\n",
          getLocalPosition().x,
          getLocalPosition().y,
          getLocalPosition().z,
          getWorldPosition().x,
          getWorldPosition().y,
          getWorldPosition().z
  );
  const dReal * pos=dGeomGetPosition(geometry);
  printf ("ODE WP: %f %f %f\n",
          pos[0],
          pos[1],
          pos[2]
  );


}
