/***************************************************************************
                          SimAtom.cpp  -  description
                             -------------------
    begin                : mer jun 25 2003
    copyright            : (C) 2003 by Michel DUBOIS, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "ConfigReader.h"
#include <ode/ode.h>

#include "mmgr.h"
#include "SimAtom.h"
#include "Event.h"
#include "Geometry.h"
#include "SphereGeometry.h"
#include "RigidBody.h"
#include "RigidBodyCollider.h"
#include "Timer.h"
#include "Surface.h"
#include "Node.h"
#include "CollisionManager.h"

/**
  * changes :
  *  - 08/06/03 MD Creation !
  */

CR_DECLARE (KERNELSIDE);
CR_DECLARE (KERNELMASS);
CR_DECLARE (KERNELMU);
CR_DECLARE (KERNELMU2);
CR_DECLARE (KERNELSLIP1);
CR_DECLARE (KERNELSLIP2);
CR_DECLARE (SHOWCONTACTS);
CR_DECLARE (SHOWAABB);

std::string SimAtom::showaabb="";
std::string SimAtom::showcontacts="";

SimAtom::SimAtom()
{
  static bool isConfiged = false;

	if (!isConfiged)
	{
		isConfiged = true;

		CR_SETUP (Atom, KERNELSIDE, CR_DOUBLE);
    CR_SETUP (Atom, KERNELMASS, CR_DOUBLE);
    CR_SETUP (Atom, KERNELMU, CR_DOUBLE);
    CR_SETUP (Atom, KERNELMU2, CR_DOUBLE);
    CR_SETUP (Atom, KERNELSLIP1, CR_DOUBLE);
    CR_SETUP (Atom, KERNELSLIP2, CR_DOUBLE);
    CR_SETUP (Atom, SHOWCONTACTS,CR_STRING);
    CR_SETUP (Atom, SHOWAABB, CR_STRING);
    SimAtom::showaabb=SVAR(SHOWAABB);
    SimAtom::showcontacts=SVAR(SHOWCONTACTS);
    
  }
    if (showcontacts=="TRUE")
      {
        //printf("show contacts\n");  
        CollisionManager::getInstance ()->setShowMode(true);
      }
    else
      {
        //printf("no show contacts\n");
        CollisionManager::getInstance ()->setShowMode(false);
      }
      
  Node* robot = new Node ("Atom Robot");
	this->model = robot;

	rigidBody        = new SimRigidBody ();
	motionController = rigidBody;
	rigidBody->setModel (robot);

  dMass kernelMass;
  dMassSetSphere (&kernelMass,1,0.5*DVAR (KERNELSIDE));
	dMassAdjust  (&kernelMass, DVAR (KERNELMASS));
	dBodySetMass (rigidBody->getBody (), &kernelMass);

	Geometry* kernelGeometry = new SphereGeometry (0.5*DVAR (KERNELSIDE), "Kernel");
  if (showaabb=="TRUE")
  {
      //printf("showaabb\n");
      kernelGeometry->setAABBShowMode(true);
  }
  else
  {
      //printf("no showaabb\n");
      
      kernelGeometry->setAABBShowMode(false);
  }
      
  kernelGeometry->setColor (Vector3(1,1,0));
  Surface* kernelSurface = new Surface ();
	dSurfaceParameters* sp = &kernelSurface->getSurfaceParameters ();
	sp->mode  = dContactApprox1 | dContactFDir1 | dContactMu2 | dContactSlip1 ;//| dContactSlip2;
	sp->mu    = DVAR (KERNELMU);
	sp->mu2   = DVAR (KERNELMU2);
	sp->slip1 = DVAR (KERNELSLIP1);
	sp->slip2 = DVAR (KERNELSLIP2);
  kernelCollider=new RigidBodyCollider (rigidBody);
  kernelCollider->setSurface (kernelSurface);
	robot->addChild                       (kernelGeometry);
  kernelGeometry->setCollisionController (kernelCollider);
	kernelGeometry->setLocalPosition       (Vector3 (0,0,0));
  
  for (int i=0;i<getSimLegCount();i++) {
    legs[i]=new SimLeg(this, i);
  }
   
}

SimAtom::~SimAtom(){
  Timer::getInstance ()->clearEvents(this);

  for (int i=0;i<getSimLegCount();i++) {
    delete legs[i];
    legs[i]=0;
  }

//delete aclock;
//delete [] aleg;
//delete [] agiro;
//delete [] aaccel;

//delete  hingeWorld; // see "locks kernel to world" section

  //dJointID fixedLink[6];

}

unsigned int SimAtom::getID() const
{
  return ID;
}

void SimAtom::setID(unsigned int anAtomID)
{
  this->ID=anAtomID;
}

RigidBody * SimAtom::getKernel() const
{
  return rigidBody;
}

unsigned int SimAtom::getSimLegCount() const
{
  return 6;
}

SimLeg * SimAtom::getSimLeg(unsigned int aSimLegID) const
{
  if (aSimLegID<getSimLegCount())
    return legs[aSimLegID];
  else
    return 0;
}


void SimAtom::handleEvent (Event* event)
{
	if (!event)
	{
		return;
	}

	////


	////

	RigidBodyEntity::handleEvent (event);
}

void SimAtom::print()
{
  printf("SimAtom: ID=%i\n",ID);
  
}

Vector3 SimAtom::getPosition()
{
  return this->getMotionController()->getWorldPosition();
}    