/***************************************************************************
                          SimServoMotor.cpp  -  description
                             -------------------
    begin                : mar jui 15 2003
    copyright            : (C) 2003 by Michel DUBOIS, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ConfigReader.h"
#include <ode/ode.h>

#include "mmgr.h"
#include "SimServoMotor.h"
#include "Timer.h"

CR_DECLARE (SERVOMOTORMINPOS);
CR_DECLARE (SERVOMOTORMAXPOS);
CR_DECLARE (SERVOMOTORLOWSTOPANGLE);
CR_DECLARE (SERVOMOTORHIGHSTOPANGLE);
CR_DECLARE (SERVOMOTORMAXFORCE);
CR_DECLARE (SERVOMOTORMAXVELOCITY);
CR_DECLARE (SERVOMOTORPID);
CR_DECLARE (SERVOMOTORSLDANGLE);

double SimServoMotor::servoMotorLowStopAngle=0;
double SimServoMotor::servoMotorHighStopAngle=0;
int SimServoMotor::servoMotorMinPos=0;
int SimServoMotor::servoMotorMaxPos=0;
double SimServoMotor::servoMotorRatioPos2Angle=0;
double SimServoMotor::servoMotorRatioAngle2Pos=0;
double SimServoMotor::servoMotorMaxAmplitudePos=0;
double SimServoMotor::servoMotorMaxAmplitudeAngle=0;

double SimServoMotor::getCurrentVelocity() const
{
  if (hinge)
    return hinge->getAngleRate();
  else
    return 0;
}

double SimServoMotor::getTargetVelocity() const
{
  return this->targetSpeed;
}

HingeJoint * SimServoMotor::getHingeJoint() const
//OldHingeJoint * SimServoMotor::getHingeJoint() const
{
  return this->hinge;
}

AMotorJoint  * SimServoMotor::getAMotorJoint() const
//AMotor  * SimServoMotor::getAMotor() const
{
  return this->amotor;
}

SimServoMotor::SimServoMotor(SimRigidBody * body1, SimRigidBody * body2)
{
  checkConfig();
  this->setHingeJoint(new HingeJoint(body1, body2));
//  this->setHingeJoint(new OldHingeJoint(body1, body2));
  this->setAMotorJoint(new AMotorJoint(body1,body2));
//  this->setAMotorJoint(new AMotor(body1,body2));
  this->coupledSSMotor=0;
  this->direction=0;
  this->ID=0;
  this->sleg=0;
  this->targetSpeed=0;
  this->model=0;
}

SimServoMotor::SimServoMotor(HingeJoint * aHingeJoint, AMotorJoint * anAMotor)
//SimServoMotor::SimServoMotor(HingeJoint * aHingeJoint, AMotor * anAMotor)
//SimServoMotor::SimServoMotor(OldHingeJoint * aHingeJoint, AMotor * anAMotor)
{
  checkConfig();
  this->setHingeJoint(aHingeJoint);
  this->setAMotorJoint(anAMotor);  
  this->coupledSSMotor=0;
  this->direction=0;
  this->ID=0;
  this->sleg=0;
  this->targetSpeed=0;
  this->model=0;      
}

void SimServoMotor::checkConfig()
{
  static bool isConfiged = false;
  if (!isConfiged)
	{
		isConfiged = true;
		CR_SETUP (Atom, SERVOMOTORMINPOS, CR_INT);
		CR_SETUP (Atom, SERVOMOTORMAXPOS, CR_INT);
		CR_SETUP (Atom, SERVOMOTORLOWSTOPANGLE, CR_DOUBLE);
		CR_SETUP (Atom, SERVOMOTORHIGHSTOPANGLE, CR_DOUBLE);
    CR_SETUP (Atom, SERVOMOTORMAXFORCE, CR_DOUBLE);
    CR_SETUP (Atom, SERVOMOTORMAXVELOCITY, CR_DOUBLE);
    CR_SETUP (Atom, SERVOMOTORPID, CR_DOUBLE);
    CR_SETUP (Atom, SERVOMOTORSLDANGLE, CR_DOUBLE);
    servoMotorLowStopAngle=Math::min(DVAR(SERVOMOTORLOWSTOPANGLE)*M_PI,DVAR(SERVOMOTORHIGHSTOPANGLE)*M_PI);
    servoMotorHighStopAngle=Math::max(DVAR(SERVOMOTORLOWSTOPANGLE)*M_PI,DVAR(SERVOMOTORHIGHSTOPANGLE)*M_PI);
    servoMotorMinPos=Math::min(IVAR(SERVOMOTORMINPOS),IVAR(SERVOMOTORMAXPOS));
    servoMotorMaxPos=Math::max(IVAR(SERVOMOTORMINPOS),IVAR(SERVOMOTORMAXPOS));
    servoMotorMaxAmplitudePos=servoMotorMaxPos-servoMotorMinPos;
    servoMotorMaxAmplitudeAngle=servoMotorHighStopAngle-servoMotorLowStopAngle;
    servoMotorRatioAngle2Pos=servoMotorMaxAmplitudePos/servoMotorMaxAmplitudeAngle;
    servoMotorRatioPos2Angle=servoMotorMaxAmplitudeAngle/servoMotorMaxAmplitudePos;
  }  
}    

SimServoMotor::SimServoMotor()
{
  checkConfig();
  this->hinge=0;
  this->amotor=0;
  this->coupledSSMotor=0;
  this->direction=0;
  this->ID=0;
  this->sleg=0;
  this->targetSpeed=0;
  this->model=0;
}


SimServoMotor::~SimServoMotor()
{
  Timer::getInstance ()->clearEvents(this);

 
  if (hinge)
  {
    //printf("Hinge a detruire :\n");
    //hinge->print();
    delete hinge;
  }
  if (amotor)
  {
    //printf("AMotor a detruire :\n");
    //amotor->print();
    delete amotor;
  }
  hinge=0;
  amotor=0;  
  if (coupledSSMotor)
  {
    this->coupledSSMotor->coupledSSMotor=0;
    this->coupledSSMotor=0;
  }
  
  if (sleg)
    sleg->servo[ID]=0;
  sleg=0;
  ID=0;
  direction=0;
  targetSpeed=0;  
}

void SimServoMotor::setHingeJoint(HingeJoint * aHingeJoint)
//void SimServoMotor::setHingeJoint(OldHingeJoint * aHingeJoint)
{
  this->hinge=aHingeJoint;
  this->hinge->setParam(dParamLoStop,servoMotorLowStopAngle);
  this->hinge->setParam(dParamHiStop,servoMotorHighStopAngle);
  this->hinge->setParam(dParamFMax,DVAR(SERVOMOTORMAXFORCE));
 // this->hinge->setParam(dParamFMax,0.1); //????
}

void SimServoMotor::setAxis(Vector3 anAxis)
{
  if (hinge)
    hinge->setAxis(anAxis);
  if (amotor)
    hinge->setAxis(anAxis);
}

void SimServoMotor::setAMotorJoint(AMotorJoint * anAMotor)
//void SimServoMotor::setAMotorJoint(AMotor * anAMotor)
{
  this->amotor=anAMotor;
  this->amotor->setNumAxes (1);
  this->amotor->setMode(dAMotorUser);
  this->amotor->setParam(dParamLoStop,servoMotorLowStopAngle);
  this->amotor->setParam(dParamHiStop,servoMotorHighStopAngle);
  this->amotor->setParam(dParamFMax,DVAR(SERVOMOTORMAXFORCE));
  this->amotor->setParam(dParamBounce,0);
  this->amotor->setParam(dParamStopERP,0.2);
  this->amotor->setParam(dParamCFM,0.5); 
  this->amotor->setParam(dParamVel,0);
  this->amotor->setParam(dParamFudgeFactor,0.5);  
}

Real SimServoMotor::getParam(int parameter) const
{
  Real value=0;
  if (hinge)
    value=hinge->getParam(parameter);
  if (amotor)
    if (amotor->getParam(parameter)!=value)
      value=0;
  return value;    
}

void SimServoMotor::setParam(int parameter, Real value)
{
  if (hinge)
    hinge->setParam(parameter,value);
  if (amotor)
    amotor->setParam(parameter,value);
}

double SimServoMotor::getAngle()
{
  if (hinge)
    return hinge->getAngle();
  else
    return 0;    
}

int SimServoMotor::getHalfPosition() const
{
  return servoMotorMaxAmplitudePos/2+servoMotorMinPos;
}

double SimServoMotor::Position2Angle(int aPosition) const
{
  double amplitude=aPosition-servoMotorMinPos;
  if (amplitude<0)
    amplitude=0;
  if (amplitude>servoMotorMaxAmplitudePos)
    amplitude=servoMotorMaxAmplitudePos;
  return amplitude*servoMotorRatioPos2Angle+servoMotorLowStopAngle;    
}

int SimServoMotor::Angle2Position(double anAngle) const
{
  double amplitude=anAngle-servoMotorLowStopAngle;
//  printf ("S1: %f\n", amplitude);
  if (amplitude<0)
    amplitude=0;
  if (amplitude>servoMotorMaxAmplitudeAngle)
    amplitude=servoMotorMaxAmplitudeAngle;
  /*
  printf ("S2: %f\n", amplitude);  
  printf ("double %f\n",amplitude*servoMotorRatioAngle2Pos+servoMotorMinPos);
  printf ("int %i\n",(int) (amplitude*servoMotorRatioAngle2Pos+servoMotorMinPos));
  printf ("double2int %i\n",Math::double2int(amplitude*servoMotorRatioAngle2Pos+servoMotorMinPos));
  
  int ret=amplitude*servoMotorRatioAngle2Pos+servoMotorMinPos;
  //return (int) (amplitude*servoMotorRatioAngle2Pos+servoMotorMinPos);
  */
  return Math::double2int(amplitude*servoMotorRatioAngle2Pos+servoMotorMinPos);
  //return ret;
}

int SimServoMotor::getMinPosition() const
{
  return servoMotorMinPos;
}

int SimServoMotor::getMaxPosition() const
{
  return servoMotorMaxPos;
}

double SimServoMotor::getLowStopAngle() const
{
  return servoMotorLowStopAngle;
}

double SimServoMotor::getHighStopAngle() const
{
  return servoMotorHighStopAngle;
}

SimServoMotor * SimServoMotor::getCoupledSimServoMotor() const
{
  return this->coupledSSMotor;
}

void SimServoMotor::setCoupledServoMotor(SimServoMotor * aCoupledSimServoMotor)
{
  this->coupledSSMotor=aCoupledSimServoMotor;
}
  
void SimServoMotor::handleEvent (Event* event)
{ double currentAngle;
  double targetAngle;
//  printf("queue.count = %f\n",queue.count);
	if (!event)
	{
		return;
	}

	////
	ServoMotorStopEvent * stopEvent = dynamic_cast<ServoMotorStopEvent *> (event);

	if (stopEvent)
	{

		delete event;
		return;
	}

  ServoMotorTargetPositionEvent * posEvent = dynamic_cast<ServoMotorTargetPositionEvent *> (event);

	if (posEvent) {
    currentAngle = hinge->getAngle();
    //if (currentAngle==-0)
    //  currentAngle=0;
    int targetpos=posEvent->targetPosition;
/*    if (targetpos==0)
      targetpos=1;
    if (targetpos==250)
      targetpos=249;  */
    targetAngle = this->Position2Angle(targetpos)*fabs(sin(this->coupledSSMotor->hinge->getAngle()*2+M_PI_2));
   
/*
    printf("posEvent: Atom=%i Leg=%i Servo=%i Wished Position=%i Wished Angle=%f \nCurrent Angle=%f Target Angle=%f \nCurrent Pos=%i Target Pos=%i\n Delta Position=%i Delta Angle=%f\n",sleg->simatom->getID(),sleg->ID, ID, posEvent->targetPosition, this->Position2Angle(posEvent->targetPosition),
     currentAngle, targetAngle, this->Angle2Position(currentAngle), this->Angle2Position(targetAngle), this->Angle2Position(servoMotorLowStopAngle+M_PI_2*DVAR(SERVOMOTORSLDANGLE)),M_PI_2*DVAR(SERVOMOTORSLDANGLE));
*/
    if (this->Angle2Position(targetAngle) == this->Angle2Position(currentAngle))
        this->targetSpeed=0;
    else
    if (this->direction == 0)
      if ((currentAngle < targetAngle) && (currentAngle >= targetAngle-(M_PI_2*DVAR(SERVOMOTORSLDANGLE))))
        // SLOWDOWN
        this->targetSpeed=0.2f+fabs(currentAngle-targetAngle)*(2.0f*0.6f/(M_PI*DVAR(SERVOMOTORSLDANGLE)));
      else
//        if (currentAngle >= targetAngle-M_PI_2*0.03) {
        if (currentAngle >= targetAngle) {

          // TURN
          this->targetSpeed=-0.1f; // initial speed == -0.1
          this->direction=1;
        }
        else {
            // ACCELERATE
            if (this->targetSpeed<1.0f)   // maximum speed == 1
              this->targetSpeed+=0.1f;
        }
    else { // this->direction == 1
      //targetAngle*=-1.0f;
      if ((currentAngle > targetAngle) && (currentAngle <= targetAngle+(M_PI_2*DVAR(SERVOMOTORSLDANGLE)))) 
        // SLOWDOWN
        this->targetSpeed=-0.2f-fabs(currentAngle-targetAngle)*(2.0f*0.6f/(M_PI*DVAR(SERVOMOTORSLDANGLE)));
      else
//        if (currentAngle < targetAngle+M_PI_2*0.03) {
        if (currentAngle < targetAngle) {

          // TURN
          this->targetSpeed=+0.1f; // initial speed == 0.1
          this->direction=0;
        }
        else {
            // ACCELERATE
            if (this->targetSpeed>-1.0f)   // maximum speed == 1
              this->targetSpeed-=0.1f;
        }
    }
   

    // applique la consigne
    this->amotor->setParam(dParamVel,this->targetSpeed);
    this->hinge->setParam(dParamVel,this->targetSpeed);
//    printf("\n targetspeed : %f\n",this->targetSpeed);
    delete event;

//    if (!(Math::isEqual(this->Position2Angle(posEvent->targetPosition),currentAngle))) {
    if (this->targetSpeed!=0)
      Timer::getInstance ()->addEvent (this, new ServoMotorTargetPositionEvent (posEvent->targetPosition), 0,posEvent->ID,posEvent->syncIndex);

      
      // new event a generer dans auxqueue ?
//    }
    
		return;
	}

  ServoMotorTargetVelocityEvent * velEvent = dynamic_cast<ServoMotorTargetVelocityEvent *> (event);

	if (velEvent)
	{

		delete event;
/*    if (NOT_ATTEINT_VITESSE) {
      // new event a generer
    }
    */
		return;
	}

  ////

  Entity::handleEvent (event);
  
}
void SimServoMotor::print()
{
  printf("SimServoMotor: Atom ID=%i Leg ID=%i Servo ID=%i\n",sleg->simatom->getID(),sleg->ID, ID);
}                    