/***************************************************************************
                          SimWorld.cpp  -  description
                             -------------------
    begin                : dim jui 6 2003
    copyright            : (C) 2003 by Michel DUBOIS, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "nommgr.h"
#include <ode/ode.h>

#ifdef USE_OPENGL
#include <drawstuff/drawstuff.h>
#endif

#include "ConfigReader.h"

#include "mmgr.h"

#include "SimWorld.h"
#include "Simulator.h"
#include "BoxGeometry.h"
#include "Geometry.h"
#include "PlaneGeometry.h"
#include "Surface.h"

/**
  * changes :
  *  - 22/6/03 Ajout methode print() !
  */

#ifndef NOENV
  
CR_DECLARE (LONG_LENGTH);
CR_DECLARE (LONG_WIDTH);
CR_DECLARE (LONG_HEIGHT);
CR_DECLARE (TOP_LONG_POSITION);
CR_DECLARE (BOTTOM_LONG_POSITION);
CR_DECLARE (SHORT_LENGTH);
CR_DECLARE (SHORT_WIDTH);
CR_DECLARE (SHORT_HEIGHT);
CR_DECLARE (TOP_SHORT_POSITION);
CR_DECLARE (BOTTOM_SHORT_POSITION);
CR_DECLARE (GOAL_SIDE_LENGTH);
CR_DECLARE (GOAL_SIDE_WIDTH);
CR_DECLARE (GOAL_SIDE_HEIGHT);
CR_DECLARE (TOP_GOAL_SIDE_POSITION);
CR_DECLARE (BOTTOM_GOAL_SIDE_POSITION);
CR_DECLARE (GOAL_BACK_LENGTH);
CR_DECLARE (GOAL_BACK_WIDTH);
CR_DECLARE (GOAL_BACK_HEIGHT);
CR_DECLARE (GOAL_BACK_POSITION);
#endif

CR_DECLARE (GROUNDMU);
CR_DECLARE (GROUNDMU2);
CR_DECLARE (GROUNDSLIP1);
CR_DECLARE (GROUNDSLIP2);
CR_DECLARE (GROUNDSOFTERP);
CR_DECLARE (GROUNDSOFTCFM);
CR_DECLARE (GROUNDBOUNCE);
CR_DECLARE (GROUNDBOUNCEVEL);
CR_DECLARE (GRAVITY);
CR_DECLARE (WORLDCFM);
CR_DECLARE (WORLDERP);


SimWorld::SimWorld(char* name) : Node (name)
{
	static bool isConfiged = false;

	if (!isConfiged)
	{
		isConfiged = true;
#ifndef NOENV

		CR_SETUP (SimWorld, LONG_LENGTH, CR_DOUBLE);
		CR_SETUP (SimWorld, LONG_WIDTH, CR_DOUBLE);
		CR_SETUP (SimWorld, LONG_HEIGHT, CR_DOUBLE);
		CR_SETUP (SimWorld, TOP_LONG_POSITION, CR_DOUBLE);
		CR_SETUP (SimWorld, BOTTOM_LONG_POSITION, CR_DOUBLE);
		CR_SETUP (SimWorld, SHORT_LENGTH, CR_DOUBLE);
		CR_SETUP (SimWorld, SHORT_WIDTH, CR_DOUBLE);
		CR_SETUP (SimWorld, SHORT_HEIGHT, CR_DOUBLE);
		CR_SETUP (SimWorld, TOP_SHORT_POSITION, CR_DOUBLE);
		CR_SETUP (SimWorld, BOTTOM_SHORT_POSITION, CR_DOUBLE);
		CR_SETUP (SimWorld, GOAL_SIDE_LENGTH, CR_DOUBLE);
		CR_SETUP (SimWorld, GOAL_SIDE_WIDTH, CR_DOUBLE);
		CR_SETUP (SimWorld, GOAL_SIDE_HEIGHT, CR_DOUBLE);
		CR_SETUP (SimWorld, TOP_GOAL_SIDE_POSITION, CR_DOUBLE);
		CR_SETUP (SimWorld, BOTTOM_GOAL_SIDE_POSITION, CR_DOUBLE);
		CR_SETUP (SimWorld, GOAL_BACK_LENGTH, CR_DOUBLE);
		CR_SETUP (SimWorld, GOAL_BACK_WIDTH, CR_DOUBLE);
		CR_SETUP (SimWorld, GOAL_BACK_HEIGHT, CR_DOUBLE);
		CR_SETUP (SimWorld, GOAL_BACK_POSITION, CR_DOUBLE);
#endif
    CR_SETUP (SimWorld, GROUNDMU, CR_DOUBLE);
    CR_SETUP (SimWorld, GROUNDMU2, CR_DOUBLE);
    CR_SETUP (SimWorld, GROUNDSLIP1, CR_DOUBLE);
    CR_SETUP (SimWorld, GROUNDSLIP2, CR_DOUBLE);
    CR_SETUP (SimWorld, GROUNDSOFTERP, CR_DOUBLE);
    CR_SETUP (SimWorld, GROUNDSOFTCFM, CR_DOUBLE);
    CR_SETUP (SimWorld, GROUNDBOUNCE, CR_DOUBLE);
    CR_SETUP (SimWorld, GROUNDBOUNCEVEL, CR_DOUBLE);

    CR_SETUP (SimWorld, GRAVITY, CR_DOUBLE);
    CR_SETUP (SimWorld, WORLDCFM, CR_DOUBLE);
    CR_SETUP (SimWorld, WORLDERP, CR_DOUBLE);

  }
  printf("World Gravity : %f\n",DVAR(GRAVITY));
  dWorldSetGravity (Simulator::getInstance()->getWorld(), 0, 0, DVAR(GRAVITY));
  printf("World Error Reduction Parameter : %f\n",DVAR(WORLDERP));
  dWorldSetERP (Simulator::getInstance()->getWorld(),DVAR(WORLDERP));
  printf("World Constraint Force Mixing : %f\n",DVAR(WORLDCFM));
  dWorldSetCFM (Simulator::getInstance()->getWorld(),DVAR(WORLDCFM));

	PlaneGeometry* plane = new PlaneGeometry (Vector3::UNIT_Z, 0, "Ground");
  Surface* groundSurface = new Surface ();
  dSurfaceParameters* sp;
  sp = &groundSurface->getSurfaceParameters ();
	sp->mode  = dContactMu2 | dContactSlip1 | dContactSlip2 | dContactSoftERP | dContactSoftCFM | dContactApprox1 |dContactBounce ;
  // dContactSlip1 | dContactSlip2 | dContactSoftCFM | dContactApprox1 | dContactFDir1 | dContactMu2 | ;
	sp->mu    = DVAR (GROUNDMU);
	sp->mu2   = DVAR (GROUNDMU2);
	sp->slip1 = DVAR (GROUNDSLIP1);
	sp->slip2 = DVAR (GROUNDSLIP2);
  sp->soft_erp = DVAR (GROUNDSOFTERP);
  sp->soft_cfm = DVAR (GROUNDSOFTCFM);
  sp->bounce=DVAR (GROUNDBOUNCE);
  sp->bounce_vel=DVAR(GROUNDBOUNCEVEL);
  groundCollider=new GeometryCollider ();
  groundCollider->setSurface (groundSurface);
  plane->setCollisionController (groundCollider);
	addChild (plane);
  printf("Ground Surface :\n");
  groundSurface->print();
  
#ifndef NOENV

	Geometry* topLong    = new BoxGeometry (DVAR (LONG_LENGTH), DVAR (LONG_WIDTH), DVAR (LONG_HEIGHT), "Top Long");
	Geometry* bottomLong = new BoxGeometry (DVAR (LONG_LENGTH), DVAR (LONG_WIDTH), DVAR (LONG_HEIGHT), "Bottom Long");

  Geometry* leftLong    = new BoxGeometry (DVAR (LONG_LENGTH), DVAR (LONG_WIDTH), DVAR (LONG_HEIGHT), "Left Long");
	Geometry* rightLong   = new BoxGeometry (DVAR (LONG_LENGTH), DVAR (LONG_WIDTH), DVAR (LONG_HEIGHT), "Right Long");

//  Geometry* leftTopShort    = new BoxGeometry (DVAR (SHORT_LENGTH), DVAR (SHORT_WIDTH), DVAR (SHORT_HEIGHT), "Left Top Short");
//	Geometry* leftBottomShort = new BoxGeometry (DVAR (SHORT_LENGTH), DVAR (SHORT_WIDTH), DVAR (SHORT_HEIGHT), "Left Bottom Short");

//	Geometry* rightTopShort    = new BoxGeometry (DVAR (SHORT_LENGTH), DVAR (SHORT_WIDTH), DVAR (SHORT_HEIGHT), "Right Top Short");
//	Geometry* rightBottomShort = new BoxGeometry (DVAR (SHORT_LENGTH), DVAR (SHORT_WIDTH), DVAR (SHORT_HEIGHT), "Right Bottom Short");

//	Geometry* leftTopGoalSide    = new BoxGeometry (DVAR (GOAL_SIDE_LENGTH), DVAR (GOAL_SIDE_WIDTH), DVAR (GOAL_SIDE_HEIGHT), "Left Top Goal Side");
//	Geometry* leftBottomGoalSide = new BoxGeometry (DVAR (GOAL_SIDE_LENGTH), DVAR (GOAL_SIDE_WIDTH), DVAR (GOAL_SIDE_HEIGHT), "Left Bottom Goal Side");

//	Geometry* rightTopGoalSide    = new BoxGeometry (DVAR (GOAL_SIDE_LENGTH), DVAR (GOAL_SIDE_WIDTH), DVAR (GOAL_SIDE_HEIGHT), "Right Top Goal Side");
//	Geometry* rightBottomGoalSide = new BoxGeometry (DVAR (GOAL_SIDE_LENGTH), DVAR (GOAL_SIDE_WIDTH), DVAR (GOAL_SIDE_HEIGHT), "Right Bottom Goal Side");

//	Geometry* leftGoalBack  = new BoxGeometry (DVAR (GOAL_BACK_LENGTH), DVAR (GOAL_BACK_WIDTH), DVAR (GOAL_BACK_HEIGHT), "Left Goal Back");

//	Geometry* rightGoalBack = new BoxGeometry (DVAR (GOAL_BACK_LENGTH), DVAR (GOAL_BACK_WIDTH), DVAR (GOAL_BACK_HEIGHT), "Right Goal Back");

	addChild (topLong);
	addChild (bottomLong);
  addChild (leftLong);
	addChild (rightLong);

//  addChild (leftTopShort);
//	addChild (leftBottomShort);
//	addChild (rightTopShort);
//	addChild (rightBottomShort);
//	addChild (leftTopGoalSide);
//	addChild (leftBottomGoalSide);
//	addChild (rightTopGoalSide);
//	addChild (rightBottomGoalSide);
//	addChild (leftGoalBack);
//	addChild (rightGoalBack);

	topLong->            setLocalPosition ( Vector3 (VDVAR (TOP_LONG_POSITION)));
	bottomLong->         setLocalPosition ( Vector3 (VDVAR (BOTTOM_LONG_POSITION)));
	leftLong->           setLocalPosition ( Vector3 (VDVAR (TOP_LONG_POSITION)));
	rightLong->          setLocalPosition ( Vector3 (VDVAR (BOTTOM_LONG_POSITION)));

//  leftTopShort->       setLocalPosition (-Vector3 (VDVAR (TOP_SHORT_POSITION)));
//	leftBottomShort->    setLocalPosition (-Vector3 (VDVAR (BOTTOM_SHORT_POSITION)));
//	rightTopShort->      setLocalPosition ( Vector3 (VDVAR (TOP_SHORT_POSITION)));
//	rightBottomShort->   setLocalPosition ( Vector3 (VDVAR (BOTTOM_SHORT_POSITION)));
//	leftTopGoalSide->    setLocalPosition (-Vector3 (VDVAR (TOP_GOAL_SIDE_POSITION)));
//	leftBottomGoalSide-> setLocalPosition (-Vector3 (VDVAR (BOTTOM_GOAL_SIDE_POSITION)));
//	rightTopGoalSide->   setLocalPosition ( Vector3 (VDVAR (TOP_GOAL_SIDE_POSITION)));
//	rightBottomGoalSide->setLocalPosition ( Vector3 (VDVAR (BOTTOM_GOAL_SIDE_POSITION)));
//	leftGoalBack->       setLocalPosition (-Vector3 (VDVAR (GOAL_BACK_POSITION)));
//	rightGoalBack->      setLocalPosition ( Vector3 (VDVAR (GOAL_BACK_POSITION)));

	topLong->         setLocalRotation (Matrix33::rotateAboutX (-Math::PI / 4));
	bottomLong->      setLocalRotation (Matrix33::rotateAboutX ( Math::PI / 4));
	leftLong->        setLocalRotation (Matrix33::rotateAboutY (-Math::PI / 4));
	rightLong->       setLocalRotation (Matrix33::rotateAboutY ( Math::PI / 4));

//  leftTopShort->    setLocalRotation (Matrix33::rotateAboutY (-Math::PI / 4));
//	leftBottomShort-> setLocalRotation (Matrix33::rotateAboutY (-Math::PI / 4));
//	rightTopShort->   setLocalRotation (Matrix33::rotateAboutY ( Math::PI / 4));
//	rightBottomShort->setLocalRotation (Matrix33::rotateAboutY ( Math::PI / 4));
#endif
}

SimWorld::~SimWorld()
{
}
void SimWorld::print ()
{
  printf ("SimWorld %s\n",name);
    printf ("UBERSIM LP: %f %f %f WP: %f %f %f\n",
          getLocalPosition().x,
          getLocalPosition().y,
          getLocalPosition().z,
          getWorldPosition().x,
          getWorldPosition().y,
          getWorldPosition().z
  );
 

  for (unsigned int i = 0; i < children.size (); i++)
	{
		Spatial* spatial = children[i];

		if (spatial)
		{
			spatial->print ();
		}
	}

}  

