/*=========================================================================
	UberSim Source Code Release
	-------------------------------------------------------------------------
	Copyright (C) 2002 Manuela Veloso, Brett Browning, Mike Bowling,
	James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
	Erick Tryzelaar {erickt}@andrew.cmu.edu
	School of Computer Science, Carnegie Mellon University
	-------------------------------------------------------------------------
	This software is distributed under the GNU General Public License,
	version 2.  If you do not have a copy of this licence, visit
	www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
	Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
	in the hope that it will be useful, but WITHOUT ANY WARRANTY,
	including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	-------------------------------------------------------------------------*/

#include <stdio.h>

#include "mmgr.h"

#include "Engine.h"
#include "Entity.h"
#include "Event.h"
#include "Spatial.h"
#include "Timer.h"
/**
  * changes :
  *  - 22/6/03 MD Rien !
  *  - 17/07/03 MD&YLG ClearAllEvent
  *  - 17/07/03 MD
  *  - 11/09/03 addEvent, queuesize,print
  */

Timer* Timer::getInstance ()
{
	static Timer instance;

	return &instance;
}

Timer::Timer  ()
{
	isInitialized = false;
	isShutdown    = true;
  qs=0;
}

Timer::~Timer ()
{
}

void Timer::initialize ()
{
	if (!isInitialized)
	{
		isInitialized = true;
		isShutdown    = false;
	}
}

void Timer::shutdown ()
{
	if (!isShutdown)
	{
		isShutdown    = true;

		clearEvents ();
		
		isInitialized = false;
	}
}

void Timer::addEvent (Entity* entity, Event* event, Real deltaTime, int ID,unsigned int syncIndex)
{
  //printf("entity: %i \n",entity);
  //entity->print();
  static int maxID=-1;
  if (ID==-1)
    event->ID=++maxID;
  else
    event->ID=ID;
  event->syncIndex=syncIndex;       


  Engine* engine = Engine::getInstance ();
  #ifndef TIMERDYN
  Timer::Node node=Timer::Node (entity, event, deltaTime );
  if (ID==-1)
    node=Timer::Node (entity, event, deltaTime + (engine->getFrameCount ()) / engine->getTargetFps ());
  
    
  #else
  Timer::Node * node;
  if (ID==-1)
    node=new Timer::Node (entity, event, deltaTime + (engine->getFrameCount ()) / engine->getTargetFps ());
  else
    node=new Timer::Node (entity, event, deltaTime);

  #endif

	queue.push (node);
  //node.print();
  this->qs++;
//  int qsize=qs;
//  printf("addEvent (ID=%i): queueSize()=%i\n", event->ID, this->queueSize());
//  this->print();
}                     


void Timer::clearEvents(Entity* entity)
{
//  printf("QueueSize()=%i\n", this->queueSize());
//  this->print();
//  printf("ClearEvents(Entity):\n" );
  
  #ifndef TIMERDYN
		std::priority_queue<Node, std::vector<Node>, std::greater<Node> > auxqueue;
  #else
		std::priority_queue<Node *, std::vector<Node *>, std::greater<Node *> > auxqueue;
  #endif

	while (!queue.empty ())
	{
    #ifndef TIMERDYN
    const Node& node = queue.top ();
    if (node.entity != entity) {
      auxqueue.push (Timer::Node (node.entity, node.event, node.endTime));
    } else {
  		delete node.event;
      qs--;
    }
    #else
    const Node * node = queue.top ();
    if (node->entity != entity) {
      auxqueue.push (new Timer::Node (node->entity, node->event, node->endTime));
    } else {
  		delete node->event;
      qs--;
    }
    delete node;
    #endif
		queue.pop ();
	}
  queue = auxqueue;
//  printf("clearEvent(Entity): queueSize()=%i\n", this->queueSize()); 
//  this->print();
  
}

int Timer::queueSize()
{
  int s=0;
  #ifndef TIMERDYN
		std::priority_queue<Node, std::vector<Node>, std::greater<Node> > auxqueue;
  #else
		std::priority_queue<Node *, std::vector<Node *>, std::greater<Node *> > auxqueue;
  #endif
	while (!queue.empty ())
	{
    #ifndef TIMERDYN
    const Node& node = queue.top ();
    auxqueue.push (Timer::Node (node.entity, node.event, node.endTime));
    #else
    const Node * node = queue.top ();
    auxqueue.push (new Timer::Node (node->entity, node->event, node->endTime));
    delete node;
    #endif    
    s++;
		queue.pop ();
	}
  queue = auxqueue;
  return s;
}

bool Timer::isEmpty(unsigned int syncIndex)
{
  bool res=true;
  #ifndef TIMERDYN
		std::priority_queue<Node, std::vector<Node>, std::greater<Node> > auxqueue;
  #else
		std::priority_queue<Node *, std::vector<Node *>, std::greater<Node *> > auxqueue;
  #endif
	while (!queue.empty ())
	{
    #ifndef TIMERDYN
    const Node& node = queue.top ();
//    printf("syncIndex: %i\n",node.event->syncIndex);
    if (node.event->syncIndex==syncIndex)
      res=false;
    auxqueue.push (Timer::Node (node.entity, node.event, node.endTime));
    #else
    const Node * node = queue.top ();
    if (node->event->syncIndex==syncIndex)
      res=false;
    auxqueue.push (new Timer::Node (node->entity, node->event, node->endTime));
    delete node;
    #endif
		queue.pop ();
	}
  queue = auxqueue;
  return res;
}
void Timer::print()
{
  #ifndef TIMERDYN
		std::priority_queue<Node, std::vector<Node>, std::greater<Node> > auxqueue;
  #else
		std::priority_queue<Node *, std::vector<Node *>, std::greater<Node *> > auxqueue;
  #endif
	while (!queue.empty ())
	{
    #ifndef TIMERDYN
    const Node& node = queue.top ();
    ServoMotorTargetPositionEvent * stpe=(ServoMotorTargetPositionEvent *) (node.event);
    if (stpe)
        printf("Event : ID=%i targetPosition=%i End Time=%f\n",node.event->ID,stpe->targetPosition,node.endTime);
    else
      printf("Event : ID=%i End Time=%f\n",node.event->ID,node.endTime);
    node.entity->print();
    auxqueue.push (Timer::Node (node.entity, node.event, node.endTime));
    #else
    const Node* node = queue.top ();
    ServoMotorTargetPositionEvent stpe=dynamic_cast <ServoMotorTargetPositionEvent * (node->event);
    if (stpe)
        printf("Event : ID=%i targetPosition=%i End Time=%f\n",node->event->ID,stpe->targetPosition,node->endTime);
    else
      printf("Event : ID=%i End Time=%f\n",node->event->ID,node->endTime);
    node->entity->print();
    auxqueue.push (new Timer::Node (node->entity, node->event, node->endTime));
    delete node;
    #endif	  
    queue.pop ();
	}
  queue = auxqueue;
}

void Timer::clearEvents ()
{	
	while (!queue.empty ())
	{
    #ifndef TIMERDYN
    const Node& node = queue.top ();
		delete node.event;
    #else
    const Node * node = queue.top ();
		delete node->event;
    delete node;
    #endif		
		queue.pop ();
    qs--;
	}
//  printf("clearEvent: queueSize()=%i\n", this->queueSize());
}

void Timer::beginFrame ()
{
}

void Timer::frame ()
{
  #ifndef TIMERDYN
		std::priority_queue<Node, std::vector<Node>, std::greater<Node> > auxqueue;
  #else
		std::priority_queue<Node *, std::vector<Node *>, std::greater<Node *> > auxqueue;
  #endif
	std::vector<Entity * > hasHandled;
  int queuesize = 0;
  if (!queue.empty ())
	{
		Engine* engine = Engine::getInstance ();
		
		Real currentTime = engine->getFrameCount() / engine->getTargetFps ();
//    printf("****** Frame : currentTime=%f ******\n",currentTime);
    int i=0;
    
   #ifndef TIMERDYN
    while (!queue.empty () && queue.top ().endTime <= currentTime)
		{
			const Node& node = queue.top ();
	
      // on ne traite que la premiere occurence de entity
      if (node.entity->hasHandledEvent==false)
      {
        node.entity->hasHandledEvent=true;
        hasHandled.push_back (node.entity);
        node.entity->handleEvent (node.event);
        // if (node.event) {
        //   auxqueue.push (Timer::Node (node.entity, node.event, node.endTime));
        // }
        qs--;
      }
      // ce sera pour la prochaine fois !
      else {
        auxqueue.push (Timer::Node (node.entity, node.event, node.endTime));			
        queuesize++;
      }
   #else
    while (!queue.empty () && queue.top ()->endTime <= currentTime)
		{
			const Node* node = queue.top ();

      // on ne traite que la premiere occurence de entity
      if (node->entity->hasHandledEvent==false)
      {
        node->entity->hasHandledEvent=true;
        hasHandled.push_back (node->entity);
        node->entity->handleEvent (node->event);
        // if (node.event) {
        //   auxqueue.push (Timer::Node (node.entity, node.event, node.endTime));
        // }
        qs--;
      }
      // ce sera pour la prochaine fois !
      else {
        auxqueue.push (new Timer::Node (node->entity, node->event, node->endTime));
        queuesize++;
      }
      delete node;
   #endif

      queue.pop ();
      i++;
		}
    //on met aussi les evenements pas a echeance !
    while (!queue.empty ())
		{
    #ifndef TIMERDYN
      const Node& node = queue.top ();
      auxqueue.push (Timer::Node (node.entity, node.event, node.endTime));
    #else
      const Node* node = queue.top ();
      auxqueue.push (new Timer::Node (node->entity, node->event, node->endTime));
      delete node;
    #endif

      queue.pop ();
      queuesize++;
		}
    //printf("\n queuesize : %i\n",queuesize);
    //printf("\n qs        : %i\n",qs);
    //On affecte les evenement a traiter pour la prochaine fois
    queue=auxqueue;
    //this->print();
    //on remet a false les entites traites !
    for (unsigned int i=0; i<hasHandled.size();i++)
      hasHandled[i]->hasHandledEvent=false;
    //on vide le vector
    hasHandled.clear();
	}
}

void Timer::endFrame ()
{
}

Timer::Node::Node (Entity* entity, Event* event, Real endTime)
{
	this->entity  = entity;
	this->event   = event;
	this->endTime = endTime;
}

void Timer::Node::print ()
{
  printf("Timer::Node eventID=%i endTime=%f\n",this->event->ID,this->endTime);
  this->entity->print();
}

#ifndef TIMERDYN

bool Timer::Node::operator > (const Timer::Node& node) const
{
  if (endTime==node.endTime)
  {
    return event->ID>node.event->ID;
  }  
  return endTime > node.endTime;
}
#else
bool Timer::Node::operator > (const Timer::Node* node) const
{
  if (endTime==node->endTime)
  {
    return event->ID>node->event->ID;
  }
  return endTime > node->endTime;
}

#endif
