/*=========================================================================
	UberSim Source Code Release
	-------------------------------------------------------------------------
	Copyright (C) 2002 Manuela Veloso, Brett Browning, Mike Bowling,
	James Bruce; {mmv, brettb, mhb, jbruce}@cs.cmu.edu
	Erick Tryzelaar {erickt}@andrew.cmu.edu
	School of Computer Science, Carnegie Mellon University
	-------------------------------------------------------------------------
	This software is distributed under the GNU General Public License,
	version 2.  If you do not have a copy of this licence, visit
	www.gnu.org, or write: Free Software Foundation, 59 Temple Place,
	Suite 330 Boston, MA 02111-1307 USA.  This program is distributed
	in the hope that it will be useful, but WITHOUT ANY WARRANTY,
	including MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	-------------------------------------------------------------------------*/
/**
  * changes :
  *  - 08/06/03 [N] creation/destruction d'un atome.
  *             [B] information sur la scene.
  *             [G] creation/destruction de groupe d'atomes
  *             Elimination de gadget d'UBERSIM
  */
/*
   Touches disponibles pour le test :
   ----------------------------------
   La touche N permet d'ajouter ou d'enlever un atome de l'environnement.
   La touche G permet d'ajouter ou d'enlever un groupe d'atome de l'environnement.
   La touche P active/desactive la pause (synchronisation des commandes !)
   Les touches 0 a 5 permettent d'activer/desactiver les pattes.
   La touche 6 permet d'inverser l'activation des pattes.
   La touche 7 active toutes les pattes.
   La touche 8 desactive toutes les pattes.
   La touche M bascule le mode d'ordonnancement (KEEP/REPLACE).
   La commande S envoie l'ordre Stop aux pattes actives.
   La commande H envoie l'ordre de centrage des pattes actives.
   La commande O envoie l'ordre d'oscillation aux pattes actives (les 2 axes, dans un sens puis dans l'autre).
   La commande A envoie l'ordre de positionnement vertical au maximum aux pattes actives.
   La commande Q envoie l'ordre de positionnement vertical au centre aux pattes actives.
   La commande W envoie l'ordre de positionnement vertical au minimum aux pattes actives.
   La commande X envoie l'ordre de positionnement horizontal au maximum aux pattes actives.
   La commande C envoie l'ordre de positionnement horizontal au centre aux pattes actives.
   La commande V envoie l'ordre de positionnement horizontal au minimum aux pattes actives.
   La commande D permet d'afficher la distances des atomes par rapport a une position de reference.
   La commande R permet de fixer la position de reference des atomes a leur position courante.
   
*/
#include <string>

#include <drawstuff/drawstuff.h>
#include <ode/ode.h>

#include <time.h>
#include <sys/time.h>

#include "ConfigReader.h"

#include "mmgr.h"

#include "CollisionGroup.h"
#include "CollisionManager.h"
#include "Engine.h"
#include "Entity.h"
#include "Event.h"
#include "SimWorld.h"
#include "Math.h"
#include "RigidBody.h"
#include "HingeJoint.h"
#include "AMotorJoint.h"
#include "SimServoMotor.h"
#include "Simulator.h"
#include "Timer.h"
#include "Atom.h"
#include "Graph.h"

/*******************************/

CR_DECLARE (TARGET_FPS);
CR_DECLARE (GROUP_POSITION);
CR_DECLARE (ATOM_POSITION);
CR_DECLARE (GROUP_ATOM_COUNT);
CR_DECLARE (GROUP_ATOM_PER_ROW);
CR_DECLARE (GROUP_ATOM_PER_LINE);
CR_DECLARE (KERNELSIDE);
CR_DECLARE (SPHERESIDE);
CR_DECLARE (LEGRADIUS);
CR_DECLARE (LEGSIDE);


Engine*         engine         = 0;
CollisionGroup* collisionGroup = 0;

#define NMAX          (125)   // max number of atoms in the simulation

API::Atom * atom = 0;
API::Atom * atoms[NMAX];
dJointID atomsFixedWorld[NMAX];
bool activeAtom[NMAX];
bool activeLeg[6]={true,true,true,true,true,true};
int mode;

bool isShowingFps = false;
int locked;
dJointID atomFixedWorld=0;

bool atomsAreCreated=false;
bool atomFixedJointIsCreated=false;
bool atomsFixedJointsAreCreated=false;
unsigned int ATOMCOUNT=0;

void step(int pause)
{
	engine->beginFrame ();

	if (!engine->getIsSkippingFrame () && (!engine->getIsPaused () || engine->getIsTakingOneStep ()))
	{
		if (isShowingFps)  {
			printf ("\nframe: %d fps: %f time: %f\n", engine->getFrameCount (), engine->getFps (), engine->getFrameCount () / engine->getTargetFps ());
		}
	}

	engine->frame      ();
	engine->endFrame   ();


	if (!engine->getIsSkippingFrame () && (!engine->getIsPaused () || engine->getIsTakingOneStep ()))
	{

	}
}

/*******************************/
void showActive()
{

  printf ("***********************\n");
  for(unsigned int i=0;i<6;i++)
  {
    printf ("Leg n=%i (",i);
    switch (i) {
      case 0 : printf ("rouge, devant");break; // red
      case 1 : printf ("vert, derriere");break; // green
      case 2 : printf ("bleu fonce, bas");break; // dark blue
      case 3 : printf ("jaune, haut");break; // yellow
      case 4 : printf ("bleu clair, droite");break; // light blue
      case 5 : printf ("violet, gauche");break; // pink
    }
    printf (") ");    
    if (activeLeg[i]==true)
      printf ("activee.\n");
    else
      printf ("desactivee.\n");
  }
  if (mode==KEEP)
    printf("Mode: KEEP\n");
  else
    printf("Mode: REPLACE\n");

  if (atomsAreCreated)
    for(unsigned int i=0;i<ATOMCOUNT;i++)
    {
      printf ("Atoms[%i] ",i);
      if (activeAtom[i]==true)
        printf ("active.\n");
      else
        printf ("desactive.\n");
    }
  if (atomsAreCreated)
  {
    printf("Nombre d'ensemble de synchronisation : %i\n", atoms[0]->getMolecule()->getSyncIndexCount());
    for(unsigned int i=0;i<atoms[0]->getMolecule()->getSyncIndexCount();i++)
    {
      printf("Ensemble de synchronisation[%i] indexe par %i :",i,atoms[0]->getMolecule()->getSyncIndex(i));
      if (atoms[0]->getMolecule()->isFinished(atoms[0]->getMolecule()->getSyncIndex(i))==true)
        printf ("termine\n");
      else
        printf ("non termine\n");
    }
  }  
}
void command (int cmd)
{
  
  if (ATOMCOUNT==0)
  ATOMCOUNT=IVAR(GROUP_ATOM_COUNT);
  
  if (ATOMCOUNT>NMAX)
    ATOMCOUNT=NMAX;    
  static unsigned int AROW=IVAR(GROUP_ATOM_PER_ROW);
  static unsigned int ALINE=IVAR(GROUP_ATOM_PER_LINE);
  static unsigned int LINECOUNT=ATOMCOUNT/ALINE;
  if (ATOMCOUNT % ALINE!=0)
    LINECOUNT++;
  static unsigned int ROWCOUNT=Math::min(ALINE,ATOMCOUNT)/AROW;
  if ((int) Math::min(ALINE,ATOMCOUNT) % AROW!=0)
    LINECOUNT++;
  if (ATOMCOUNT<ALINE)
    LINECOUNT=1;
  if (ATOMCOUNT<AROW)
    ROWCOUNT=1;
  static double atomsPosX[NMAX];
  static double atomsPosY[NMAX];
  static double atomsPosZ[NMAX];
  static double atomPosX=0;
  static double atomPosY=0;
  static double atomPosZ=0;
  
  if (atom) {
		switch (cmd) {
			case 'w':
			case 'W':
				break;
			case 'a':
			case 'A':
				{
				}
				break;
			case 'd':
			case 'D':
				{
				}
				break;
			case 'q':
			case 'Q':
				{
				}
				break;
			case 'e':
			case 'E':
				{
				}
				break;
			case 't':
			case 'T':
				{
				}
				break;
			case 'z':
			case 'Z':
				{
				}
				break;
			case 'y':
			case 'Y':
				{
				}
				break;
			case 'x':
			case 'X':
				{
				}
				break;
			case 'c':
			case 'C':
				{
				}
				break;
		}
	}

	switch (cmd) {
    case 'u':
	  case 'U':
			{
        if (locked)
        {

            locked=0;
            if ((atom) && (atomFixedJointIsCreated))
            {
              dJointDestroy(atomFixedWorld);
              atomFixedJointIsCreated=false;
              printf("**Atome decroche de l'environnement**\n");
            }  
            if ((atomsAreCreated) && (atomsFixedJointsAreCreated))
            {
              for(unsigned int i=0;i<ATOMCOUNT;i++)
              {
                if (atomsFixedWorld[i])
                {
                  dJointDestroy(atomsFixedWorld[i]);
                  printf("**Atoms[%i] decroche de l'environnement**\n",i);
                }
                else
                {
                  printf("**Atoms[%i] non decroche de l'environnement**\n",i);
                }
              }
              atomsFixedJointsAreCreated=false;

            }
        }
        else
        {
          locked=1;
          if (atom)
          {
              atomFixedWorld= dJointCreateFixed (Simulator::getInstance ()->getWorld (), 0);
              dJointAttach (atomFixedWorld,0, atom->util->getSimulatedObject()->getKernel()->getBody());
              dJointSetFixed (atomFixedWorld);
              atomFixedJointIsCreated=true;
              printf("**Atome accroche a l'environnement**\n");
          }
          if (atomsAreCreated)
          {
            for(unsigned int i=0;i<ATOMCOUNT;i++)
            {
              atomsFixedWorld[i]= dJointCreateFixed (Simulator::getInstance ()->getWorld (), 0);
              dJointAttach (atomsFixedWorld[i],0, atoms[i]->util->getSimulatedObject()->getKernel()->getBody());
              dJointSetFixed (atomsFixedWorld[i]);
              printf("**Atoms[%i] accroche a l'environnement**\n",i);
            }
            atomsFixedJointsAreCreated=true;
          }
        }
			}
			break;

    case 'p':
		case 'P':
			{
				engine->setIsPaused (!engine->getIsPaused ());
        if (engine->getIsPaused ())
          printf("**PAUSE**\n");
        else
          printf("**NO PAUSE**\n");

			}
			break;
		case 'o':
		case 'O':
			{
        /*
				engine->setIsPaused        (true);
				engine->setIsTakingOneStep (true);
        */
            if (atom)
            {
              for(unsigned int i=0;i<6;i++) {
                if (activeLeg[i]) {
                  atom->getLeg(i)->getServoMotor(0)->setTargetPosition(0,mode);
                  atom->getLeg(i)->getServoMotor(0)->setTargetPosition(250,mode);
                  atom->getLeg(i)->getServoMotor(1)->setTargetPosition(0,KEEP);
                  atom->getLeg(i)->getServoMotor(1)->setTargetPosition(250,KEEP);
                }
              }
            }
            if (atomsAreCreated)
            {
              for(unsigned int j=0;j<ATOMCOUNT;j++)
                if (activeAtom[j])
                  for(unsigned int i=0;i<6;i++) {
                    if (activeLeg[i]) {
                      atoms[j]->getLeg(i)->getServoMotor(0)->setTargetPosition(0,mode);
                      atoms[j]->getLeg(i)->getServoMotor(0)->setTargetPosition(250,mode);
                      atoms[j]->getLeg(i)->getServoMotor(1)->setTargetPosition(0,KEEP);
                      atoms[j]->getLeg(i)->getServoMotor(1)->setTargetPosition(250,KEEP);
                    }
                  }

            }
        
			}
			break;
		case 'd':
		case 'D':
			{
        if (atom)
        {
          printf("Atom : dx=%f dy=%f dz=%f\n",atom->util->getCartesianPosition().getX()-atomPosX,atom->util->getCartesianPosition().getY()-atomPosY,atom->util->getCartesianPosition().getZ()-atomPosZ);
          printf("Atom : D=%f\n",Math::sqrt(
            (atom->util->getCartesianPosition().getX()-atomPosX)*(atom->util->getCartesianPosition().getX()-atomPosX)
            +(atom->util->getCartesianPosition().getY()-atomPosY)*(atom->util->getCartesianPosition().getY()-atomPosY)
            +(atom->util->getCartesianPosition().getZ()-atomPosZ)*(atom->util->getCartesianPosition().getZ()-atomPosZ)
            )
            );

        }
        if (atomsAreCreated)
          for(unsigned int i=0;i<ATOMCOUNT;i++)
          {
            printf("Atoms[%i] : dx=%f dy=%f dz=%f\n",i,atoms[i]->util->getCartesianPosition().getX()-atomsPosX[i],atoms[i]->util->getCartesianPosition().getY()-atomsPosY[i],atoms[i]->util->getCartesianPosition().getZ()-atomsPosZ[i]);
            printf("Atoms[%i] : D=%f\n",i,Math::sqrt(
            (atoms[i]->util->getCartesianPosition().getX()-atomsPosX[i])*(atoms[i]->util->getCartesianPosition().getX()-atomsPosX[i])
            +(atoms[i]->util->getCartesianPosition().getY()-atomsPosY[i])*(atoms[i]->util->getCartesianPosition().getY()-atomsPosY[i])
            +(atoms[i]->util->getCartesianPosition().getZ()-atomsPosZ[i])*(atoms[i]->util->getCartesianPosition().getZ()-atomsPosZ[i])
            )
            );
          }
      }
			break;

    case 'r':
		case 'R':
			{
        if (atom) {
          atomPosX=atom->util->getCartesianPosition().getX();
          atomPosY=atom->util->getCartesianPosition().getY();
          atomPosZ=atom->util->getCartesianPosition().getZ();
          printf("Atom : Position referencee.\n");
          printf("Atom : Position referencee : x=%f y=%f z=%f\n",atom->util->getCartesianPosition().getX(),atom->util->getCartesianPosition().getY(),atom->util->getCartesianPosition().getZ());

        }
        if (atomsAreCreated)
          for(unsigned int i=0;i<ATOMCOUNT;i++)
          {            
            atomsPosX[i]=atoms[i]->util->getCartesianPosition().getX();
            atomsPosY[i]=atoms[i]->util->getCartesianPosition().getY();
            atomsPosZ[i]=atoms[i]->util->getCartesianPosition().getZ();
            printf("Atoms[%i] : Position referencee : x=%f y=%f z=%f\n",i,atoms[i]->util->getCartesianPosition().getX(),atoms[i]->util->getCartesianPosition().getY(),atoms[i]->util->getCartesianPosition().getZ());            
          }
      }
			break;

      /*******
      *
      ********/
		case 'n':
		case 'N':
			{
//				Timer::getInstance ()->clearEvents ();

				if (atom) {
					delete atom;
					atom = 0;
				} else {
					 atom = new API::Atom ();
           atom->util->setCartesianPosition(new API::CartesianPosition(
           Vector3(VDVAR (ATOM_POSITION)).x,
           Vector3(VDVAR (ATOM_POSITION)).y,
           Vector3(VDVAR (ATOM_POSITION)).z));
           int i;
           /*
           for(i=0;i<6;i++) {
             atom->getLeg(i)->getServoMotor(0)->setTargetPosition(0,mode);
             atom->getLeg(i)->getServoMotor(0)->setTargetPosition(250,mode);
             atom->getLeg(i)->getServoMotor(1)->setTargetPosition(0,mode);
             atom->getLeg(i)->getServoMotor(1)->setTargetPosition(250,mode);
             atom->getLeg(i)->getServoMotor(0)->setTargetPosition(125,mode);
             atom->getLeg(i)->getServoMotor(1)->setTargetPosition(125,mode);
             
             
           }
           */
           //SimRigidBody * tmp= dynamic_cast<SimRigidBody *> (atom->util->getSimulatedObject()->getMotionController());
           //tmp->getGraph()->print();
           /*
           printf ("\nPE abs: %f %f %f \n",Vector3(VDVAR (ATOM_POSITION)).x, Vector3(VDVAR (ATOM_POSITION)).y,Vector3(VDVAR (ATOM_POSITION)).z );
           printf ( "\nSM0 min:%i max:%i half:%i low:%f high:%f \n",
                      atom->getLeg(0)->getServoMotor(0)->getSimulatedObject()->getMinPosition(),
                      atom->getLeg(0)->getServoMotor(0)->getSimulatedObject()->getMaxPosition(),        
                      atom->getLeg(0)->getServoMotor(0)->getSimulatedObject()->getHalfPosition(),
                      atom->getLeg(0)->getServoMotor(0)->getSimulatedObject()->getLowStopAngle(),
                      atom->getLeg(0)->getServoMotor(0)->getSimulatedObject()->getHighStopAngle());
           printf ( "SM0\n Pos=0 Angle=%f\n Pos=250 Angle=%f\n Pos=125 Angle=%f\n",
           atom->getLeg(0)->getServoMotor(0)->util->getSimulatedObject()->Position2Angle(0),
           atom->getLeg(0)->getServoMotor(0)->util->getSimulatedObject()->Position2Angle(250),
           atom->getLeg(0)->getServoMotor(0)->util->getSimulatedObject()->Position2Angle(125));           
           printf ( "SM0\n Angle=0 Pos=%i\n Angle=-PI/4 Pos=%i\n Angle=+PI/4 Pos=%i\n",
           atom->getLeg(0)->getServoMotor(0)->util->getSimulatedObject()->Angle2Position(0),
           atom->getLeg(0)->getServoMotor(0)->util->getSimulatedObject()->Angle2Position(-M_PI_4),
           atom->getLeg(0)->getServoMotor(0)->util->getSimulatedObject()->Angle2Position(+M_PI_4));
           */
           atomPosX=atom->util->getCartesianPosition().getX();
           atomPosY=atom->util->getCartesianPosition().getY();
           atomPosZ=atom->util->getCartesianPosition().getZ();

        }
			}
			break;
    case 'B':
    case 'b':
      {
          if (atom) {
              for(unsigned int i=0;i<6;i++)
                printf("Atom, Patte[%i]: %f, %f, %f : %i\n",i, atom->getLeg(i)->util->getSimulatedObject()->getAbsPosEnd().x,atom->getLeg(i)->util->getSimulatedObject()->getAbsPosEnd().y,atom->getLeg(i)->util->getSimulatedObject()->getAbsPosEnd().z,atom->getLeg(i)->util->isOnGround());
                     
            
          }
          if (atomsAreCreated) {
                     printf("Atom3: %f, %f, %f\n", atoms[3]->getLeg(1)->util->getSimulatedObject()->getAbsPosEnd().x,atoms[3]->getLeg(1)->util->getSimulatedObject()->getAbsPosEnd().y,atoms[3]->getLeg(1)->util->getSimulatedObject()->getAbsPosEnd().z);
                     printf("Atom0: %f, %f, %f\n", atoms[0]->getLeg(1)->util->getSimulatedObject()->getAbsPosEnd().x,atoms[0]->getLeg(1)->util->getSimulatedObject()->getAbsPosEnd().y,atoms[0]->getLeg(1)->util->getSimulatedObject()->getAbsPosEnd().z);
                     printf ("Diff: %f, %f, %f\n", atoms[3]->getLeg(1)->util->getSimulatedObject()->getAbsPosEnd().x-atoms[0]->getLeg(1)->util->getSimulatedObject()->getAbsPosEnd().x,atoms[3]->getLeg(1)->util->getSimulatedObject()->getAbsPosEnd().y-atoms[0]->getLeg(1)->util->getSimulatedObject()->getAbsPosEnd().y,atoms[3]->getLeg(1)->util->getSimulatedObject()->getAbsPosEnd().z-atoms[0]->getLeg(1)->util->getSimulatedObject()->getAbsPosEnd().z);
                     
                     //if (atoms[3]->getLeg(1)->util->isConnectable(atoms[0]->getLeg(1)))
                        atoms[3]->getLeg(1)->setLink(atoms[0]->getLeg(1));
                        atoms[7]->getLeg(0)->setLink(atoms[2]->getLeg(0));
                        atoms[3+8]->getLeg(1)->setLink(atoms[0+8]->getLeg(1));
                        atoms[7+8]->getLeg(0)->setLink(atoms[2+8]->getLeg(0));
          }
          
      }
			break;

    case 'z':
    case 'Z':
      {

				if (atomsAreCreated) {
//          for(unsigned int i=0;i<16;i++)
          for(unsigned int i=0;i<8;i++)

             if (atoms[i])
             {
					      delete atoms[i];
                atoms[i]=0;
              }
          atomsAreCreated=false;
				} else {
      // create a raw/cube of ATOMCOUNT interconnected atoms
          for(unsigned int i=0;i<3;i++) {
            atoms[i] = new API::Atom ();
            atoms[i]->util->setCartesianPosition(
                  new API::CartesianPosition(
                      Vector3(VDVAR (GROUP_POSITION)).x+
                      (i+1)*(DVAR(KERNELSIDE)+2*DVAR(LEGSIDE)),
                      Vector3(VDVAR (GROUP_POSITION)).y,
                      Vector3(VDVAR (GROUP_POSITION)).z    
                      ));
                      if ((i>0) && (atoms[i-1]->getLeg(0)->util->isConnectable(atoms[i]->getLeg(1))))
                        atoms[i-1]->getLeg(0)->setLink(atoms[i]->getLeg(1));
          
            
          };
          
          for(unsigned int i=0;i<5;i++) {
            atoms[i+3] = new API::Atom ();
            atoms[i+3]->util->setCartesianPosition(
                  new API::CartesianPosition(
                      Vector3(VDVAR (GROUP_POSITION)).x+
                      i *(DVAR(KERNELSIDE)+2*DVAR(LEGSIDE)),
                      Vector3(VDVAR (GROUP_POSITION)).y,
                      Vector3(VDVAR (GROUP_POSITION)).z+
                      2.828427*DVAR(LEGSIDE)+DVAR(KERNELSIDE)
                      ))
                      ;
                      if ((i>0) && (atoms[i+3-1]->getLeg(0)->util->isConnectable(atoms[i+3]->getLeg(1))))
                        atoms[i+3-1]->getLeg(0)->setLink(atoms[i+3]->getLeg(1));

          };
          for(unsigned int i=0;i<3;i++) {
            atoms[i+8] = new API::Atom ();
            atoms[i+8]->util->setCartesianPosition(
                  new API::CartesianPosition(
                      Vector3(VDVAR (GROUP_POSITION)).x+
                      (i+1)*(DVAR(KERNELSIDE)+2*DVAR(LEGSIDE)),
                      Vector3(VDVAR (GROUP_POSITION)).y+
                      2*DVAR(LEGSIDE)+DVAR(KERNELSIDE),
                      Vector3(VDVAR (GROUP_POSITION)).z
                      ));
                      if ((i>0) && (atoms[i+8-1]->getLeg(0)->util->isConnectable(atoms[i+8]->getLeg(1))))
                        atoms[i+8-1]->getLeg(0)->setLink(atoms[i+8]->getLeg(1));


          };

          for(unsigned int i=0;i<5;i++) {
            atoms[i+8+3] = new API::Atom ();
            atoms[i+8+3]->util->setCartesianPosition(
                  new API::CartesianPosition(
                      Vector3(VDVAR (GROUP_POSITION)).x+
                      i *(DVAR(KERNELSIDE)+2*DVAR(LEGSIDE)),
                      Vector3(VDVAR (GROUP_POSITION)).y+
                      2*DVAR(LEGSIDE)+DVAR(KERNELSIDE),
                      Vector3(VDVAR (GROUP_POSITION)).z+
                      2.828427*DVAR(LEGSIDE)+DVAR(KERNELSIDE)
                      ))
                      ;
                      if ((i>0) && (atoms[i+8+3-1]->getLeg(0)->util->isConnectable(atoms[i+8+3]->getLeg(1))))
                        atoms[i+8+3-1]->getLeg(0)->setLink(atoms[i+8+3]->getLeg(1));

          };
          

          for(unsigned int i=0;i<8;i++) {
                      if (atoms[i]->getLeg(4)->util->isConnectable(atoms[i+8]->getLeg(5)))
                        atoms[i]->getLeg(4)->setLink(atoms[i+8]->getLeg(5));

          }
                              
            for(unsigned int i=0;i<16;i++)
            {
              if ((i!=3) && (i!=7) && (i!=3+8) && (i!=7+8))
              {
              atomsFixedWorld[i]= dJointCreateFixed (Simulator::getInstance ()->getWorld (), 0);
              dJointAttach (atomsFixedWorld[i],0, atoms[i]->util->getSimulatedObject()->getKernel()->getBody());
              dJointSetFixed (atomsFixedWorld[i]);
              printf("**Atoms[%i] accroche a l'environnement**:%i\n",i,atomsFixedWorld[i]);
              }
            }
            locked=1;
            atomsFixedJointsAreCreated=true;
    	   ATOMCOUNT=16;      
//    	   ATOMCOUNT=8;
          atomsAreCreated=true;
          unsigned int synchro=atoms[0]->getMolecule()->getNextSyncIndex();
          atoms[3]->getLeg(0)->getServoMotor(0)->setTargetPosition(0,REPLACE,0,synchro);
          atoms[4]->getLeg(1)->getServoMotor(0)->setTargetPosition(250,REPLACE,0,synchro);
          atoms[7]->getLeg(1)->getServoMotor(0)->setTargetPosition(250,REPLACE,0,synchro);
          atoms[6]->getLeg(0)->getServoMotor(0)->setTargetPosition(0,REPLACE,0,synchro);
          atoms[3+8]->getLeg(0)->getServoMotor(0)->setTargetPosition(0,REPLACE,0,synchro);
          atoms[4+8]->getLeg(1)->getServoMotor(0)->setTargetPosition(250,REPLACE,0,synchro);
          atoms[7+8]->getLeg(1)->getServoMotor(0)->setTargetPosition(250,REPLACE,0,synchro);
          atoms[6+8]->getLeg(0)->getServoMotor(0)->setTargetPosition(0,REPLACE,0,synchro);

          
        }
                
        for(unsigned int i=0;i<ATOMCOUNT;i++)
        {
          atomsPosX[i]=atoms[i]->util->getCartesianPosition().getX();
          atomsPosY[i]=atoms[i]->util->getCartesianPosition().getY();
          atomsPosZ[i]=atoms[i]->util->getCartesianPosition().getZ();
        }
        for(unsigned int i=0;i<ATOMCOUNT;i++)
         activeAtom[i]=true;
        
      }
      break;
    case 'Y':
    case 'y':
      {
          if (atomsAreCreated) {
            unsigned int synchro=atoms[0]->getMolecule()->getNextSyncIndex();
          
            atoms[0]->getLeg(1)->getServoMotor(0)->setTargetPosition(0,REPLACE,0,synchro);
            atoms[3]->getLeg(1)->getServoMotor(0)->setTargetPosition(250,REPLACE,0,synchro);
            atoms[7]->getLeg(0)->getServoMotor(0)->setTargetPosition(0,REPLACE,0,synchro);
            atoms[2]->getLeg(0)->getServoMotor(0)->setTargetPosition(250,REPLACE,0,synchro);
            atoms[0+8]->getLeg(1)->getServoMotor(0)->setTargetPosition(0,REPLACE,0,synchro);
            atoms[3+8]->getLeg(1)->getServoMotor(0)->setTargetPosition(250,REPLACE,0,synchro);
            atoms[7+8]->getLeg(0)->getServoMotor(0)->setTargetPosition(0,REPLACE,0,synchro);
            atoms[2+8]->getLeg(0)->getServoMotor(0)->setTargetPosition(250,REPLACE,0,synchro);
          }

      }

      break;
    case 'g':
    case 'G':


        printf("NB Atom=%i\n",ATOMCOUNT);
        printf("NB Atom per Row=%i\n",AROW);
        printf("NB Atom per Line=%i\n",ALINE);
        printf("NB Row=%i\n",ROWCOUNT);
        printf("NB Line=%i\n",LINECOUNT);

//        Timer::getInstance ()->clearEvents ();

				if (atomsAreCreated) {
          for(unsigned int i=0;i<ATOMCOUNT;i++)
             if (atoms[i])
             {
					      delete atoms[i];
                atoms[i]=0;
              }
          atomsAreCreated=false;
				} else {
      // create a raw/cube of ATOMCOUNT interconnected atoms
          for(unsigned int i=0;i<ATOMCOUNT;i++) {
            atoms[i] = new API::Atom ();
            atoms[i]->util->setCartesianPosition(
                  new API::CartesianPosition(
                      Vector3(VDVAR (GROUP_POSITION)).x+
                      (i % AROW)*(DVAR(KERNELSIDE)+2*DVAR(LEGSIDE)),
                      Vector3(VDVAR (GROUP_POSITION)).y+
                      (i % ALINE/AROW)*(DVAR(KERNELSIDE)+2*DVAR(LEGSIDE)),
                      Vector3(VDVAR (GROUP_POSITION)).z+
                      (i / ALINE)*(DVAR(KERNELSIDE)+2*DVAR(LEGSIDE))                      
                      ));
            atomsAreCreated=true;
          };

        
          /*
          for(unsigned int i=0;i<ATOMCOUNT;i++)
            for(unsigned int j=0;j<6;j++)
              {
                atoms[i]->getLeg(j)->getServoMotor(0)->getSimulatedObject()->print();
                atoms[i]->getLeg(j)->getServoMotor(1)->getSimulatedObject()->print();                
              }
          */  
        for(unsigned int i=0;i<ATOMCOUNT-1;i++) {
          /*
          Vector3 pos;
          pos=atoms[i]->getLeg(0)->util->getSimulatedObject()->getAbsPosEnd();
          pos=atoms[i+1]->getLeg(1)->util->getSimulatedObject()->getAbsPosEnd();
          for(int j=0;j<4;j++)
            pos=atoms[i]->getLeg(0)->util->getSimulatedObject()->getAbsPosConnexionDevice(j);
          for(int j=0;j<4;j++)
            pos=atoms[i+1]->getLeg(1)->util->getSimulatedObject()->getAbsPosConnexionDevice(j);
          */
          printf("%i/%i\n",i+1,ATOMCOUNT);
          if (atoms[i]->getLeg(0)->util->isConnectable(atoms[i+1]->getLeg(1)))
            atoms[i]->getLeg(0)->setLink(atoms[i+1]->getLeg(1));
          if (ROWCOUNT>1 && i<Math::max(ATOMCOUNT-AROW,0) && atoms[i]->getLeg(4)->util->isConnectable(atoms[i+AROW]->getLeg(5)))
            atoms[i]->getLeg(4)->setLink(atoms[i+AROW]->getLeg(5));
          if (LINECOUNT>1 && i<Math::max(ATOMCOUNT-ALINE,0) && atoms[i]->getLeg(3)->util->isConnectable(atoms[i+ALINE]->getLeg(2)))
            atoms[i]->getLeg(3)->setLink(atoms[i+ALINE]->getLeg(2));

        }
/*
        for(unsigned int i=0;i<ATOMCOUNT;i++)
            for(unsigned int j=0;j<6;j++)
              {
                atoms[i]->getLeg(j)->getServoMotor(0)->getSimulatedObject()->print();
                atoms[i]->getLeg(j)->getServoMotor(1)->getSimulatedObject()->print();
              }  
        for(unsigned int i=0;i<ATOMCOUNT;i++) 
          for(int j=0;j<6;j++) {
             atoms[i]->getLeg(j)->getServoMotor(0)->setTargetPosition(0,mode);
             atoms[i]->getLeg(j)->getServoMotor(1)->setTargetPosition(0,mode);
           }
*/
/*
        atoms[0]->getMolecule()->getSimulatedObject()->handleEvent(
          new RotationEvent ( Matrix33::rotateAboutZ (M_PI_4)));
        atoms[0]->getMolecule()->getSimulatedObject()->handleEvent(
          new PositionEvent ( Vector3(VDVAR (ATOM_POSITION))));
*/

        for(unsigned int i=0;i<ATOMCOUNT;i++)
        {
          atomsPosX[i]=atoms[i]->util->getCartesianPosition().getX();
          atomsPosY[i]=atoms[i]->util->getCartesianPosition().getY();
          atomsPosZ[i]=atoms[i]->util->getCartesianPosition().getZ();
        }
        for(unsigned int i=0;i<ATOMCOUNT;i++)
         activeAtom[i]=true;
               
      }
      break;
    case 'h':
    case 'H':
            if (atom)
            {
              for(unsigned int i=0;i<6;i++) {
                if (activeLeg[i]) {
                  atom->getLeg(i)->getServoMotor(0)->setToHalfPosition(mode);
                  atom->getLeg(i)->getServoMotor(1)->setToHalfPosition(mode);
                }
              }
            }
            if (atomsAreCreated) 
            {
              for(unsigned int j=0;j<ATOMCOUNT;j++)
                if (activeAtom[j])
                  for(unsigned int i=0;i<6;i++) {
                    if (activeLeg[i]) {
                      atoms[j]->getLeg(i)->getServoMotor(0)->setToHalfPosition(mode);
                      atoms[j]->getLeg(i)->getServoMotor(1)->setToHalfPosition(mode);
                    }
                }

            }
        
      break;
    case 'a':
    case 'A':
            if (atom)
            {
              for(unsigned int i=0;i<6;i++) {
                if (activeLeg[i]) {
                  atom->getLeg(i)->getServoMotor(0)->setTargetPosition(250,mode);
                }
              }
            }
            if (atomsAreCreated)
            {
              for(unsigned int j=0;j<ATOMCOUNT;j++)
                if (activeAtom[j])
                  for(unsigned int i=0;i<6;i++) {
                    if (activeLeg[i]) {
                      atoms[j]->getLeg(i)->getServoMotor(0)->setTargetPosition(250,mode);
                    }
                  }

            }

      break;
    case 'q':
    case 'Q':
            if (atom)
            {
              for(unsigned int i=0;i<6;i++) {
                if (activeLeg[i]) {
                  atom->getLeg(i)->getServoMotor(0)->setTargetPosition(125,mode);
                }
              }
            }
            if (atomsAreCreated)
            {
              for(unsigned int j=0;j<ATOMCOUNT;j++)
                if (activeAtom[j])
                  for(unsigned int i=0;i<6;i++) {
                    if (activeLeg[i]) {
                      atoms[j]->getLeg(i)->getServoMotor(0)->setTargetPosition(125,mode);
                    }
                  }

            }

      break;

    case 'w':
    case 'W':
            if (atom)
            {
              for(unsigned int i=0;i<6;i++) {
                if (activeLeg[i]) {
                  atom->getLeg(i)->getServoMotor(0)->setTargetPosition(0,mode);
                }
              }
            }
            if (atomsAreCreated)
            {
              for(unsigned int j=0;j<ATOMCOUNT;j++)
                if (activeAtom[j])
                  for(unsigned int i=0;i<6;i++) {
                    if (activeLeg[i]) {
                      atoms[j]->getLeg(i)->getServoMotor(0)->setTargetPosition(0,mode);
                    }
                  }

            }

      break;
    case 'x':
    case 'X':
            if (atom)
            {
              for(unsigned int i=0;i<6;i++) {
                if (activeLeg[i]) {
                  atom->getLeg(i)->getServoMotor(1)->setTargetPosition(250,mode);
                }
              }
            }
            if (atomsAreCreated)
            {
              for(unsigned int j=0;j<ATOMCOUNT;j++)
                if (activeAtom[j])
                  for(unsigned int i=0;i<6;i++) {
                    if (activeLeg[i]) {
                      atoms[j]->getLeg(i)->getServoMotor(1)->setTargetPosition(250,mode);
                    }
                  }

            }

      break;
    case 'c':
    case 'C':
            if (atom)
            {
              for(unsigned int i=0;i<6;i++) {
                if (activeLeg[i]) {
                  atom->getLeg(i)->getServoMotor(1)->setTargetPosition(125,mode);
                }
              }
            }
            if (atomsAreCreated)
            {
              for(unsigned int j=0;j<ATOMCOUNT;j++)
                if (activeAtom[j])
                  for(unsigned int i=0;i<6;i++) {
                    if (activeLeg[i]) {
                      atoms[j]->getLeg(i)->getServoMotor(1)->setTargetPosition(125,mode);
                    }
                  }

            }

      break;

    case 'v':
    case 'V':
            if (atom)
            {
              for(unsigned int i=0;i<6;i++) {
                if (activeLeg[i]) {
                  atom->getLeg(i)->getServoMotor(1)->setTargetPosition(0,mode);
                }
              }
            }
            if (atomsAreCreated)
            {
              for(unsigned int j=0;j<ATOMCOUNT;j++)
                if (activeAtom[j])
                  for(unsigned int i=0;i<6;i++) {
                    if (activeLeg[i]) {
                      atoms[j]->getLeg(i)->getServoMotor(1)->setTargetPosition(0,mode);
                    }
                  }

            }

      break;

    case 's':
    case 'S':
            if (atom)
            {
              for(unsigned int i=0;i<6;i++) {
                if (activeLeg[i]) {
                  atom->getLeg(i)->getServoMotor(0)->stop();
                  atom->getLeg(i)->getServoMotor(1)->stop();
                }
              }
            }
            if (atomsAreCreated)
            {
              for(unsigned int j=0;j<ATOMCOUNT;j++)
                if (activeAtom[j])
                  for(unsigned int i=0;i<6;i++) {
                    if (activeLeg[i]) {
                      atoms[j]->getLeg(i)->getServoMotor(0)->stop();
                      atoms[j]->getLeg(i)->getServoMotor(1)->stop();
                    }
                  }
            }

      break;
/*
    case 'b':
    case 'B':
        
        printf("\n**********************************************************\n");
        if (atomsAreCreated) {

          for(unsigned int i=0;i<ATOMCOUNT-1;i++)
            {
              printf("\n**%i*******************************************************\n",i);
              atoms[i]->getLeg(1)->unlink();
            }
        }
        
        printf("\n**********************************************************\n");
        if (atomsAreCreated) {
          atoms[0]->getMolecule()->atomize();          
        }

      break;
*/
		case 'm':
		case 'M':
			{
//				isShowingFps = !isShowingFps;
        if (mode==KEEP)
          mode=REPLACE;
        else
          mode=KEEP;
        showActive();

      }
			break;
/*
    case 'b':
		case 'B':
			{
        
          Engine::getInstance ()->getRoot()->print();
          //Engine::getInstance ()->getRoot()->draw ();                    
				
			}
			break;
*/
    case '0':
			{
				activeLeg[0] = !activeLeg[0];
        showActive();   
			}
			break;

    case '1':
			{
				activeLeg[1] = !activeLeg[1];
        showActive();
			}
			break;

    case '2':
			{
				activeLeg[2] = !activeLeg[2];
        showActive();
			}
		  break;

    case '3':
			{
				activeLeg[3] = !activeLeg[3];
        showActive();
			}
			break;

    case '4':
			{
				activeLeg[4] = !activeLeg[4];
        showActive();
			}
			break;

    case '5':
			{
				activeLeg[5] = !activeLeg[5];
        showActive();
			}
			break;
    case '6':
			{
        for(unsigned int i=0;i<6;i++)
          activeLeg[i]=!activeLeg[i];
        showActive();
			}
			break;

    case '7':
			{
        for(unsigned int i=0;i<6;i++)
          activeLeg[i]=true;
        showActive();
			}
			break;

    case '8':
			{
        for(unsigned int i=0;i<6;i++)
          activeLeg[i]=false;
        showActive();
			}
			break;

    case 't':
    case 'T':
			{
        if (atomsAreCreated)
        {
          for(unsigned int i=0;i<ATOMCOUNT;i++)
            activeAtom[i]=true;
          showActive();
        }
			}
			break;
      
    case 'i':
    case 'I':
			{
        if (atomsAreCreated)
        {
          for(unsigned int i=0;i<ATOMCOUNT;i++)
            activeAtom[i]=!activeAtom[i];
          showActive();
        }
			}
			break;

    case 'k':
    case 'K':
			{
        if (atomsAreCreated)
        {
          for(unsigned int i=0;i<ATOMCOUNT;i++)
            if (i % 2==0)
              activeAtom[i]=true;
            else
              activeAtom[i]=false;
          showActive();
        }
			}
			break;

    case 'l':
    case 'L':
			{
        if (atomsAreCreated)
        {

          for(unsigned int j=0;j<ATOMCOUNT;j++)
          {
            atoms[j]->getLeg(5)->getServoMotor(1)->setTargetPosition(0,KEEP);
            atoms[j]->getLeg(2)->getServoMotor(1)->setTargetPosition(250,KEEP);
          }
          for(unsigned int j=0;j<ATOMCOUNT;j++)
            if (j % 2==0)
            {
              atoms[j]->getLeg(5)->getServoMotor(0)->setTargetPosition(125,KEEP);
              atoms[j]->getLeg(5)->getServoMotor(1)->setTargetPosition(125,KEEP);
              atoms[j]->getLeg(5)->getServoMotor(0)->setTargetPosition(0,KEEP);
              atoms[j]->getLeg(5)->getServoMotor(1)->setTargetPosition(0,KEEP);
              atoms[j]->getLeg(2)->getServoMotor(0)->setTargetPosition(125,KEEP);
              atoms[j]->getLeg(2)->getServoMotor(1)->setTargetPosition(125,KEEP);
              atoms[j]->getLeg(2)->getServoMotor(0)->setTargetPosition(250,KEEP);
              atoms[j]->getLeg(2)->getServoMotor(1)->setTargetPosition(0,KEEP);
            }
            else
            {
              atoms[j]->getLeg(5)->getServoMotor(0)->setTargetPosition(125,KEEP,10);
              atoms[j]->getLeg(5)->getServoMotor(1)->setTargetPosition(125,KEEP,10);
              atoms[j]->getLeg(5)->getServoMotor(0)->setTargetPosition(0,KEEP,10);
              atoms[j]->getLeg(5)->getServoMotor(1)->setTargetPosition(0,KEEP,10);
              atoms[j]->getLeg(2)->getServoMotor(0)->setTargetPosition(125,KEEP,10);
              atoms[j]->getLeg(2)->getServoMotor(1)->setTargetPosition(125,KEEP,10);
              atoms[j]->getLeg(2)->getServoMotor(0)->setTargetPosition(250,KEEP,10);
              atoms[j]->getLeg(2)->getServoMotor(1)->setTargetPosition(0,KEEP),10;
            }

        }        
			}
			break;


   

  }
}

void stop ()
{
  if (atom)
    delete atom;
  for (int i=0;i<NMAX;i++)
    if (atoms[i])
    {
      delete atoms[i];
      atoms[i]=0;
    }
  printf("Desallocation faite !");
  engine->shutdown ();
}

/*******************************/

int main (int argc, char** argv)
{
	CR_SETUP (Main, TARGET_FPS, CR_DOUBLE);
  CR_SETUP (Main, GROUP_POSITION, CR_DOUBLE);
  CR_SETUP (Main, ATOM_POSITION, CR_DOUBLE);
  CR_SETUP (Main, GROUP_ATOM_COUNT, CR_INT);
  CR_SETUP (Main, GROUP_ATOM_PER_ROW, CR_INT);
  CR_SETUP (Main, GROUP_ATOM_PER_LINE, CR_INT);

  CR_SETUP (Atom, KERNELSIDE, CR_DOUBLE);
  CR_SETUP (Atom, SPHERESIDE, CR_DOUBLE);
  CR_SETUP (Atom, LEGRADIUS, CR_DOUBLE);
  CR_SETUP (Atom, LEGSIDE, CR_DOUBLE);


	char* odehome_env = getenv("ODEHOME");
	std::string odehome_string;


#ifdef USE_OPENGL
	dsFunctions fn;
	fn.version          = DS_VERSION;
	fn.start            = 0;
	fn.step             = &step;
	fn.command          = &command;
	fn.stop             = &stop;

	if (odehome_env == NULL) 
	{
		odehome_string = "../../ode/drawstuff/textures";
	} 
	else 
	{
		odehome_string = odehome_env;
		odehome_string += "/drawstuff/textures";
	}

	fn.path_to_textures = (char*)odehome_string.c_str ();
#endif

	engine = Engine::getInstance ();
	engine->initialize           ();
	engine->setTargetFps         (DVAR (TARGET_FPS));

	collisionGroup = new CollisionGroup ();
  SimMolecule::setCollisionGroup(collisionGroup);
  	
	CollisionManager::getInstance ()->addCollisionGroup (collisionGroup);	

	Node* root     = engine->getRoot  ();
	collisionGroup->addCollidee (root);

	SimWorld* world = new SimWorld ();
	root->addChild (world);


#ifdef USE_OPENGL
	dsSimulationLoop (argc, argv, 600, 480, &fn);
#else

	bool quit = false;
	bool pause = false;

	printf("Starting main loop...\n");
	engine->setIsPaused (false);

	while (!quit) {
		step(pause);
	}
	printf("Stopping!!\n");
	stop();
#endif

	return 0;
}

