/***************************************************************************
                          Molecule.cpp  -  description
                             -------------------
    begin                : mar juin 8 2003
    copyright            : (C) 2003 by Michel DUBOIS, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


// Modle statique
/**
  * changes :
  *  - 08/06/03 MD Creation !
  */

#include <algorithm>
#include <iostream>
#include "mmgr.h"
#include "Molecule.h"
#include "Timer.h"

using namespace API;

unsigned int Molecule::getID()
{
	return this->ID;
}

void Molecule::stop()
{
  for (unsigned int i=0; i < atoms.size(); i++)
    atoms[i]->stop();	
}

void Molecule::setID(int aMoleculeID)
{
  this->ID=aMoleculeID;
}

Molecule::Molecule ()
{
  static int maxID=-1;
  this->setID(++maxID);
  this->util=new Util(this);
  atomCount = 0;
  this->util->simmolecule=new SimMolecule;
  linkCount = 0;
  for (unsigned int i=0; i<atomCount; i++)
    adj[i].assign(atomCount,0);
//  std::cout << "Molecule constructeur \n" << std::endl ;
  syncIndexes.clear();
  
}

Molecule::~Molecule ()
{
  delete util;
//  std::cout << "Molecule destructeur \n" << std::endl ;
 
}

Atom* Molecule::getAtom (unsigned int index) const
{
	Atom* atom = 0;

	if (0 <= index && index < atoms.size ())
	{
		atom = atoms[index];
	}

	return atom;
}

unsigned int Molecule::getAtomCount () const
{
	return atomCount;
}

void Molecule::addAtom (Atom* atom)
{
  if (atom)
	{
    if (this->getAtomIndex(atom)==-1) {
      atomCount++;
		  atom->setMolecule (this);
      atom->getMolecule()->util->getSimulatedObject()->addAtom(atom->util->getSimulatedObject());
		  atoms.push_back (atom);
      //agrandissement de la matrice d'adjascence
      for (unsigned int i=0; i<atomCount-1; i++)
        adj[i].push_back(0);
      std::vector<Link *> tmp(atomCount,0);
      adj.push_back(tmp);
    }
	}
//  std::cout << "Molecule addAtom \n" << std::endl ;
//  this->print();
}

void Molecule::removeAtom (Atom* atom)
{
	if (atom)
	{
    if (this->getAtomIndex(atom)!=-1) {

      int index=(unsigned int) this->getAtomIndex (atom);
      // Move elements after the index up one position
      // And remove spare element at the end.
      for (unsigned int j=0;j<adj.size();j++)
      {
        for (unsigned int i=index; i + 1 < adj[j].size(); ++i)
          adj[j][i] = adj[j][i + 1];
        adj[j].pop_back();
      }
      for ( unsigned int i=index; i + 1 < adj.size(); ++i)
        adj[i] = adj[i + 1];
      adj.pop_back();
      for ( unsigned int i=index; i + 1 < atoms.size(); ++i)
        atoms[i] = atoms[i + 1];
      atoms.pop_back();
      linkCount=0;
      for ( unsigned int i=1; i < atoms.size(); i++)
        for ( unsigned int j=0; j < i; j++)
          if (adj[i][j])
            linkCount++;
      atomCount=atoms.size();
      atom->getMolecule()->util->getSimulatedObject()->removeAtom(atom->util->getSimulatedObject());
      atom->setMolecule (0);

    }
	}
//  std::cout << "Molecule removeAtom \n" << std::endl ;
//  this->print();
}

SimMolecule * Molecule::Util::getSimulatedObject()
{
  return simmolecule;
}

/*
 * Insertion d'un lien
 * avec fusion eventuelle des molecules
 */
void Molecule::insertLink(Link* link)
{

  Molecule * otherMolecule=link->getLeg1()->getAtom()->getMolecule();
  if (this==otherMolecule)
    otherMolecule=link->getLeg2()->getAtom()->getMolecule();
  if (this!=otherMolecule)
  {
    /************************
     * Fusion de deux molecules
     ***********************/
    std::vector<Atom *> atraiter;
    // Ajout des atoms et agrandissement de la matrice d'adjascence
    for (unsigned int i=0; i<otherMolecule->atomCount;i++)
    {
      this->addAtom(otherMolecule->atoms[i]);
      atraiter.push_back(otherMolecule->atoms[i]);
    }
    // Mise a jour de la matrice d'adjascence
    for (unsigned int i=0; i<otherMolecule->linkCount;i++)
    {
      this->insertLink(otherMolecule->getLink(i));
    }
    // Destruction de la molecule absorbee
    /*for (unsigned int i=0; i<atraiter.size();i++)
    {
      otherMolecule->removeAtom(atraiter[i]);
    }*/

    delete otherMolecule;
  }

  int v1=(unsigned int) this->getAtomIndex(link->getLeg1()->getAtom());
  int v2=(unsigned int) this->getAtomIndex(link->getLeg2()->getAtom());

  if (adj[v1][v2]==0) {
    linkCount++;
//    link->setMolecule(this);
    adj[v1][v2]=link;
    adj[v2][v1]=link;
  }
//  std::cout << "Molecule insertLink \n" << std::endl ;
//  this->print();
}

/*
 * Retrait d'un lien
 * avec creation eventuelle d'une autre molecule
 */

void Molecule::removeLink(Link* link)
{
  int v1=(unsigned int) this->getAtomIndex(link->getLeg1()->getAtom());
  int v2=(unsigned int) this->getAtomIndex(link->getLeg2()->getAtom());
  //on efface le lien de la matrice d'adjacence
  if (adj[v1][v2]!=0) {
    linkCount--;
    adj[v1][v2]=0;
    adj[v2][v1]=0;
//    link->setMolecule(0);
  }
  //on calcule le nombre de composants connectes
  this->computeConnectedComponent();
  //on veut garder dans la molecule l'atome de base (qui a fait unlink)
  this->setBaseConnectedComponent(this->getAtomIndex(link->getLeg1()->getAtom()));
  std::vector<Atom *> atomAtraiter;
  std::vector<Link *> linkAtraiter;
  // Si des atomes plus connectes avec celui ci
  while (connectedComponentCount>1)
  {

    atomAtraiter.clear();
    linkAtraiter.clear();

    // Sauvegarde des atoms et des liens
    for (unsigned int i=0; i<atomCount;i++)
    {
      if (this->connectedComponentID[i]==connectedComponentCount-1)
      {
        atomAtraiter.push_back(atoms[i]);
        this->adjItInit(i);
        for (int v=adjItBeg();!adjItEnd();v=adjItNext())
           linkAtraiter.push_back(adj[i][v]);
      }
    }
    Molecule * newMolecule=new Molecule;
    for (unsigned int i=0; i<atomAtraiter.size();i++)
    {
        this->removeAtom(atomAtraiter[i]);
        newMolecule->addAtom(atomAtraiter[i]);
    }
    for (unsigned int i=0; i<linkAtraiter.size();i++)
    {
        newMolecule->insertLink(linkAtraiter[i]);
    }
    this->computeConnectedComponent();
    this->setBaseConnectedComponent(this->getAtomIndex(link->getLeg1()->getAtom()));
  }
  //std::cout << "Molecule removeLink \n" << std::endl ;
  //this->print();
}

void Molecule::atomize()
{
  bool problem=false;
  while (atomCount>1 and problem==false)
  {
    Link * link;
    Leg * leg;
    leg=0;
    link=0;
    for ( unsigned int i=1; i < atoms.size() and !leg; i++)
      for ( unsigned int j=0; j< i and !leg; j++)
        if (adj[i][j])
        {
          link=adj[i][j];
          adj[i][j]=0;
          adj[j][i]=0;
          this->computeConnectedComponent();
          int card=getConnectedComponentCard(1);
          adj[i][j]=link;
          adj[j][i]=link;

          if (card<2)
          {
            leg=link->getLeg1();
            if (this->connectedComponentID[this->getAtomIndex(link->getLeg1()->getAtom())]==1)
              leg=link->getLeg2();
          }
        }
    if (leg)
      leg->unlink();
    else
      problem=true;
  }
  //std::cout << "Molecule atomize \n" << std::endl ;
  //this->print();
}


unsigned int Molecule::getLinkCount () const
{
	return linkCount;
}

int Molecule::getAtomIndex (Atom * anAtom) const
{
  for (unsigned int i = 0; i < atoms.size (); i++)
	{
		if (atoms[i] == anAtom)
		{
			return i;
		}
	}
	return -1;
}

//Link * Molecule::getLink(unsigned int index1,unsigned int index2) const
Link * Molecule::getLink(unsigned int atomIndex1,unsigned int atomIndex2, unsigned linkIndex) const
{
  return adj[atomIndex1][atomIndex2];
}


Link * Molecule::getLink(unsigned int index) const
{
      unsigned int k=0;
      for ( unsigned int i=1; i < atoms.size(); i++)
        for ( unsigned int j=0; j < i; j++)
          if (adj[i][j])
              if (k==index)
                return adj[i][j];
              else
                k++;
      return 0;                      
}

int Molecule::getLinkIndex (Link * aLink) const
{
  unsigned int k=0;
  for ( unsigned int i=1; i < atoms.size(); i++)
    for ( unsigned int j=0; j < i; j++)
      if (adj[i][j]==aLink)
        return k;
      else
        k++;
	return -1;
}


void Molecule::adjItInit(unsigned int atomIndex)
{
  this->adjItAtom=atomIndex;
  this->adjItPos=-1;
}

int Molecule::adjItBeg()
{
  this->adjItPos=-1;
  return this->adjItNext();
}
  
int Molecule::adjItNext()
{
  for (adjItPos++; adjItPos<(int)atoms.size();adjItPos++)
    if (adj[adjItAtom][adjItPos])
      return adjItPos;
  return -1;
}
  
bool Molecule::adjItEnd() const
{
  return adjItPos >=(int)atoms.size();
}


void Molecule::computeConnectedComponent()
{
  connectedComponentID.clear();
  connectedComponentID.assign(atoms.size(),-1);
  connectedComponentCount=0;
  for (unsigned int v=0; v<atoms.size();v++)
    if (connectedComponentID[v]==-1)
    {
      computeConnectedComponent(v);
      connectedComponentCount++;
    }
}
  
void Molecule::setBaseConnectedComponent(unsigned int atomIndex)
{
  int CCID=connectedComponentID[atomIndex];
  if (CCID!=0)
  for (unsigned int v=0; v<atoms.size();v++)
  {
    if (connectedComponentID[v]==0)
      connectedComponentID[v]=CCID;
    else if (connectedComponentID[v]==CCID)
      connectedComponentID[v]=CCID;
  }                
}
  

void Molecule::computeConnectedComponent(unsigned int atomIndex)
{
  connectedComponentID[atomIndex]=connectedComponentCount;
  this->adjItInit(atomIndex);
  for (int v=adjItBeg();!adjItEnd();v=adjItNext())
    if (connectedComponentID[v]==-1)
      computeConnectedComponent(v);
}

unsigned int Molecule::getConnectedComponentCount() const
{
  return connectedComponentCount;
}

unsigned int Molecule::getConnectedComponentCard(unsigned int index) const
{
  unsigned int k=0;
  for ( unsigned int i=0; i < atoms.size(); i++)
    if (connectedComponentID[i]==index)
      k++;
  return k;
}

bool Molecule::isConnected(unsigned int atomIndex, unsigned int anotherAtomIndex) const
{
  return connectedComponentID[atomIndex]==connectedComponentID[anotherAtomIndex];
}

void Molecule::Util::print () const
{
  printf ("Molecule: ID=%i\n",this->molecule->ID);
  this->printAtoms();
  this->printLinks();
}

void Molecule::Util::printAtoms () const
{
  printf ("Atoms: size=%i \n",molecule->atoms.size());
  for (unsigned int i=0; i < molecule->atoms.size(); i++)
    molecule->atoms[i]->util->print();
}

void Molecule::Util::printLinks () const
{
  printf ("Links: size=%i \n",molecule->linkCount);
  for (unsigned int i=0; i < molecule->linkCount; i++)
    molecule->getLink(i)->util->print();
}

void Molecule::Util::printAdjacencyMatrix() const
{
  printf("Adjacency Matrix\n");
  for (unsigned int i=0; i<molecule->atoms.size();i++)
    printf("-");
  printf("\n");
  for (unsigned int i=0; i<molecule->atoms.size();i++)
  {
    for (unsigned int j=0; j<molecule->atoms.size();j++)
      if (molecule->adj[i][j])
        printf("*");
      else
        printf(" ");
    printf("\n");
  }    
}

void Molecule::Util::printConnectedComponentID() const
{
  printf ("Atoms: size=%i \n",molecule->atoms.size());
  for (unsigned int i=0; i < molecule->atoms.size(); i++)
    printf ("[%i] CCID=%i\n",i,this->molecule->connectedComponentID[i]);
  
}
Molecule::Util::Util(Molecule * molecule)
{
this->molecule=molecule;
this->simmolecule=0;
}

Molecule::Util::~Util()
{
  delete simmolecule;
}

unsigned int Molecule::getNextSyncIndex()
{
  unsigned int index=0;
  bool found=true;
  while (found==true)
  {
    index++;
    found=false;
    for (unsigned int i = 0; i < syncIndexes.size (); i++)
		{
       if(syncIndexes[i]==index)
          found=true;
    }		
  }
  for (unsigned int i = 0; i < syncIndexes.size (); i++)
		{
       if(syncIndexes[i]==0)
       {
          syncIndexes[i]=index;
          return index;
       }
    }
  syncIndexes.push_back (index);
  return index;
}

void Molecule::releaseSyncIndex(unsigned int index)
{
  if (isFinished(index))
  {
    for (unsigned int i = 0; i < syncIndexes.size (); i++)
		{
       if(syncIndexes[i]==index)
       {
          syncIndexes[i]=0;
          return;
       }
    }    
  }  
}

unsigned int Molecule::getSyncIndexCount()
{
  unsigned int index=0;
  for (unsigned int i = 0; i < syncIndexes.size (); i++)
    if(syncIndexes[i]!=0)
      index++;
  return index;
}

unsigned int Molecule::getSyncIndex(unsigned int index)
{
  unsigned int k=0;
  for (unsigned int i = 0; i < syncIndexes.size (); i++)
    if (syncIndexes[i]!=0)
      if (k==index)
        return syncIndexes[i];
      else
        k++;
  return 0;
}

bool Molecule::isFinished(unsigned int index)
{
  return Timer::getInstance ()->isEmpty(index);
}
