/***************************************************************************
                          ServoMotor.cpp  -  description
                             -------------------
    begin                : mar juin 8 2003
    copyright            : (C) 2003 by Michel DUBOIS, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



// Modle statique
 /**
  * changes :
  *  - 08/06/03 MD Creation !
  */

#include <ode/ode.h>
#include "mmgr.h"
#include "ServoMotor.h"
#include "Timer.h"
#include "Event.h"


using namespace API;


ServoMotor::ServoMotor()
{
  this->leg=0;
  this->util=new Util(this);
  this->servoID=0;
  this->targetPos=0;
  this->targetVelocity=0;
}

ServoMotor::~ServoMotor()
{
  delete this->util;
}

int ServoMotor::getID() const
{
	return servoID;
}

Leg* ServoMotor::getLeg() const
{
 return leg;	
}

int ServoMotor::getTargetPosition() const
{
 return this->targetPos;	
}

double ServoMotor::getTargetAngle() const
{
 return util->simservo->Position2Angle(this->targetPos);
}

int ServoMotor::getCurrentPosition() const
{
  return util->simservo->Angle2Position(util->simservo->getAngle());
}

double ServoMotor::getTargetVelocity() const
{
	return this->targetVelocity;
}

double ServoMotor::getCurrentVelocity() const
{
	return util->simservo->getCurrentVelocity();
}

double ServoMotor::getCurrentAngle() const
{
  return util->simservo->getAngle();
}

double ServoMotor::getHighStopAngle() const
{
	return util->simservo->getHighStopAngle(); 
}

double ServoMotor::getLowStopAngle() const
{
	return util->simservo->getLowStopAngle();
}

void ServoMotor::setTargetAngle(double aTargetAngle, int mode, Real deltaTime, unsigned int syncIndex)
{
  this->setTargetPosition(this->util->simservo->Position2Angle(aTargetAngle), mode, deltaTime,syncIndex);
}

void ServoMotor::setTargetPosition(int aTargetPosition, int mode, Real deltaTime, unsigned int syncIndex)
{
	this->targetPos=aTargetPosition;
  if (mode == REPLACE) {
    Timer::getInstance ()->clearEvents(util->simservo);
    deltaTime=0;
  }
  Timer::getInstance ()->addEvent (util->simservo, new ServoMotorTargetPositionEvent (aTargetPosition), deltaTime,-1,syncIndex);
}

void ServoMotor::setToHalfPosition( int mode, Real deltaTime, unsigned int syncIndex)
{
   // mi-chemin entre min et max
  setTargetPosition(util->simservo->getHalfPosition(),mode,deltaTime,syncIndex);  
}

void ServoMotor::setTargetVelocity(int aTargetVelocity, int mode, Real deltaTime, unsigned int syncIndex)
{ // voir maxvelocity et minvelocity == 0
  this->targetVelocity=aTargetVelocity;
  // supprimer la velocity precedente
  Timer::getInstance ()->addEvent (util->simservo, new ServoMotorTargetVelocityEvent (aTargetVelocity), 0);
}

void ServoMotor::stop()
{ // implementer via un targetvelocityevent a 0 ou un targetpositionevent a currentposition
  // en mode REPLACE
  Timer::getInstance ()->clearEvents(util->simservo);
  Timer::getInstance ()->addEvent (util->simservo, new ServoMotorTargetPositionEvent (this->getCurrentPosition()), 0);	
}


ServoMotor::Util::Util(ServoMotor * aServoMotor)
{
  this->servo=aServoMotor;
  this->simservo=0;
}

ServoMotor::Util::~Util()
{
}

void ServoMotor::setID(int aServoMotorID)
{
  this->servoID=aServoMotorID;	
}

void ServoMotor::setLeg( Leg * aLeg)
{
  this->leg=aLeg;
}

SimServoMotor * ServoMotor::Util::getSimulatedObject() const
{
  return this->simservo;
}

void ServoMotor::Util::setSimulatedObject(SimServoMotor * aSimServoMotor )
{
  this->simservo=aSimServoMotor;
}





                        