/***************************************************************************
                          SimIsland.cpp  -  description
                             -------------------
    begin                : mar jui 29 2003
    copyright            : (C) 2003 by Michel DUBOIS, Yann Le Guyadec 
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mmgr.h"
#include "SimIsland.h"
#include "SimRigidBody.h"

SimIsland::SimIsland() : Island(), Graph() , Controller()
{
  //par prudence
  rigidBodies.clear();
  rigidBodyCount=0;  
}
SimIsland::~SimIsland()
{

}

void SimIsland::setIsEnabled (const bool isEnabled)
{
  Controller::setIsEnabled (isEnabled);
  for (int i=0; i<getVertexCount();i++)
  {
    SimRigidBody * tmp=dynamic_cast<SimRigidBody *> (getVertex(i));
    tmp->setIsEnabled(isEnabled);
  }
}

void SimIsland::destroyGraph(Graph * aGraph)
{
  delete aGraph;
}

Graph * SimIsland::createGraph ()
{
  return new SimIsland();
}

void SimIsland::print()
{
  printf ("SimIsland: rigidBodyCount=%i\n", rigidBodyCount);
  Graph::print ();
}

unsigned int SimIsland::getRigidBodyCount ()
{
  return rigidBodyCount;
//  return getVertexCount();
}

unsigned int SimIsland::size ()
{
  return rigidBodyCount;
  //return getVertexCount();
}

SimRigidBody* SimIsland::getSimRigidBody (unsigned int index)
{
	return dynamic_cast<SimRigidBody *> (getVertex(index));
}

void SimIsland::addSimRigidBody (SimRigidBody* rigidBody)
{
  if (rigidBody)
    addVertex(rigidBody);
  synchronize();
}

void SimIsland::removeSimRigidBody (SimRigidBody* rigidBody)
{
  if (rigidBody)
    removeVertex(rigidBody);
  synchronize();
}

SimRigidBody* SimIsland::removeSimRigidBody (unsigned int i)
{
	SimRigidBody* rigidBody = dynamic_cast<SimRigidBody *> (getVertex(i));
  if (rigidBody)
    removeVertex(rigidBody);
  synchronize();
	return rigidBody;
}

RigidBody* SimIsland::getRigidBody (unsigned int index)
{
	return dynamic_cast<RigidBody *> (getVertex(index));
}

void SimIsland::addRigidBody (RigidBody* rigidBody)
{
  printf("Attention ! addRigidBody(RigidBody)\n");
  /*
  SimRigidBody * tmp =dynamic_cast<SimRigidBody *> (rigidBody);
  if (tmp)
    addVertex(tmp);
  */
}

void SimIsland::removeRigidBody (RigidBody* rigidBody)
{
  printf("Attention ! removeRigidBody(RigidBody)\n");
  /*
  SimRigidBody * tmp =dynamic_cast<SimRigidBody *> (rigidBody);
  if (tmp)
    removeVertex(tmp);
  */
}

RigidBody* SimIsland::removeRigidBody (unsigned int i)
{
	SimRigidBody* rigidBody = dynamic_cast<SimRigidBody *> (getVertex(i));
  if (rigidBody)
  {
    printf("Attention ! RigidBody removeRigidBody(i)\n");
    /*
    removeVertex(rigidBody);
    */
    
  }
  return rigidBody;
}


void SimIsland::synchronize()
{
  //printf ("Synchronize\n");
  rigidBodies.clear();
  rigidBodyCount=0;
  for (int i=0;i<getVertexCount();i++)
  {
    rigidBodies.push_back(dynamic_cast<RigidBody *> (getVertex(i)));
    getSimRigidBody(i)->setSimIsland(this);
    rigidBodyCount++;
  }     
}
