/***************************************************************************
                          SimJoint.cpp  -  description
                             -------------------
    begin                : mar jui 29 2003
    copyright            : (C) 2003 by Michel DUBOIS, Yann Le Guyadec 
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mmgr.h"
#include "SimJoint.h"

SimJoint::SimJoint(SimRigidBody * body1, SimRigidBody * body2) : Joint(), Connection(body1,body2), Controller()
{
	this->rigidBody1 = body1;
	this->rigidBody2 = body2;
  this->getVertex1()->getGraph()->insertConnection(this);

  SimIsland * island=dynamic_cast<SimIsland *> (body1->getIsland());
  if (island)
    island->synchronize();
  
  
}

void SimJoint::swap()
{
  RigidBody * tmpR=this->rigidBody1;
  this->rigidBody1=this->rigidBody2;
  this->rigidBody2=tmpR;
  Vertex * tmpV=this->vertex1;
  this->vertex1=vertex2;
  this->vertex2=tmpV;  
}

void SimJoint::setIsEnabled (const bool isEnabled)
{
  Controller::setIsEnabled (isEnabled);
}

SimJoint::~SimJoint()
{
  this->getGraph()->removeConnection(this);
  SimIsland * island1=0;
  SimIsland * island2=0;
  SimRigidBody * r1=dynamic_cast<SimRigidBody *> (rigidBody1);
  if (r1)
  {
    island1=dynamic_cast<SimIsland *> (r1->getIsland());
    if (island1)
      island1->synchronize();
  }  
  SimRigidBody * r2=dynamic_cast<SimRigidBody *> (rigidBody2);
  if (r2)
  {
    island2=dynamic_cast<SimIsland *> (r2->getIsland());
    if (island2 && island1!=island2)
      island2->synchronize();
  }    
  dJointAttach (joint,0,0);
  dJointDestroy (joint);
}

void SimJoint::print() const
{
  printf ("SimJoint: dJointID=%i Enable=%i\n",this->joint,this->getIsEnabled());
  Connection::print ();  
}
