/***************************************************************************
                          SimLeg.cpp  -  description
                             -------------------
    begin                : mer jun 25 2003
    copyright            : (C) 2003 by Michel DUBOIS, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <ode/ode.h>
#include "ConfigReader.h"
#include "mmgr.h"

#include "SimLeg.h"
#include "SimAtom.h"
#include "Event.h"
#include "Geometry.h"
#include "CylinderGeometry.h"
#include "SphereGeometry.h"
#include "RigidBody.h"
#include "RigidBodyCollider.h"
#include "Timer.h"
#include "Surface.h"
#include "Node.h"

CR_DECLARE (KERNELSIDE);
CR_DECLARE (SPHEREMASS);
CR_DECLARE (SPHERESIDE);
CR_DECLARE (SPHEREMU);
CR_DECLARE (SPHEREMU2);
CR_DECLARE (LEGRADIUS);
CR_DECLARE (LEGSIDE);
CR_DECLARE (LEGMASS);
CR_DECLARE (LEGMU);
CR_DECLARE (LEGMU2);
CR_DECLARE (LEGSLIP1);
CR_DECLARE (LEGSLIP2);
CR_DECLARE (COLLISIONLEG);

std::string SimLeg::legCollision="";

SimLeg::SimLeg(SimAtom * atom, int aLegID){
  static bool isConfiged = false;

	if (!isConfiged)
	{
		isConfiged = true;

		CR_SETUP (Atom, KERNELSIDE, CR_DOUBLE);
    CR_SETUP (Atom, SPHEREMASS, CR_DOUBLE);
    CR_SETUP (Atom, SPHERESIDE, CR_DOUBLE);
    CR_SETUP (Atom, SPHEREMU, CR_DOUBLE);
    CR_SETUP (Atom, SPHEREMU2, CR_DOUBLE);
    CR_SETUP (Atom, LEGRADIUS, CR_DOUBLE);
    CR_SETUP (Atom, LEGSIDE, CR_DOUBLE);
    CR_SETUP (Atom, LEGMASS, CR_DOUBLE);
    CR_SETUP (Atom, LEGMU, CR_DOUBLE);
    CR_SETUP (Atom, LEGMU2, CR_DOUBLE);
    CR_SETUP (Atom, LEGSLIP1, CR_DOUBLE);
    CR_SETUP (Atom, LEGSLIP2, CR_DOUBLE);
    CR_SETUP (Atom, COLLISIONLEG, CR_STRING);
    SimLeg::legCollision=SVAR(COLLISIONLEG);
  }
  this->ID=aLegID;
  this->simatom=atom;
  Node* leg = new Node ("Leg");
	this->model = leg;
 #ifdef LEGLINK
  linkedLeg=0;
  #endif
  #ifndef LEGLINK
  slink=0;
  #endif



/*
  rigidBody        = new RigidBody ();
	motionController = rigidBody;
	rigidBody->setModel (leg);
*/
  rigidBody        = 0;
	motionController = 0;
/*
  if (legCollision=="OLD")
    printf("Old\n");
  else
    printf("New\n");
*/
  dMass sphereMass;
  dMassSetSphere (&sphereMass,1,0.5*DVAR (SPHERESIDE));
  dMassAdjust (&sphereMass,DVAR (SPHEREMASS));
  sphere=new SimRigidBody ();
  dBodySetMass (sphere->getBody (), &sphereMass);
  sphereGeometry = new SphereGeometry (0.5*DVAR (SPHERESIDE), "Sphere");
  sphere->setModel(sphereGeometry);
  leg->addChild(sphereGeometry);
  sphere->setLocalPosition(getVector()*0.5*DVAR (KERNELSIDE));
  sphereGeometry->setColor (Vector3(0,1,1));
  dSurfaceParameters* sp;
    
  Surface* sphereSurface = new Surface ();
	sp = &sphereSurface->getSurfaceParameters ();
  sp->mode  = dContactSlip1 | dContactSlip2;
  sp->mu    = DVAR (SPHEREMU);
	sp->mu2   = DVAR (SPHEREMU2);
	sp->slip1 = 0;
	sp->slip2 = 0;
  sphereCollider=new RigidBodyCollider (sphere);
  sphereCollider->setSurface (sphereSurface);
	sphereGeometry->setCollisionController (sphereCollider);
  int geometry=CAPPEDCYLINDER;
  if (legCollision=="OLD")
      bodyGeometry = new CylinderGeometry (DVAR (LEGSIDE),DVAR (LEGRADIUS), "Body");
  else 
  {
      if (legCollision=="BOX")
        geometry=BOX;
      if (legCollision=="FLATCYLINDER")
        geometry=FLATCYLINDER;
      if (legCollision=="COMPLEX")
        geometry=COMPLEX;
      bodyGeometry = new LegGeometry (DVAR (LEGSIDE),DVAR (LEGRADIUS), "Body",geometry,this->ID);
  }     
  leg->addChild(bodyGeometry);
  dMass legMass;
  dMassSetCylinder (&legMass,1,1,DVAR (LEGRADIUS), DVAR (LEGSIDE));
  dMassAdjust (&legMass,DVAR (LEGMASS));
  body=new SimRigidBody();
  dBodySetMass (body->getBody (), &legMass);
  body->setModel(bodyGeometry);
  bodyGeometry->setColor (getColor());
  body->setLocalPosition(getVector()*(0.5*DVAR (KERNELSIDE)+0.5*DVAR (LEGSIDE)));
  body->setWorldRotation(Matrix33::rotate (getRotationAngle(geometry),getAxis(0,geometry)));
  servo[0] = new SimServoMotor( dynamic_cast<SimRigidBody *> (atom->rigidBody), sphere);
  servo[0]->sleg=this;
  servo[0]->ID=0;  
  servo[0]->setAxis(getAxis(0));
  servo[0]->getHingeJoint()->setAnchor(getVector() *(0.5*DVAR (KERNELSIDE)));
  servo[1] = new SimServoMotor(sphere,body);
  servo[1]->sleg=this;
  servo[1]->ID=1;
  servo[1]->setAxis(getAxis(1));
  servo[1]->getHingeJoint()->setAnchor(getVector() *(0.5*DVAR (KERNELSIDE)));
  servo[0]->setCoupledServoMotor(servo[1]);
  servo[1]->setCoupledServoMotor(servo[0]);

  Surface* bodySurface = new Surface ();
  sp = &bodySurface->getSurfaceParameters ();
	sp->mode  = dContactApprox1 | dContactFDir1 | dContactMu2 | dContactSlip1 ;//| dContactSlip2;
	sp->mu    = DVAR (LEGMU);
	sp->mu2   = DVAR (LEGMU2);
	sp->slip1 = DVAR (LEGSLIP1);
	sp->slip2 = DVAR (LEGSLIP2);
  bodyCollider=new RigidBodyCollider (body);
  bodyCollider->setSurface (bodySurface);
  bodyGeometry->setCollisionController (bodyCollider);

/*
  dMass sphereMass;
  dMassSetSphere (&sphereMass,1,0.5*DVAR (SPHERESIDE));
  dMassAdjust (&sphereMass,DVAR (SPHEREMASS));
  sphere=new RigidBody ();
  dBodySetMass (sphere->getBody (), &sphereMass);
  sphereGeometry = new SphereGeometry (0.5*DVAR (SPHERESIDE), "Sphere");
  sphere->setModel(sphereGeometry);
  leg->addChild(sphereGeometry);
  switch (aLegID) {
    case 0 : //right
      sphereGeometry->setColor (Vector3(1,0,0));
      sphere->setLocalPosition(Vector3 (0.5*DVAR (KERNELSIDE),0,0));
      break;
    case 1 : //left
      sphereGeometry->setColor (Vector3(0,1,0));
      sphere->setLocalPosition(Vector3 (-0.5*DVAR (KERNELSIDE),0,0));
      break;
    case 2 : //bottom
      sphereGeometry->setColor (Vector3(0,0,1));
      sphere->setLocalPosition(Vector3 (0,0,-0.5*DVAR (KERNELSIDE)));
      break;
    case 3 : //top
      sphereGeometry->setColor (Vector3(1,1,0));
      sphere->setLocalPosition(Vector3 (0,0,+0.5*DVAR (KERNELSIDE)));
      break;
    case 4 : //rear
      sphereGeometry->setColor (Vector3(0,1,1));
      sphere->setLocalPosition(Vector3 (0,0.5*DVAR (KERNELSIDE),0));
      break;
    case 5 : //front
      sphereGeometry->setColor (Vector3(1,0,1));
      sphere->setLocalPosition(Vector3 (0,-0.5*DVAR (KERNELSIDE),0));
      break;
  }
  sphereGeometry->setColor (Vector3(0,1,1));
  Surface* sphereSurface = new Surface ();
	dSurfaceParameters* sp = &sphereSurface->getSurfaceParameters ();
  sp->mode  = dContactSlip1 | dContactSlip2;
  sp->mu    = DVAR (SPHEREMU);
	sp->mu2   = DVAR (SPHEREMU2);
	sp->slip1 = 0;
	sp->slip2 = 0;
  sphereCollider=new RigidBodyCollider (sphere);
  sphereCollider->setSurface (sphereSurface);
	sphereGeometry->setCollisionController (sphereCollider);

  bodyGeometry = new CappedCylinderGeometry (DVAR (LEGSIDE),DVAR (LEGRADIUS), "Body");
  leg->addChild(bodyGeometry);
  dMass legMass;
  dMassSetCappedCylinder (&legMass,1,1,DVAR (LEGRADIUS), DVAR (LEGSIDE));
  dMassAdjust (&legMass,DVAR (LEGMASS));
  body=new RigidBody();
  dBodySetMass (body->getBody (), &legMass);
  body->setModel(bodyGeometry);

  switch (aLegID) {
    case 0 : //right
      bodyGeometry->setColor (Vector3(1,0,0));
      body->setLocalPosition(Vector3 (0.5*DVAR (KERNELSIDE)+0.5*DVAR (LEGSIDE),0,0));
	    body->setWorldRotation (Matrix33::rotateAboutY (Math::HALF_PI));
      break;
    case 1 : //left
      bodyGeometry->setColor (Vector3(0,1,0));
      body->setLocalPosition(Vector3 (-0.5*DVAR (KERNELSIDE)-0.5*DVAR (LEGSIDE),0,0));
	    body->setWorldRotation (Matrix33::rotateAboutY (-Math::HALF_PI));
      break;
    case 2 : //bottom
      bodyGeometry->setColor (Vector3(0,0,1));
      body->setLocalPosition(Vector3 (0,0,-0.5*DVAR (KERNELSIDE)-0.5*DVAR (LEGSIDE)));
	    //body->setWorldRotation (Matrix33::rotateAboutX (0));
      break;
    case 3 : //top
      bodyGeometry->setColor (Vector3(1,1,0));
      body->setLocalPosition(Vector3 (0,0,0.5*DVAR (KERNELSIDE)+0.5*DVAR (LEGSIDE)));
	    //body->setWorldRotation (Matrix33::rotateAboutX (0);
      break;
    case 4 : //rear
      bodyGeometry->setColor (Vector3(0,1,1));
      body->setLocalPosition(Vector3 (0,0.5*DVAR (KERNELSIDE)+0.5*DVAR (LEGSIDE),0));
	    body->setWorldRotation (Matrix33::rotateAboutX (Math::HALF_PI));
      break;
    case 5 : //front
      bodyGeometry->setColor (Vector3(1,0,1));
      body->setLocalPosition(Vector3 (0,-0.5*DVAR (KERNELSIDE)-0.5*DVAR (LEGSIDE),0));
	    body->setWorldRotation (Matrix33::rotateAboutX (-Math::HALF_PI));
      break;
  }
  servo[0] =new SimServoMotor();
  servo[0]->setHingeJoint(new HingeJoint (atom->rigidBody, sphere));
  servo[0]->setAMotor(new AMotor (atom->rigidBody, sphere));
  servo[0]->getAMotor()->setNumAxes (1);
  servo[0]->getAMotor()->setMode(dAMotorUser);
  switch (aLegID) {
    case 0 : //right
      servo[0]->getHingeJoint()->setAxis(Vector3 (0,1,0));
      servo[0]->getAMotor()->setAxis(0,Vector3 (0,1,0));
      servo[0]->getHingeJoint()->setAnchor(Vector3 (0.5*DVAR (KERNELSIDE),0,0));
      break;
    case 1 : //left
      servo[0]->getHingeJoint()->setAxis(Vector3 (0,1,0));
      servo[0]->getAMotor()->setAxis(0,Vector3 (0,1,0));
      servo[0]->getHingeJoint()->setAnchor(Vector3 (-0.5*DVAR (KERNELSIDE),0,0));
      break;
    case 2 : //bottom
      servo[0]->getHingeJoint()->setAxis(Vector3 (1,0,0));
      servo[0]->getAMotor()->setAxis(0,Vector3 (1,0,0));
      servo[0]->getHingeJoint()->setAnchor(Vector3 (0,0,-0.5*DVAR (KERNELSIDE)));
      break;
    case 3 : //top
      servo[0]->getHingeJoint()->setAxis(Vector3 (1,0,0));
      servo[0]->getAMotor()->setAxis(0,Vector3 (1,0,0));
      servo[0]->getHingeJoint()->setAnchor(Vector3 (0,0,0.5*DVAR (KERNELSIDE)));
      break;
    case 4 : //rear
      servo[0]->getHingeJoint()->setAxis(Vector3 (1,0,0));
      servo[0]->getAMotor()->setAxis(0,Vector3 (1,0,0));
      servo[0]->getHingeJoint()->setAnchor(Vector3 (0,0.5*DVAR (KERNELSIDE),0));
      break;
    case 5 : //front
      servo[0]->getHingeJoint()->setAxis(Vector3 (1,0,0));
      servo[0]->getAMotor()->setAxis(0,Vector3 (1,0,0));
      servo[0]->getHingeJoint()->setAnchor(Vector3 (0,-0.5*DVAR (KERNELSIDE),0));
      break;
  }

  servo[1] =new SimServoMotor();
  servo[1]->setHingeJoint(new HingeJoint (sphere,body));
  servo[1]->setAMotor(new AMotor (sphere, body));
  servo[1]->getAMotor()->setNumAxes (1);
  servo[1]->getAMotor()->setMode(dAMotorUser);
  switch (aLegID) {
    case 0 : //right
      servo[1]->getHingeJoint()->setAxis(Vector3 (0,0,1));
      servo[1]->getAMotor()->setAxis(0,Vector3 (0,0,1));
      servo[1]->getHingeJoint()->setAnchor(Vector3 (0.5*DVAR (KERNELSIDE),0,0));
      break;
    case 1 : //left
      servo[1]->getHingeJoint()->setAxis(Vector3 (0,0,1));
      servo[1]->getAMotor()->setAxis(0,Vector3 (0,0,1));
      servo[1]->getHingeJoint()->setAnchor(Vector3 (-0.5*DVAR (KERNELSIDE),0,0));
      break;
    case 2 : //bottom
      servo[1]->getHingeJoint()->setAxis(Vector3 (0,1,0));
      servo[1]->getAMotor()->setAxis(0,Vector3 (0,1,0));
      servo[1]->getHingeJoint()->setAnchor(Vector3 (0,0,-0.5*DVAR (KERNELSIDE)));
      break;
    case 3 : //top
      servo[1]->getHingeJoint()->setAxis(Vector3 (0,1,0));
      servo[1]->getAMotor()->setAxis(0,Vector3 (0,1,0));
      servo[1]->getHingeJoint()->setAnchor(Vector3 (0,0,0.5*DVAR (KERNELSIDE)));
      break;
    case 4 : //rear
      servo[1]->getHingeJoint()->setAxis(Vector3 (0,0,1));
      servo[1]->getAMotor()->setAxis(0,Vector3 (0,0,1));
      servo[1]->getHingeJoint()->setAnchor(Vector3 (0,0.5*DVAR (KERNELSIDE),0));
      break;
    case 5 : //front
      servo[1]->getHingeJoint()->setAxis(Vector3 (0,0,1));
      servo[1]->getAMotor()->setAxis(0,Vector3 (0,0,1));
      servo[1]->getHingeJoint()->setAnchor(Vector3 (0,-0.5*DVAR (KERNELSIDE),0));
      break;
  }
  servo[0]->setCoupledServoMotor(servo[1]);
  servo[1]->setCoupledServoMotor(servo[0]);
    
  Surface* bodySurface = new Surface ();
  sp = &bodySurface->getSurfaceParameters ();
	sp->mode  = dContactApprox1 | dContactFDir1 | dContactMu2 | dContactSlip1;// | dContactSlip2;
	sp->mu    = DVAR (LEGMU);
	sp->mu2   = DVAR (LEGMU2);
	sp->slip1 = DVAR (LEGSLIP1);
	sp->slip2 = DVAR (LEGSLIP2);
  bodyCollider=new RigidBodyCollider (body);
  bodyCollider->setSurface (bodySurface);
  bodyGeometry->setCollisionController (bodyCollider);
*/
//  Timer::getInstance ()->addEvent (this, new FrameEvent (), 0);
  Node * atomN=(Node *) atom->getModel();
  atomN->addChild(leg);

}

SimLeg::~SimLeg(){
//  delete bodyCollider;
//  delete body;
//  delete bodyGeometry;
//  delete sphereCollider;
//  delete sphereGeometry;
//  delete sphere;
//  delete rigidBody;
//  delete model;
#ifdef LEGLINK  
  if (this->linkedLeg)
  {
    delete this->linkedLeg;    
  }
#endif

  Timer::getInstance ()->clearEvents(this);
  /*
  printf("Avant Destruction\n");
  body->getGraph()->print();
  sphere->getGraph()->print(); 
  printf("Servo0\n");
  */
  if (servo[0])
    delete servo[0];
  SimRigidBody * atomtmp;
  /*
  atomtmp=dynamic_cast<SimRigidBody *> (simatom->rigidBody);
  atomtmp->getGraph()->print();

  body->getGraph()->print();
  sphere->getGraph()->print();
  */
  servo[0]=0;
  if (servo[1])
    delete servo[1];  
  servo[1]=0;
  /*
  atomtmp=dynamic_cast<SimRigidBody *> (simatom->rigidBody);
  atomtmp->getGraph()->print();
  body->getGraph()->print();
  sphere->getGraph()->print();
  */
}

bool SimLeg::isLinked()
{
#ifdef LEGLINK
  if (this->linkedLeg)
    return true;
  return false;
#endif
#ifndef LEGLINK
  if (this->slink)
    return true;
  return false;
#endif
}

void SimLeg::link(SimLeg * aLeg)
{
  if ((!isLinked()) && (!aLeg->isLinked()))
  {

  #ifdef LEGLINK
    this->linkedLeg=new SimLinkedLeg(this,aLeg);    
  #endif
  #ifndef LEGLINK

  #endif

  }
}

void SimLeg::unlink()
{
  if (isLinked())
  {

  #ifdef LEGLINK
    delete linkedLeg;
  #endif
  #ifndef LEGLINK

  #endif

  }
}
unsigned int SimLeg::getSimServoMotorCount() const
{
  return 2;
}

SimServoMotor * SimLeg::getSimServoMotor(unsigned int aSimServoMotorID) const
{
  if (aSimServoMotorID<getSimServoMotorCount())
    return servo[aSimServoMotorID];
  else
    return 0;
}

Vector3 SimLeg::getVector(int Geometry) const
{
  Real x=0;
  Real y=0;
  Real z=0;
  if (Geometry==0) {
    switch (this->ID) {
      case 0 : //right
        x=1;
        break;
      case 1 : //left
        x=-1;
      break;
      case 2 : //bottom
        z=-1;
        break;
      case 3 : //top
        z=1;
        break;
      case 4 : //rear
        y=1;
        break;
      case 5 : //front
        y=-1;
        break;
    }
  }  
  return Vector3(x,y,z);  
}

Real SimLeg::getRotationAngle(int Geometry) const
{
  Real angle=0;
  if (Geometry==0) {
    switch (this->ID) {
      case 0 : //right,devant,rouge
        angle=M_PI_2;
        break;
      case 1 : //left,derriere, vert
        angle=-M_PI_2;
        break;
      case 2 : //bottom,bas, bleu fonce
        angle=0;
        break;
      case 3 : //top,haut, jaune
        angle=0;
        break;
      case 4 : //rear,droite, bleu clair
        angle=M_PI_2;
        break;
      case 5 : //front,gauche, violet
        angle=-M_PI_2;
        break;
    }
  }
  if (Geometry==BOX) {
    switch (this->ID) {
      case 0 : //right,devant,rouge
        angle=0;
        break;
      case 1 : //left,derriere, vert
        angle=0;
        break;
      case 2 : //bottom,bas, bleu fonce
        angle=M_PI_2;
        break;
      case 3 : //top,haut, jaune
        angle=-M_PI_2;
        break;
      case 4 : //rear,droite, bleu clair
        angle=M_PI_2;
        break;
      case 5 : //front,gauche, violet
        angle=-M_PI_2;
        break;
    }
  }
  if (Geometry==FLATCYLINDER) {
    switch (this->ID) {
      case 0 : //right,devant,rouge
        angle=M_PI_2;
        break;
      case 1 : //left,derriere, vert
        angle=-M_PI_2;
        break;
      case 2 : //bottom,bas, bleu fonce
        angle=M_PI_2;
        break;
      case 3 : //top,haut, jaune
        angle=-M_PI_2;
        break;
      case 4 : //rear,droite, bleu clair
        angle=0;
        break;
      case 5 : //front,gauche, violet
        angle=0;
        break;
    }
  }

  return angle;
}

Vector3 SimLeg::getColor() const
{
  switch (this->ID) {
    case 0 : //right
        return Vector3(1,0,0);
      break;
    case 1 : //left
        return Vector3(0,1,0);
      break;
    case 2 : //bottom
        return Vector3(0,0,1);
      break;
    case 3 : //top
        return Vector3(1,1,0);
      break;
    case 4 : //rear
        return Vector3(0,1,1);
      break;
    case 5 : //front
        return Vector3(1,0,1);
      break;
  }
}
  
Vector3 SimLeg::getAxis(unsigned int index,int Geometry) const
{
  Real x=0;
  Real y=0;
  Real z=0;
  if (Geometry==0) {
    switch (index) {
      case 0 :
        switch (this->ID) {
          case 0 : //right
            y=1;
            break;
          case 1 : //left
            y=1;
            break;
          case 2 : //bottom
            x=1;
            break;
          case 3 : //top
            x=1;
            break;
          case 4 : //rear
            x=1;
            break;
          case 5 : //front
            x=1;
            break;
          }
        break;
      case 1 :
        switch (this->ID) {
          case 0 : //right
            z=1;
            break;
          case 1 : //left
            z=1;
            break;
          case 2 : //bottom
            y=1;
            break;
          case 3 : //top
            y=1;
            break;
          case 4 : //rear
            z=1;
            break;
          case 5 : //front
            z=1;
            break;
          }
        break;
      }
    }  
  if (Geometry==BOX) {
    switch (index) {
      case 0 :
        switch (this->ID) {
          case 0 : //right
            y=1;
            break;
          case 1 : //left
            y=1;
            break;
          case 2 : //bottom
            y=1;
            break;
          case 3 : //top
            y=1;
            break;
          case 4 : //rear
            z=1;
            break;
          case 5 : //front
            z=1;
            break;
          }
        break;
      case 1 :
        switch (this->ID) {
          case 0 : //right
            z=1;
            break;
          case 1 : //left
            z=1;
            break;
          case 2 : //bottom
            y=1;
            break;
          case 3 : //top
            y=1;
            break;
          case 4 : //rear
            z=1;
            break;
          case 5 : //front
            z=1;
            break;
          }
        break;
      }
    }
  if (Geometry==FLATCYLINDER) {
    switch (index) {
      case 0 :
        switch (this->ID) {
          case 0 : //right
            z=1;
            break;
          case 1 : //left
            z=1;
            break;
          case 2 : //bottom
            x=1;
            break;
          case 3 : //top
            x=1;
            break;
          case 4 : //rear
            z=1;
            break;
          case 5 : //front
            z=1;
            break;
          }
        break;
      case 1 :
        switch (this->ID) {
          case 0 : //right
            z=1;
            break;
          case 1 : //left
            z=1;
            break;
          case 2 : //bottom
            y=1;
            break;
          case 3 : //top
            y=1;
            break;
          case 4 : //rear
            z=1;
            break;
          case 5 : //front
            z=1;
            break;
          }
        break;
      }
    }
  
  return Vector3(x,y,z);
}

Vector3 SimLeg::getAbsPosSphere() const
{
  return sphere->getWorldPosition();
}

Vector3 SimLeg::getAbsPosBody() const
{        
  return body->getWorldPosition();
}

Vector3 SimLeg::getAbsPosEnd() const
{
  Real x=0;
  Real y=0;
  Real z=0;
  switch (this->ID) {
    case 0 : //right
      z=+DVAR(LEGSIDE)/2;
      break;
    case 1 : //left
      z=+DVAR(LEGSIDE)/2;
    break;
    case 2 : //bottom
      z=-DVAR(LEGSIDE)/2;
      break;
    case 3 : //top
      z=+DVAR(LEGSIDE)/2;
      break;
    case 4 : //rear
      z=-DVAR(LEGSIDE)/2;
      break;
    case 5 : //front
      z=-DVAR(LEGSIDE)/2;
      break;
  }

  dVector3 result;
  dBodyGetRelPointPos (body->getBody(),x,y,z, result);
/*  printf ("\nEnd (GFC) ODE (id=%i) %f %f %f\n",
          ID,
          result[0],
          result[1],
          result[2]
        );
*/
  return Vector3(result[0], result[1], result[2]);
}

Vector3 SimLeg::getAbsPosConnexionDevice(unsigned int deviceIndex) const
{
  Real x=0;
  Real y=0;
  Real z=0;
  switch (this->ID) {
    case 0 : //right
      z=+DVAR(LEGSIDE)/2;
      break;
    case 1 : //left
      z=+DVAR(LEGSIDE)/2;
    break;
    case 2 : //bottom
      z=-DVAR(LEGSIDE)/2;
      break;
    case 3 : //top
      z=+DVAR(LEGSIDE)/2;
      break;
    case 4 : //rear
      z=-DVAR(LEGSIDE)/2;
      break;
    case 5 : //front
      z=-DVAR(LEGSIDE)/2;
      break;
  }  
  switch (deviceIndex) {
    case 0 : //right
      y=+DVAR(LEGRADIUS)/2;
      break;
    case 1 : //left
      x=+DVAR(LEGRADIUS)/2;
    break;
    case 2 : //bottom
      y=-DVAR(LEGRADIUS)/2;
      break;
    case 3 : //top
      x=-DVAR(LEGRADIUS)/2;
      break;
  }
  dVector3 result;
  dBodyGetRelPointPos (body->getBody(),x,y,z, result);
  /*
  printf ("\nDevice(%i)  (GFC) ODE (id=%i) %f %f %f\n",
          deviceIndex,
          ID,
          result[0],
          result[1],
          result[2]
        );
  */
  return Vector3(result[0], result[1], result[2]);  
}
  
bool SimLeg::isConnectable(SimLeg * aSimLeg) const
{
  if (!Math::isEqual(
      dDISTANCE (sphere->getWorldPosition(),aSimLeg->sphere->getWorldPosition()),
      DVAR (LEGSIDE)*2))
      {
        printf("Distance pas OK\n");
        return false;
      }

  if (!Vector3::isEqual(
      this->getAbsPosEnd(), aSimLeg->getAbsPosEnd()))
      {
        printf("Coordonnees pas OK\n");
        return false;
      }

  for (int i=0; i<4;i++)
  {
    bool found=false;
    for (int j=0; j<4 && found==false ;j++)
      if (Vector3::isEqual(this->getAbsPosConnexionDevice(i), this->getAbsPosConnexionDevice(i)))
        found=true;
    if (found==false)
      return false;    
  }  
  return true;
}  

void SimLeg::handleEvent (Event* event)
{
	if (!event)
	{
		return;
	}

	////


	////

	RigidBodyEntity::handleEvent (event);
}

void SimLeg::print()
{
  printf("SimLeg: Atom ID=%i Leg ID=%i\n",simatom->ID,ID);
}
