/***************************************************************************
                          SimLinkedLeg.cpp  -  description
                             -------------------
    begin                : lun jui 21 2003
    copyright            : (C) 2003 by Michel DUBOIS, Yann Le Guyadec 
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ConfigReader.h"
#include "mmgr.h"

#include "SimLinkedLeg.h"
#include "Surface.h"
#include "Timer.h"

#ifdef LEGLINK


CR_DECLARE (KERNELSIDE);
CR_DECLARE (SPHERESIDE);
CR_DECLARE (LEGRADIUS);
CR_DECLARE (LEGSIDE);
CR_DECLARE (LEGMASS);
CR_DECLARE (LEGMU);
CR_DECLARE (LEGMU2);
CR_DECLARE (LEGSLIP1);
CR_DECLARE (LEGSLIP2);


SimLinkedLeg::SimLinkedLeg(SimLeg * aSimLeg, SimLeg * anotherSimLeg)
{
  static bool isConfiged = false;

	if (!isConfiged)
	{
		isConfiged = true;
		CR_SETUP (Atom, KERNELSIDE, CR_DOUBLE);
    CR_SETUP (Atom, SPHERESIDE, CR_DOUBLE);
    CR_SETUP (Atom, LEGRADIUS, CR_DOUBLE);
    CR_SETUP (Atom, LEGSIDE, CR_DOUBLE);
    CR_SETUP (Atom, LEGMASS, CR_DOUBLE);
    CR_SETUP (Atom, LEGMU, CR_DOUBLE);
    CR_SETUP (Atom, LEGMU2, CR_DOUBLE);
    CR_SETUP (Atom, LEGSLIP1, CR_DOUBLE);
    CR_SETUP (Atom, LEGSLIP2, CR_DOUBLE);
 
  }
  this->sleg1=aSimLeg;
  this->sleg2=anotherSimLeg;
  this->linkBody=0;
  this->linkCollider=0;
  this->linkGeometry=0;
  if (isConnectable())
    link();  
}

bool SimLinkedLeg::isConnectable()
{
  return true;  
}

void SimLinkedLeg::link()
{
  sleg1->linkedLeg=this;
  sleg2->linkedLeg=this;  
  linkGeometry = new CylinderGeometry (DVAR (LEGSIDE)*2,DVAR (LEGRADIUS), "Link");
  linkGeometry->setColor (Vector3(1,1,1));
  dMass legMass;
  dMassSetCylinder (&legMass,1,1,DVAR (LEGRADIUS), DVAR (LEGSIDE)*2);
  dMassAdjust (&legMass,DVAR (LEGMASS)*2);
  linkBody=new SimRigidBody();
  dBodySetMass (linkBody->getBody (), &legMass);
  linkBody->setModel(linkGeometry);
  linkBody->setLocalPosition(sleg1->getAbsPosEnd());  
  linkBody->setWorldRotation(sleg1->body->getWorldRotation());
  Surface* linkSurface = new Surface ();
  dSurfaceParameters * sp = &linkSurface->getSurfaceParameters ();
	sp->mode  = dContactApprox1 | dContactFDir1 | dContactMu2 | dContactSlip1;// | dContactSlip2;
	sp->mu    = DVAR (LEGMU);
	sp->mu2   = DVAR (LEGMU2);
	sp->slip1 = DVAR (LEGSLIP1);
	sp->slip2 = DVAR (LEGSLIP2);
  linkCollider=new RigidBodyCollider (linkBody);
  linkCollider->setSurface (linkSurface);
  linkGeometry->setCollisionController (linkCollider);
  
  this->sleg1->linkedLeg=this;
  this->sleg2->linkedLeg=this;
  Node * nodeLeg1=(Node *) sleg1->getModel();
  nodeLeg1->removeChild(sleg1->bodyGeometry);

  delete sleg1->servo[1];
  delete sleg1->bodyGeometry;
  sleg1->body=0;
  sleg1->bodyCollider=0;
  sleg1->bodyGeometry=0;

  Node * nodeLeg2=(Node *) sleg2->getModel();
  nodeLeg2->removeChild(sleg2->bodyGeometry);

  delete sleg2->servo[1];
  delete sleg2->bodyGeometry;
  sleg2->body=0;
  sleg2->bodyCollider=0;
  sleg2->bodyGeometry=0;
  
  sleg1->servo[1]=new SimServoMotor(sleg1->sphere,linkBody);
  sleg1->servo[1]->sleg=sleg1;
  sleg1->servo[1]->ID=1;
  sleg1->servo[1]->getHingeJoint()->setAnchor(sleg1->sphere->getWorldPosition());
  sleg1->servo[1]->setAxis(sleg1->getAxis(1));
  sleg1->servo[0]->setCoupledServoMotor(sleg1->servo[1]);
  sleg1->servo[1]->setCoupledServoMotor(sleg1->servo[0]);
  sleg2->servo[1] =new SimServoMotor(sleg2->sphere,linkBody);
  sleg2->servo[1]->sleg=sleg2;
  sleg2->servo[1]->ID=1;
  sleg2->servo[1]->getHingeJoint()->setAnchor(sleg2->sphere->getWorldPosition());
  sleg2->servo[1]->setAxis(sleg2->getAxis(1));
  sleg2->servo[0]->setCoupledServoMotor(sleg2->servo[1]);
  sleg2->servo[1]->setCoupledServoMotor(sleg2->servo[0]);

  nodeLeg1->addChild(linkGeometry);
  nodeLeg2->addChild(linkGeometry);
}

void SimLinkedLeg::unlink()
{
  if (!(this->linkBody))
    return;
  if (!(this->linkCollider))
    return;
  if (!(this->linkGeometry))
    return;
  Node * nodeLeg1=(Node *) sleg1->getModel();
  nodeLeg1->removeChild(linkGeometry);
  Node * nodeLeg2=(Node *) sleg2->getModel();
  nodeLeg2->removeChild(linkGeometry);
  
  if (sleg1->servo[1])
    delete sleg1->servo[1];
  if (sleg2->servo[1])
    delete sleg2->servo[1];
  delete linkGeometry;
  linkGeometry=0;
  linkCollider=0;
  linkBody=0; 

  sleg1->bodyGeometry = new CylinderGeometry (DVAR (LEGSIDE),DVAR (LEGRADIUS), "Body");
  dMass legMass;
  dMassSetCylinder (&legMass,1,1,DVAR (LEGRADIUS), DVAR (LEGSIDE));
  dMassAdjust (&legMass,DVAR (LEGMASS));
  sleg1->body=new SimRigidBody();
  dBodySetMass (sleg1->body->getBody (), &legMass);
  sleg1->body->setModel(sleg1->bodyGeometry);
  sleg1->bodyGeometry->setColor (sleg1->getColor());
  Vector3 positionM=(sleg1->sphere->getWorldPosition()
                    +sleg2->sphere->getWorldPosition())/2;
  Vector3 position1=(sleg1->sphere->getWorldPosition()
                    +positionM)/2;
  Vector3 position2=(sleg2->sphere->getWorldPosition()
                    +positionM)/2;
  
  
  sleg1->body->setWorldPosition(position1);
  
  //sleg1->body->setLocalPosition(sleg1->getVector()*(0.5*DVAR (KERNELSIDE)+0.5*DVAR (LEGSIDE)));
  sleg1->body->setWorldRotation(Matrix33::rotate (sleg1->getRotationAngle(),sleg1->getAxis(0)));
//  sleg1->body->setWorldRotation(linkBody->getWorldRotation());
  Surface * bodySurface = new Surface ();
  dSurfaceParameters* sp = &bodySurface->getSurfaceParameters ();
	sp->mode  = dContactApprox1 | dContactFDir1 | dContactMu2 | dContactSlip1;// | dContactSlip2;
	sp->mu    = DVAR (LEGMU);
	sp->mu2   = DVAR (LEGMU2);
	sp->slip1 = DVAR (LEGSLIP1);
	sp->slip2 = DVAR (LEGSLIP2);
  sleg1->bodyCollider=new RigidBodyCollider (sleg1->body);
  sleg1->bodyCollider->setSurface (bodySurface);
  sleg1->bodyGeometry->setCollisionController (sleg1->bodyCollider);
  nodeLeg1->addChild(sleg1->bodyGeometry);

  sleg1->servo[1] =new SimServoMotor(sleg1->sphere,sleg1->body);
  sleg1->servo[1]->sleg=sleg1;
  sleg1->servo[1]->ID=1;
  sleg1->servo[1]->setAxis(sleg1->getAxis(1));
  sleg1->servo[1]->getHingeJoint()->setAnchor(sleg1->getAbsPosSphere());
  sleg1->servo[0]->setCoupledServoMotor(sleg1->servo[1]);
  sleg1->servo[1]->setCoupledServoMotor(sleg1->servo[0]);
  this->sleg1->linkedLeg=0;
  
  sleg2->bodyGeometry = new CylinderGeometry (DVAR (LEGSIDE),DVAR (LEGRADIUS), "Body");
  sleg2->body=new SimRigidBody();
  dBodySetMass (sleg2->body->getBody (), &legMass);
  sleg2->body->setModel(sleg2->bodyGeometry);
  sleg2->bodyGeometry->setColor (sleg2->getColor());
  sleg2->body->setWorldPosition(position2);
  
//  sleg2->body->setLocalPosition(sleg2->getVector()*(0.5*DVAR (KERNELSIDE)+0.5*DVAR (LEGSIDE)));
  sleg2->body->setWorldRotation(Matrix33::rotate (sleg2->getRotationAngle(),sleg2->getAxis(0)));
//  sleg2->body->setWorldRotation(linkBody->getWorldRotation());
  Surface * bodySurface2 = new Surface ();
  sp = &bodySurface2->getSurfaceParameters ();
	sp->mode  = dContactApprox1 | dContactFDir1 | dContactMu2 | dContactSlip1;// | dContactSlip2;
	sp->mu    = DVAR (LEGMU);
	sp->mu2   = DVAR (LEGMU2);
	sp->slip1 = DVAR (LEGSLIP1);
	sp->slip2 = DVAR (LEGSLIP2);
  sleg2->bodyCollider=new RigidBodyCollider (sleg2->body);
  sleg2->bodyCollider->setSurface (bodySurface2);
  sleg2->bodyGeometry->setCollisionController (sleg2->bodyCollider);
  nodeLeg2->addChild(sleg2->bodyGeometry);

  sleg2->servo[1] =new SimServoMotor(sleg2->sphere,sleg2->body);
  sleg2->servo[1]->sleg=sleg2;
  sleg2->servo[1]->ID=1;
  sleg2->servo[1]->setAxis(sleg2->getAxis(1));
  sleg2->servo[1]->getHingeJoint()->setAnchor(sleg2->getAbsPosSphere());
  sleg2->servo[0]->setCoupledServoMotor(sleg2->servo[1]);
  sleg2->servo[1]->setCoupledServoMotor(sleg2->servo[0]);
  this->sleg2->linkedLeg=0;
  
}

SimLinkedLeg::~SimLinkedLeg()
{
 unlink();
  sleg1=0;
  sleg2=0;
}

Vector3 SimLinkedLeg::getAbsPosEnd() const
{
  return Vector3 (
    (sleg1->sphere->getWorldPosition().x+sleg2->sphere->getWorldPosition().x)/2,
    (sleg1->sphere->getWorldPosition().y+sleg2->sphere->getWorldPosition().y)/2,
    (sleg1->sphere->getWorldPosition().z+sleg2->sphere->getWorldPosition().z)/2
  );
}

Vector3 SimLinkedLeg::getAbsPosBody1() const
{
  return Vector3 (
    (sleg1->sphere->getWorldPosition().x+getAbsPosEnd().x)/2,
    (sleg1->sphere->getWorldPosition().y+getAbsPosEnd().y)/2,
    (sleg1->sphere->getWorldPosition().z+getAbsPosEnd().z)/2
  );
}

Vector3 SimLinkedLeg::getAbsPosBody2() const
{
  return Vector3 (
    (sleg2->sphere->getWorldPosition().x+getAbsPosEnd().x)/2,
    (sleg2->sphere->getWorldPosition().y+getAbsPosEnd().y)/2,
    (sleg2->sphere->getWorldPosition().z+getAbsPosEnd().z)/2
  );
}
#endif
